@@ -38,6 +38,7 @@
 #include "mouse/CMouseNormal.h"
 #include "mouse/CMousePrint.h"
 #include "mouse/CMouseRangeTrk.h"
+#include "mouse/CMouseSelect.h"
 #include "mouse/CMouseWptBubble.h"
 #include "plot/CPlotProfile.h"
 #include "units/IUnit.h"
@@ -53,16 +54,14 @@ CCanvas::CCanvas(QWidget *parent, const QString &name)
 
     if(name.isEmpty())
     {
-        int count = 1;
-        while(1)
+        for(int count = 1;; ++count)
         {
             QString name = tr("View %1").arg(count);
-            if(CMainWindow::self().findChild<CCanvas*>(name) == 0)
+            if(nullptr == CMainWindow::self().findChild<CCanvas*>(name))
             {
                 setObjectName(name);
                 break;
             }
-            count++;
         }
     }
     else
@@ -78,23 +77,21 @@ CCanvas::CCanvas(QWidget *parent, const QString &name)
     gis     = new CGisDraw(this);
     mouse   = new CMouseNormal(gis, this);
 
-    connect(map, SIGNAL(sigCanvasUpdate(CCanvas::redraw_e)), this, SLOT(slotTriggerCompleteUpdate(CCanvas::redraw_e)));
-    connect(dem, SIGNAL(sigCanvasUpdate(CCanvas::redraw_e)), this, SLOT(slotTriggerCompleteUpdate(CCanvas::redraw_e)));
-    connect(gis, SIGNAL(sigCanvasUpdate(CCanvas::redraw_e)), this, SLOT(slotTriggerCompleteUpdate(CCanvas::redraw_e)));
+    connect(map, &CMapDraw::sigCanvasUpdate, this, &CCanvas::slotTriggerCompleteUpdate);
+    connect(dem, &CDemDraw::sigCanvasUpdate, this, &CCanvas::slotTriggerCompleteUpdate);
+    connect(gis, &CGisDraw::sigCanvasUpdate, this, &CCanvas::slotTriggerCompleteUpdate);
 
     timerToolTip = new QTimer(this);
     timerToolTip->setSingleShot(true);
-    connect(timerToolTip, SIGNAL(timeout()), this, SLOT(slotToolTip()));
+    connect(timerToolTip, &QTimer::timeout, this, &CCanvas::slotToolTip);
 
-    loadIndicator1 = new QMovie(this);
-    loadIndicator1->setFileName("://animation/loader.gif");
+    loadIndicator1 = new QMovie("://animation/loader.gif", QByteArray(), this);
     mapLoadIndicator = new QLabel(this);
     mapLoadIndicator->setMovie(loadIndicator1);
     loadIndicator1->start();
     mapLoadIndicator->show();
 
-    loadIndicator2 = new QMovie(this);
-    loadIndicator2->setFileName("://animation/loader2.gif");
+    loadIndicator2 = new QMovie("://animation/loader2.gif", QByteArray(), this);
     demLoadIndicator = new QLabel(this);
     demLoadIndicator->setMovie(loadIndicator2);
     loadIndicator2->start();
@@ -106,17 +103,17 @@ CCanvas::CCanvas(QWidget *parent, const QString &name)
     labelStatusMessages->setAlignment(Qt::AlignJustify);
     labelStatusMessages->hide();
 
-    connect(map, SIGNAL(sigStartThread()), mapLoadIndicator, SLOT(show()));
-    connect(map, SIGNAL(sigStopThread()), mapLoadIndicator, SLOT(hide()));
+    connect(map, &CMapDraw::sigStartThread, mapLoadIndicator, &QLabel::show);
+    connect(map, &CMapDraw::sigStopThread,  mapLoadIndicator, &QLabel::hide);
 
-    connect(dem, SIGNAL(sigStartThread()), demLoadIndicator, SLOT(show()));
-    connect(dem, SIGNAL(sigStopThread()), demLoadIndicator, SLOT(hide()));
+    connect(dem, &CDemDraw::sigStartThread, demLoadIndicator, &QLabel::show);
+    connect(dem, &CDemDraw::sigStopThread,  demLoadIndicator, &QLabel::hide);
 
     timerTrackOnFocus = new QTimer(this);
     timerTrackOnFocus->setSingleShot(false);
     timerTrackOnFocus->start(1000);
 
-    connect(timerTrackOnFocus, SIGNAL(timeout()), this, SLOT(slotCheckTrackOnFocus()));
+    connect(timerTrackOnFocus, &QTimer::timeout, this, &CCanvas::slotCheckTrackOnFocus);
 }
 
 CCanvas::~CCanvas()
@@ -130,7 +127,7 @@ CCanvas::~CCanvas()
     saveSizeTrackProfile();
 }
 
-void CCanvas::setOverrideCursor(const QCursor& cursor, const QString& src)
+void CCanvas::setOverrideCursor(const QCursor &cursor, const QString&)
 {
 //    qDebug() << "setOverrideCursor" << src;
     QApplication::setOverrideCursor(cursor);
@@ -153,9 +150,10 @@ void CCanvas::saveConfig(QSettings& cfg)
     map->saveConfig(cfg);
     dem->saveConfig(cfg);
     grid->saveConfig(cfg);
-    cfg.setValue("posFocus", posFocus);
-    cfg.setValue("proj", map->getProjection());
-    cfg.setValue("scales", map->getScalesType());
+    cfg.setValue("posFocus",  posFocus);
+    cfg.setValue("proj",      map->getProjection());
+    cfg.setValue("scales",    map->getScalesType());
+    cfg.setValue("backColor", backColor.name());
 }
 
 void CCanvas::loadConfig(QSettings& cfg)
@@ -164,6 +162,9 @@ void CCanvas::loadConfig(QSettings& cfg)
     setProjection(cfg.value("proj", map->getProjection()).toString());
     setScales((CCanvas::scales_type_e)cfg.value("scales",  map->getScalesType()).toInt());
 
+    const QString &backColorStr = cfg.value("backColor", "#FFFFBF").toString();
+    backColor = QColor(backColorStr);
+
     map->loadConfig(cfg);
     dem->loadConfig(cfg);
     grid->loadConfig(cfg);
@@ -186,114 +187,90 @@ void CCanvas::resetMouse()
     }
 }
 
+void CCanvas::setMouseCursor(IMouse& mouse, const QString& src)
+{
+    if(underMouse())
+    {
+        CCanvas::restoreOverrideCursor(src);
+        CCanvas::setOverrideCursor(mouse, src);
+    }
+}
+
 void CCanvas::setMouseMoveWpt(CGisItemWpt& wpt)
 {
     mouse->deleteLater();
     mouse = new CMouseMoveWpt(wpt, gis, this);
-    if(underMouse())
-    {
-        CCanvas::restoreOverrideCursor("setMouseMoveWpt");
-        CCanvas::setOverrideCursor(*mouse, "setMouseMoveWpt");
-    }
+    setMouseCursor(*mouse, "setMouseMoveWpt");
 }
 
 void CCanvas::setMouseEditTrk(const QPointF &pt)
 {
     mouse->deleteLater();
     mouse = new CMouseEditTrk(pt, gis, this);
-    if(underMouse())
-    {
-        CCanvas::restoreOverrideCursor("setMouseEditTrk");
-        CCanvas::setOverrideCursor(*mouse, "setMouseEditTrk");
-    }
+    setMouseCursor(*mouse, "setMouseEditTrk");
 }
 
 void CCanvas::setMouseEditRte(const QPointF &pt)
 {
     mouse->deleteLater();
     mouse = new CMouseEditRte(pt, gis, this);
-    if(underMouse())
-    {
-        CCanvas::restoreOverrideCursor("setMouseEditRte");
-        CCanvas::setOverrideCursor(*mouse, "setMouseEditRte");
-    }
+    setMouseCursor(*mouse, "setMouseEditRte");
 }
 
 void CCanvas::setMouseEditTrk(CGisItemTrk& trk)
 {
     mouse->deleteLater();
     mouse = new CMouseEditTrk(trk, gis, this);
-    if(underMouse())
-    {
-        CCanvas::restoreOverrideCursor("setMouseEditTrk");
-        CCanvas::setOverrideCursor(*mouse, "setMouseEditTrk");
-    }
+    setMouseCursor(*mouse, "setMouseEditTrk");
 }
 
 void CCanvas::setMouseRangeTrk(CGisItemTrk& trk)
 {
     mouse->deleteLater();
     mouse = new CMouseRangeTrk(trk, gis, this);
-    if(underMouse())
-    {
-        CCanvas::restoreOverrideCursor("setMouseRangeTrk");
-        CCanvas::setOverrideCursor(*mouse, "setMouseRangeTrk");
-    }
+    setMouseCursor(*mouse, "setMouseRangeTrk");
 }
 
 void CCanvas::setMouseEditArea(const QPointF& pt)
 {
     mouse->deleteLater();
     mouse = new CMouseEditArea(pt, gis, this);
-    if(underMouse())
-    {
-        CCanvas::restoreOverrideCursor("setMouseEditArea");
-        CCanvas::setOverrideCursor(*mouse, "setMouseEditArea");
-    }
+    setMouseCursor(*mouse, "setMouseEditArea");
 }
 
 void CCanvas::setMouseEditArea(CGisItemOvlArea& area)
 {
     mouse->deleteLater();
     mouse = new CMouseEditArea(area, gis, this);
-    if(underMouse())
-    {
-        CCanvas::restoreOverrideCursor("setMouseEditArea");
-        CCanvas::setOverrideCursor(*mouse, "setMouseEditArea");
-    }
+    setMouseCursor(*mouse, "setMouseEditArea");
 }
 
 void CCanvas::setMouseEditRte(CGisItemRte& rte)
 {
     mouse->deleteLater();
     mouse = new CMouseEditRte(rte, gis, this);
-    if(underMouse())
-    {
-        CCanvas::restoreOverrideCursor("setMouseEditRte");
-        CCanvas::setOverrideCursor(*mouse, "setMouseEditRte");
-    }
+    setMouseCursor(*mouse, "setMouseEditRte");
 }
 
 void CCanvas::setMouseWptBubble(const IGisItem::key_t& key)
 {
     mouse->deleteLater();
     mouse = new CMouseWptBubble(key, gis, this);
-    if(underMouse())
-    {
-        CCanvas::restoreOverrideCursor("setMouseWptBubble");
-        CCanvas::setOverrideCursor(*mouse, "setMouseWptBubble");
-    }
+    setMouseCursor(*mouse, "setMouseWptBubble");
 }
 
 void CCanvas::setMousePrint()
 {
     mouse->deleteLater();
     mouse = new CMousePrint(gis, this);
-    if(underMouse())
-    {
-        CCanvas::restoreOverrideCursor("setMousePrint");
-        CCanvas::setOverrideCursor(*mouse, "setMousePrint");
-    }
+    setMouseCursor(*mouse, "setMousePrint");
+}
+
+void CCanvas::setMouseSelect()
+{
+    mouse->deleteLater();
+    mouse = new CMouseSelect(gis, this);
+    setMouseCursor(*mouse, "setMouseSelect");
 }
 
 void CCanvas::reportStatus(const QString& key, const QString& msg)
@@ -310,7 +287,7 @@ void CCanvas::reportStatus(const QString& key, const QString& msg)
     QString report;
     QStringList keys = statusMessages.keys();
     keys.sort();
-    foreach(const QString &key, keys)
+    for(const QString &key : keys)
     {
         report += statusMessages[key] + "\n";
     }
@@ -347,10 +324,8 @@ void CCanvas::resizeEvent(QResizeEvent * e)
     setSizeTrackProfile();
 }
 
-void CCanvas::paintEvent(QPaintEvent * e)
+void CCanvas::paintEvent(QPaintEvent*)
 {
-    Q_UNUSED(e);
-
     if(!isVisible())
     {
         return;
@@ -361,7 +336,7 @@ void CCanvas::paintEvent(QPaintEvent * e)
     USE_ANTI_ALIASING(p,true);
 
     // fill the background with default pattern
-    p.fillRect(rect(), "#FFFFBF");
+    p.fillRect(rect(), backColor);
 
     // ----- start to draw thread based content -----
     // move coordinate system to center of the screen
@@ -392,17 +367,23 @@ void CCanvas::paintEvent(QPaintEvent * e)
 
 void CCanvas::mousePressEvent(QMouseEvent * e)
 {
+    if(!mousePressMutex.tryLock())
+    {
+        return;
+    }
+
     mouse->mousePressEvent(e);
     QWidget::mousePressEvent(e);
     e->accept();
+
+    mousePressMutex.unlock();
 }
 
 void CCanvas::mouseMoveEvent(QMouseEvent * e)
 {
-    qreal ele = NOFLOAT;
     QPointF pos = e->pos();
     map->convertPx2Rad(pos);
-    ele = dem->getElevationAt(pos);
+    qreal ele = dem->getElevationAt(pos);
     emit sigMousePosition(pos * RAD_TO_DEG, ele);
 
     mouse->mouseMoveEvent(e);
@@ -451,9 +432,8 @@ void CCanvas::enterEvent(QEvent * e)
 }
 
 
-void CCanvas::leaveEvent(QEvent * e)
+void CCanvas::leaveEvent(QEvent *)
 {
-    Q_UNUSED(e);
     // bad hack to stop bad number of override cursors.
     while(QApplication::overrideCursor())
     {
@@ -480,25 +460,25 @@ void CCanvas::keyPressEvent(QKeyEvent * e)
 
     /* move the map with keys up, down, left and right */
     case Qt::Key_Up:
-        moveMap(QPointF(         0,  height()/4));
+        moveMap(QPointF(0,  height()/4));
         break;
 
     case Qt::Key_Down:
-        moveMap(QPointF(         0, -height()/4));
+        moveMap(QPointF(0, -height()/4));
         break;
 
     case Qt::Key_Left:
-        moveMap(QPointF( width()/4,           0));
+        moveMap(QPointF( width()/4, 0));
         break;
 
     case Qt::Key_Right:
-        moveMap(QPointF(-width()/4,           0));
+        moveMap(QPointF(-width()/4, 0));
         break;
 
     case Qt::Key_Escape:
     {
         IMouseEditLine *lineMouse = dynamic_cast<IMouseEditLine*>(mouse);
-        if(lineMouse != 0)
+        if(nullptr != lineMouse)
         {
             lineMouse->abortStep();
         }
@@ -636,13 +616,13 @@ void CCanvas::slotCheckTrackOnFocus()
 
         // get access to next track object
         CGisItemTrk * trk2 = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
-        if(trk2 == 0)
+        if(nullptr == trk2)
         {
             return;
         }
 
         // create new profile plot, the plot will register itself at the track
-        plotTrackProfile = new CPlotProfile(trk2, CMainWindow::self().profileIsWindow() ? IPlot::eModeWindow : IPlot::eModeIcon, this);
+        plotTrackProfile = new CPlotProfile(trk2, trk2->limitsGraph1, CMainWindow::self().profileIsWindow() ? IPlot::eModeWindow : IPlot::eModeIcon, this);
         setSizeTrackProfile();
         if(isVisible())
         {
@@ -684,6 +664,18 @@ void CCanvas::setupGrid()
     update();
 }
 
+void CCanvas::setupBackgroundColor()
+{
+    QColorDialog::setCustomColor(0, "#FFFFBF");
+    const QColor &selected = QColorDialog::getColor(backColor, this, tr("Setup Map Background"));
+
+    if(selected.isValid())
+    {
+        backColor = selected;
+        update();
+    }
+}
+
 void CCanvas::convertGridPos2Str(const QPointF& pos, QString& str, bool simple)
 {
     grid->convertPos2Str(pos, str, simple);
@@ -741,17 +733,17 @@ CCanvas::scales_type_e CCanvas::getScalesType()
 }
 
 
-qreal CCanvas::getElevationAt(const QPointF& pos)
+qreal CCanvas::getElevationAt(const QPointF& pos) const
 {
     return dem->getElevationAt(pos);
 }
 
-void CCanvas::getElevationAt(const QPolygonF& pos, QPolygonF& ele)
+void CCanvas::getElevationAt(const QPolygonF& pos, QPolygonF& ele) const
 {
     return dem->getElevationAt(pos, ele);
 }
 
-void CCanvas::getElevationAt(SGisLine& line)
+void CCanvas::getElevationAt(SGisLine& line) const
 {
     return dem->getElevationAt(line);
 }
@@ -772,7 +764,7 @@ bool CCanvas::findPolylineCloseBy(const QPointF& pt1, const QPointF& pt2, qint32
 
 void CCanvas::saveSizeTrackProfile()
 {
-    if(plotTrackProfile == 0)
+    if(plotTrackProfile.isNull())
     {
         return;
     }
@@ -794,7 +786,7 @@ void CCanvas::saveSizeTrackProfile()
 
 void CCanvas::setSizeTrackProfile()
 {
-    if(plotTrackProfile == 0)
+    if(plotTrackProfile.isNull())
     {
         return;
     }
@@ -850,16 +842,9 @@ void CCanvas::showProfileAsWindow(bool yes)
 
 void CCanvas::showProfile(bool yes)
 {
-    if(plotTrackProfile)
+    if(nullptr != plotTrackProfile)
     {
-        if(yes)
-        {
-            plotTrackProfile->show();
-        }
-        else
-        {
-            plotTrackProfile->hide();
-        }
+        plotTrackProfile->setVisible(yes);
     }
 }
 
