@@ -45,8 +45,6 @@ class QLabel;
 class IPlot;
 struct SGisLine;
 
-
-
 class CCanvas : public QWidget
 {
     Q_OBJECT
@@ -54,7 +52,7 @@ public:
     CCanvas(QWidget * parent, const QString& name);
     virtual ~CCanvas();
 
-    static void setOverrideCursor(const QCursor& cursor, const QString &src);
+    static void setOverrideCursor(const QCursor &cursor, const QString&);
     static void restoreOverrideCursor(const QString &src);
     static void changeOverrideCursor(const QCursor& cursor, const QString &src);
 
@@ -66,6 +64,8 @@ public:
     void convertRad2Px(QPointF &pos);
     void convertPx2Rad(QPointF& pos);
 
+    void setupBackgroundColor();
+
     void setup();
     QString getProjection();
     void  setProjection(const QString& proj);
@@ -79,9 +79,9 @@ public:
     void  setScales(const scales_type_e type);
     scales_type_e getScalesType();
 
-    qreal getElevationAt(const QPointF &pos);
-    void  getElevationAt(const QPolygonF& pos, QPolygonF &ele);
-    void  getElevationAt(SGisLine &line);
+    qreal getElevationAt(const QPointF &pos) const;
+    void  getElevationAt(const QPolygonF& pos, QPolygonF &ele) const;
+    void  getElevationAt(SGisLine &line) const;
 
     void moveMap(const QPointF &delta);
     void zoomTo(const QRectF& rect);
@@ -109,11 +109,11 @@ public:
     void setMouseEditArea(const QPointF& pt);
     void setMouseWptBubble(const IGisItem::key_t& key);
     void setMousePrint();
+    void setMouseSelect();
 
     void showProfileAsWindow(bool yes);
     void showProfile(bool yes);
 
-
     /**
        @brief Add a message by key to be reported on the canvas
 
@@ -152,16 +152,16 @@ public slots:
     void slotTriggerCompleteUpdate(CCanvas::redraw_e flags);
 
 protected:
-    void resizeEvent(QResizeEvent * e);
-    void paintEvent(QPaintEvent * e);
-    void mousePressEvent(QMouseEvent * e);
-    void mouseMoveEvent(QMouseEvent * e);
-    void mouseReleaseEvent(QMouseEvent *e);
-    void mouseDoubleClickEvent(QMouseEvent * e);
-    void wheelEvent(QWheelEvent * e);
-    void enterEvent(QEvent * e);
-    void leaveEvent(QEvent * e);
-    void keyPressEvent(QKeyEvent * e);
+    void resizeEvent(QResizeEvent *e) override;
+    void paintEvent(QPaintEvent  *e) override;
+    void mousePressEvent(QMouseEvent  *e) override;
+    void mouseMoveEvent(QMouseEvent  *e) override;
+    void mouseReleaseEvent(QMouseEvent  *e) override;
+    void mouseDoubleClickEvent(QMouseEvent  *e) override;
+    void wheelEvent(QWheelEvent  *e) override;
+    void enterEvent(QEvent       *e) override;
+    void leaveEvent(QEvent       *e) override;
+    void keyPressEvent(QKeyEvent    *e) override;
 
 
 private slots:
@@ -175,17 +175,15 @@ private:
     void setSizeTrackProfile();
     void saveSizeTrackProfile();
     void setDrawContextSize(const QSize& s);
+    void setMouseCursor(IMouse& mouse, const QString& src);
+
+    QColor backColor = "#FFFFBF";       //< the background color used in case of missing map tiles
+    redraw_e needsRedraw = eRedrawAll;  //< set true to initiate a complete redraw of the screen content
+    CMapDraw * map;                     //< the map object attached to this canvas
+    CDemDraw * dem;                     //< the elevation data layer attached to this canvas
+    CGisDraw * gis;                     //< the GIS data layer attached to this canvas
+    CGrid * grid;                       //< the grid attached to this canvas
 
-    /// set true to initiate a complete redraw of the screen content
-    redraw_e needsRedraw;
-    /// the map object attached to this canvas
-    CMapDraw * map;
-    /// the elevation data layer attached to this canvas
-    CDemDraw * dem;
-    /// the GIS data layer attached to this canvas
-    CGisDraw * gis;
-    /// the grid attached to this canvas
-    CGrid * grid;
     /// the current point of focus (usually the canvas center)
     QPointF posFocus {12.00 * DEG_TO_RAD, 49.00 * DEG_TO_RAD};
 
@@ -216,6 +214,8 @@ private:
 
     QLabel * labelStatusMessages;
     QMap<QString, QString> statusMessages;
+
+    QMutex mousePressMutex;
 };
 
 #endif //CCANVAS_H
