@@ -40,8 +40,8 @@ CDemDraw::CDemDraw(CCanvas *canvas)
 {
     demList = new CDemList(canvas);
     CMainWindow::self().addDemList(demList, canvas->objectName());
-    connect(canvas, SIGNAL(destroyed()), demList, SLOT(deleteLater()));
-    connect(demList, SIGNAL(sigChanged()), this, SLOT(emitSigCanvasUpdate()));
+    connect(canvas,  &CCanvas::destroyed,   demList, &CDemList::deleteLater);
+    connect(demList, &CDemList::sigChanged, this,    &CDemDraw::emitSigCanvasUpdate);
 
     buildMapList();
 
@@ -68,13 +68,21 @@ void CDemDraw::setProjection(const QString& proj)
 
 void CDemDraw::setupDemPath()
 {
-    CDemPathSetup dlg(demPaths);
+    QStringList paths = demPaths;
+    CDemPathSetup dlg(paths);
     if(dlg.exec() != QDialog::Accepted)
     {
         return;
     }
 
-    foreach(CDemDraw * dem, dems)
+    setupDemPath(paths);
+}
+
+void CDemDraw::setupDemPath(const QStringList &paths)
+{
+    demPaths = paths;
+
+    for(CDemDraw * dem : dems)
     {
         QStringList keys;
         dem->saveActiveMapsList(keys);
@@ -130,11 +138,11 @@ void CDemDraw::buildMapList()
     QMutexLocker lock(&CDemItem::mutexActiveDems);
     demList->clear();
 
-    foreach(const QString &path, demPaths)
+    for(const QString &path : demPaths)
     {
         QDir dir(path);
         // find available maps
-        foreach(const QString &filename, dir.entryList(supportedFormats, QDir::Files|QDir::Readable, QDir::Name))
+        for(const QString &filename : dir.entryList(supportedFormats, QDir::Files|QDir::Readable, QDir::Name))
         {
             QFileInfo fi(filename);
 
@@ -201,7 +209,7 @@ void CDemDraw::restoreActiveMapsList(const QStringList& keys)
 {
     QMutexLocker lock(&CDemItem::mutexActiveDems);
 
-    foreach(const QString &key, keys)
+    for(const QString &key : keys)
     {
         for(int i = 0; i < demList->count(); i++)
         {
@@ -226,7 +234,7 @@ void CDemDraw::restoreActiveMapsList(const QStringList& keys, QSettings& cfg)
 {
     QMutexLocker lock(&CDemItem::mutexActiveDems);
 
-    foreach(const QString &key, keys)
+    for(const QString &key : keys)
     {
         for(int i = 0; i < demList->count(); i++)
         {
