@@ -24,14 +24,19 @@
 #include "gis/db/CDBProject.h"
 #include "gis/db/CSelectDBFolder.h"
 #include "gis/db/CSetupFolder.h"
+#include "gis/fit/CFitProject.h"
 #include "gis/gpx/CGpxProject.h"
 #include "gis/ovl/CGisItemOvlArea.h"
 #include "gis/prj/IGisProject.h"
 #include "gis/qms/CQmsProject.h"
+#include "gis/rte/CCreateRouteFromWpt.h"
 #include "gis/rte/CGisItemRte.h"
+#include "gis/slf/CSlfProject.h"
+#include "gis/trk/CCombineTrk.h"
 #include "gis/trk/CGisItemTrk.h"
 #include "gis/wpt/CGisItemWpt.h"
 #include "gis/wpt/CProjWpt.h"
+#include "helpers/CProgressDialog.h"
 #include "helpers/CSelectCopyAction.h"
 #include "helpers/CSelectProjectDialog.h"
 #include "helpers/CSettings.h"
@@ -39,7 +44,7 @@
 #include <QtWidgets>
 #include <QtXml>
 
-CGisWidget * CGisWidget::pSelf = 0;
+CGisWidget * CGisWidget::pSelf = nullptr;
 
 CGisWidget::CGisWidget(QMenu *menuProject, QWidget *parent)
     : QWidget(parent)
@@ -53,10 +58,12 @@ CGisWidget::CGisWidget(QMenu *menuProject, QWidget *parent)
     treeWks->header()->restoreState(cfg.value("Workspace/treeWks/state", treeWks->header()->saveState()).toByteArray());
     treeDB->header()->restoreState(cfg.value("Workspace/treeDB/state", treeDB->header()->saveState()).toByteArray());
 
-    connect(treeWks, SIGNAL(sigChanged()), SIGNAL(sigChanged()));
-    connect(treeDB, SIGNAL(sigChanged()), SLOT(slotHelpText()));
+    connect(treeWks, &CGisListWks::sigChanged, this, &CGisWidget::sigChanged);
+    connect(treeDB,  &CGisListDB::sigChanged,  this, &CGisWidget::slotHelpText);
 
     slotHelpText();
+
+    treeWks->slotLoadWorkspace();
 }
 
 CGisWidget::~CGisWidget()
@@ -91,7 +98,7 @@ void CGisWidget::loadGisProject(const QString& filename)
     treeWks->blockSignals(true);
 
     QMutexLocker lock(&IGisItem::mutexItems);
-    IGisProject * item = 0;
+    IGisProject *item = nullptr;
     QString suffix = QFileInfo(filename).suffix().toLower();
     if(suffix == "gpx")
     {
@@ -101,11 +108,24 @@ void CGisWidget::loadGisProject(const QString& filename)
     {
         item = new CQmsProject(filename, treeWks);
     }
+    else if(suffix == "slf")
+    {
+        item = new CSlfProject(filename);
+
+        // the CSlfProject does not - as the other C*Project - register itself in the list
+        // of currently opened projects. This is done manually here.
+        treeWks->addProject(item);
+    }
+    else if(suffix == "fit")
+    {
+        item = new CFitProject(filename, treeWks);
+    }
+
 
     if(item && !item->isValid())
     {
         delete item;
-        item = 0;
+        item = nullptr;
     }
 
     // skip if project is already loaded
@@ -114,7 +134,7 @@ void CGisWidget::loadGisProject(const QString& filename)
         QMessageBox::information(this, tr("Load project..."), tr("The project \"%1\" is already in the workspace.").arg(item->getName()), QMessageBox::Abort);
 
         delete item;
-        item = 0;
+        item = nullptr;
     }
 
     treeWks->blockSignals(false);
@@ -125,14 +145,7 @@ void CGisWidget::loadGisProject(const QString& filename)
 
 void CGisWidget::slotHelpText()
 {
-    if(treeDB->topLevelItemCount() == 0)
-    {
-        frameHelp->show();
-    }
-    else
-    {
-        frameHelp->hide();
-    }
+    frameHelp->setVisible(treeDB->topLevelItemCount() == 0);
 }
 
 void CGisWidget::slotSaveAll()
@@ -142,11 +155,18 @@ void CGisWidget::slotSaveAll()
     for(int i = 0; i < treeWks->topLevelItemCount(); i++)
     {
         IGisProject * item = dynamic_cast<IGisProject*>(treeWks->topLevelItem(i));
-        if(item == 0)
+        if(nullptr == item)
         {
             continue;
         }
-        item->save();
+        if(item->canSave())
+        {
+            item->save();
+        }
+        else
+        {
+            item->saveAs();
+        }
     }
 
     CCanvas::restoreOverrideCursor("slotSaveAll");
@@ -160,14 +180,14 @@ IGisProject * CGisWidget::selectProject()
     CSelectProjectDialog dlg(key, name, type, treeWks);
     dlg.exec();
 
-    IGisProject * project = 0;
+    IGisProject *project = nullptr;
     if(!key.isEmpty())
     {
         QMutexLocker lock(&IGisItem::mutexItems);
         for(int i = 0; i < treeWks->topLevelItemCount(); i++)
         {
             project = dynamic_cast<IGisProject*>(treeWks->topLevelItem(i));
-            if(project == 0)
+            if(nullptr == project)
             {
                 continue;
             }
@@ -181,27 +201,28 @@ IGisProject * CGisWidget::selectProject()
     {
         quint64 idParent;
         QString db;
+        QString host;
         IDBFolder::type_e type;
 
-        CSelectDBFolder dlg1(idParent, db, this);
+        CSelectDBFolder dlg1(idParent, db, host, this);
         if(dlg1.exec() == QDialog::Rejected)
         {
-            return 0;
+            return nullptr;
         }
 
         CSetupFolder dlg2(type, name, false, this);
         if(dlg2.exec() == QDialog::Rejected)
         {
-            return 0;
+            return nullptr;
         }
 
         QMutexLocker lock(&IGisItem::mutexItems);
-        CEvtW2DCreate evt(name, type, idParent, db);
+        CEvtW2DCreate evt(name, type, idParent, db, host);
         QApplication::sendEvent(treeDB, &evt);
 
         if(evt.idChild)
         {
-            while(project == 0)
+            while(nullptr == project)
             {
                 QApplication::processEvents(QEventLoop::WaitForMoreEvents|QEventLoop::ExcludeUserInputEvents, 100);
                 project = treeWks->getProjectById(evt.idChild, db);
@@ -245,6 +266,48 @@ void CGisWidget::getItemsByPos(const QPointF& pos, QList<IGisItem*>& items)
     }
 }
 
+void CGisWidget::getItemsByKeys(const QList<IGisItem::key_t>& keys, QList<IGisItem*>& items)
+{
+    QMutexLocker lock(&IGisItem::mutexItems);
+    for(int i = 0; i < treeWks->topLevelItemCount(); i++)
+    {
+        QTreeWidgetItem * item = treeWks->topLevelItem(i);
+        IGisProject * project = dynamic_cast<IGisProject*>(item);
+        if(project)
+        {
+            project->getItemsByKeys(keys, items);
+            continue;
+        }
+        IDevice * device = dynamic_cast<IDevice*>(item);
+        if(device)
+        {
+            device->getItemsByKeys(keys, items);
+            continue;
+        }
+    }
+}
+
+void CGisWidget::getItemsByArea(const QRectF& area, IGisItem::selflags_t flags, QList<IGisItem *> &items)
+{
+    QMutexLocker lock(&IGisItem::mutexItems);
+    for(int i = 0; i < treeWks->topLevelItemCount(); i++)
+    {
+        QTreeWidgetItem * item = treeWks->topLevelItem(i);
+        IGisProject * project = dynamic_cast<IGisProject*>(item);
+        if(project)
+        {
+            project->getItemsByArea(area, flags, items);
+            continue;
+        }
+        IDevice * device = dynamic_cast<IDevice*>(item);
+        if(device)
+        {
+            device->getItemsByArea(area, flags, items);
+            continue;
+        }
+    }
+}
+
 void CGisWidget::mouseMove(const QPointF& pos)
 {
     QMutexLocker lock(&IGisItem::mutexItems);
@@ -262,7 +325,7 @@ void CGisWidget::mouseMove(const QPointF& pos)
 
 IGisItem * CGisWidget::getItemByKey(const IGisItem::key_t& key)
 {
-    IGisItem * item = 0;
+    IGisItem *item = nullptr;
     QMutexLocker lock(&IGisItem::mutexItems);
     for(int i = 0; i < treeWks->topLevelItemCount(); i++)
     {
@@ -276,7 +339,7 @@ IGisItem * CGisWidget::getItemByKey(const IGisItem::key_t& key)
             }
 
             item = project->getItemByKey(key);
-            if(item != 0)
+            if(nullptr != item)
             {
                 break;
             }
@@ -293,7 +356,7 @@ IGisItem * CGisWidget::getItemByKey(const IGisItem::key_t& key)
             }
 
             item = device->getItemByKey(key);
-            if(item != 0)
+            if(nullptr != item)
             {
                 break;
             }
@@ -310,7 +373,7 @@ void CGisWidget::delItemByKey(const IGisItem::key_t& key)
     for(int i = 0; i < treeWks->topLevelItemCount(); i++)
     {
         IGisProject * project = dynamic_cast<IGisProject*>(treeWks->topLevelItem(i));
-        if(project == 0)
+        if(nullptr == project)
         {
             continue;
         }
@@ -321,7 +384,7 @@ void CGisWidget::delItemByKey(const IGisItem::key_t& key)
             CDBProject * dbp = dynamic_cast<CDBProject*>(project);
             if(dbp)
             {
-                dbp->postStatus();
+                dbp->postStatus(true);
             }
         }
 
@@ -335,20 +398,81 @@ void CGisWidget::delItemByKey(const IGisItem::key_t& key)
     emit sigChanged();
 }
 
+void CGisWidget::delItemsByKey(const QList<IGisItem::key_t> &keys)
+{
+    QMessageBox::StandardButtons last   = QMessageBox::NoButton;
+
+    QSet<CDBProject*>   projects;
+    QSet<IGisProject*>  projectsAll;
+
+    for(const IGisItem::key_t key : keys)
+    {
+        IGisItem * gisItem = getItemByKey(key);
+        if(nullptr != gisItem)
+        {
+            bool yes = false;
+            IGisProject *project = dynamic_cast<IGisProject*>(gisItem->parent());
+            if(nullptr != project)
+            {
+                project->blockUpdateItems(true);
+                yes = project->delItemByKey(gisItem->getKey(), last);
+
+
+                /*
+                    collect database projects to update their counterpart in
+                    the database view, after all operations are done.
+                 */
+                if(yes && project->getType() == IGisProject::eTypeDb)
+                {
+                    projects << dynamic_cast<CDBProject*>(project);
+                }
+
+                /*
+                    Collect all projects to unblock update later on.
+                 */
+                projectsAll << project;
+            }
+
+            if(last == QMessageBox::Cancel)
+            {
+                break;
+            }
+        }
+    }
+
+    // make all database projects that are changed to post their new status
+    // this will update the database view.
+    for(CDBProject * project : projects)
+    {
+        project->postStatus(true);
+    }
+    // unblock update for all projects seen
+    for(IGisProject * project : projectsAll)
+    {
+        project->blockUpdateItems(false);
+    }
+
+    CCanvas * canvas = CMainWindow::self().getVisibleCanvas();
+    if(canvas)
+    {
+        canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
+    }
+}
+
 void CGisWidget::editItemByKey(const IGisItem::key_t& key)
 {
     QMutexLocker lock(&IGisItem::mutexItems);
     for(int i = 0; i < treeWks->topLevelItemCount(); i++)
     {
-        QTreeWidgetItem * item = treeWks->topLevelItem(i);
-        IGisProject * project = dynamic_cast<IGisProject*>(item);
-        if(project != 0)
+        QTreeWidgetItem *item = treeWks->topLevelItem(i);
+        IGisProject *project = dynamic_cast<IGisProject*>(item);
+        if(nullptr != project)
         {
             project->editItemByKey(key);
             continue;
         }
         IDevice * device = dynamic_cast<IDevice*>(item);
-        if(device != 0)
+        if(nullptr != device)
         {
             device->editItemByKey(key);
             continue;
@@ -362,14 +486,14 @@ void CGisWidget::copyItemByKey(const IGisItem::key_t &key)
 {
     QMutexLocker lock(&IGisItem::mutexItems);
 
-    IGisItem * item = getItemByKey(key);
-    if(item == 0)
+    IGisItem *item = getItemByKey(key);
+    if(nullptr == item)
     {
         return;
     }
 
-    IGisProject * project = selectProject();
-    if(project == 0)
+    IGisProject *project = selectProject();
+    if(nullptr == project)
     {
         return;
     }
@@ -381,12 +505,45 @@ void CGisWidget::copyItemByKey(const IGisItem::key_t &key)
     emit sigChanged();
 }
 
+void CGisWidget::copyItemsByKey(const QList<IGisItem::key_t> &keys)
+{
+    QMutexLocker lock(&IGisItem::mutexItems);
+
+    IGisProject * project = selectProject();
+    if(nullptr == project)
+    {
+        return;
+    }
+
+    int lastResult = CSelectCopyAction::eResultNone;
+
+    project->blockUpdateItems(true);
+    int cnt = 1;
+    PROGRESS_SETUP(tr("Copy items..."), 0, keys.count(), this);
+    for(const IGisItem::key_t& key : keys)
+    {
+        PROGRESS(cnt++, break);
+        IGisItem * gisItem = getItemByKey(key);
+        if(nullptr != gisItem)
+        {
+            project->insertCopyOfItem(gisItem, NOIDX, lastResult);
+        }
+    }
+    project->blockUpdateItems(false);
+
+    CCanvas *canvas = CMainWindow::self().getVisibleCanvas();
+    if(nullptr != canvas)
+    {
+        canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
+    }
+}
+
 void CGisWidget::projWptByKey(const IGisItem::key_t& key)
 {
     QMutexLocker lock(&IGisItem::mutexItems);
 
-    CGisItemWpt * wpt = dynamic_cast<CGisItemWpt*>(getItemByKey(key));
-    if(wpt != 0)
+    CGisItemWpt *wpt = dynamic_cast<CGisItemWpt*>(getItemByKey(key));
+    if(nullptr != wpt)
     {
         CProjWpt dlg(*wpt, 0);
         dlg.exec();
@@ -399,16 +556,16 @@ void CGisWidget::projWptByKey(const IGisItem::key_t& key)
 void CGisWidget::moveWptByKey(const IGisItem::key_t& key)
 {
     QMutexLocker lock(&IGisItem::mutexItems);
-    CGisItemWpt * wpt = dynamic_cast<CGisItemWpt*>(getItemByKey(key));
-    if(wpt != 0)
+    CGisItemWpt *wpt = dynamic_cast<CGisItemWpt*>(getItemByKey(key));
+    if(nullptr != wpt)
     {
         if(!wpt->setReadOnlyMode(false))
         {
             return;
         }
 
-        CCanvas * canvas = CMainWindow::self().getVisibleCanvas();
-        if(canvas != 0)
+        CCanvas *canvas = CMainWindow::self().getVisibleCanvas();
+        if(nullptr != canvas)
         {
             canvas->setMouseMoveWpt(*wpt);
         }
@@ -419,18 +576,38 @@ void CGisWidget::toggleWptBubble(const IGisItem::key_t &key)
 {
     QMutexLocker lock(&IGisItem::mutexItems);
     CGisItemWpt * wpt = dynamic_cast<CGisItemWpt*>(getItemByKey(key));
-    if(wpt != 0)
+    if(nullptr != wpt)
     {
         wpt->toggleBubble();
     }
 }
 
+void CGisWidget::addWptByPos(QPointF pt) const
+{
+    QString name;
+    QString icon;
+    if(!CGisItemWpt::getNewWptData(pt, icon, name))
+    {
+        return;
+    }
+
+    IGisProject * project = CGisWidget::self().selectProject();
+    if(nullptr == project)
+    {
+        return;
+    }
+
+    QMutexLocker lock(&IGisItem::mutexItems);
+    CGisItemWpt * wpt = new CGisItemWpt(pt, name, icon, project);
+    wpt->edit();
+}
+
 void CGisWidget::focusTrkByKey(bool yes, const IGisItem::key_t& key)
 {
     QMutexLocker lock(&IGisItem::mutexItems);
 
     CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(getItemByKey(key));
-    if(trk != 0)
+    if(nullptr != trk)
     {
         trk->gainUserFocus(yes);
     }
@@ -443,7 +620,7 @@ void CGisWidget::focusRteByKey(bool yes, const IGisItem::key_t &key)
     QMutexLocker lock(&IGisItem::mutexItems);
 
     CGisItemRte * rte = dynamic_cast<CGisItemRte*>(getItemByKey(key));
-    if(rte != 0)
+    if(nullptr != rte)
     {
         rte->gainUserFocus(yes);
     }
@@ -451,12 +628,24 @@ void CGisWidget::focusRteByKey(bool yes, const IGisItem::key_t &key)
     emit sigChanged();
 }
 
+void CGisWidget::convertRouteToTrack(const IGisItem::key_t &key)
+{
+    QMutexLocker lock(&IGisItem::mutexItems);
+    CGisItemRte * rte = dynamic_cast<CGisItemRte*>(getItemByKey(key));
+    if(nullptr != rte)
+    {
+        rte->toTrack();
+    }
+
+    emit sigChanged();
+}
+
 void CGisWidget::cutTrkByKey(const IGisItem::key_t& key)
 {
     QMutexLocker lock(&IGisItem::mutexItems);
 
     CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(getItemByKey(key));
-    if(trk != 0 && trk->cut())
+    if(nullptr != trk && trk->cut())
     {
         int res = QMessageBox::question(this, tr("Cut Track..."), tr("Do you want to delete the original track?"), QMessageBox::Ok|QMessageBox::No, QMessageBox::Ok);
         if(res == QMessageBox::Ok)
@@ -473,7 +662,7 @@ void CGisWidget::reverseTrkByKey(const IGisItem::key_t& key)
     QMutexLocker lock(&IGisItem::mutexItems);
 
     CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(getItemByKey(key));
-    if(trk)
+    if(nullptr != trk)
     {
         trk->reverse();
     }
@@ -481,14 +670,43 @@ void CGisWidget::reverseTrkByKey(const IGisItem::key_t& key)
     emit sigChanged();
 }
 
-void CGisWidget::combineTrkByKey(const IGisItem::key_t& key)
+void CGisWidget::combineTrkByKey(const IGisItem::key_t& keyTrk)
 {
+    QMutexLocker lock(&IGisItem::mutexItems);
+
     QList<IGisItem::key_t> keys;
-    keys << key;
-    combineTrkByKey(keys);
+    IGisItem * item = dynamic_cast<IGisItem*>(getItemByKey(keyTrk));
+    if(item == nullptr)
+    {
+        return;
+    }
+
+    keys << keyTrk;
+
+    IGisProject * project = dynamic_cast<IGisProject*>(item->parent());
+    if(project == nullptr)
+    {
+        return;
+    }
+
+    const int N = project->childCount();
+    for(int i = 0; i < N; i++)
+    {
+        CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(project->child(i));
+        if(trk != nullptr)
+        {
+            const IGisItem::key_t& key = trk->getKey();
+            if(key != keyTrk)
+            {
+                keys << key;
+            }
+        }
+    }
+
+    combineTrkByKey(keys, {keyTrk});
 }
 
-void CGisWidget::combineTrkByKey(const QList<IGisItem::key_t>& keys)
+void CGisWidget::combineTrkByKey(const QList<IGisItem::key_t>& keys, const QList<IGisItem::key_t>& keysPreSel)
 {
     if(keys.isEmpty())
     {
@@ -497,11 +715,8 @@ void CGisWidget::combineTrkByKey(const QList<IGisItem::key_t>& keys)
 
     QMutexLocker lock(&IGisItem::mutexItems);
 
-    CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(getItemByKey(keys.first()));
-    if(trk)
-    {
-        trk->combine(keys);
-    }
+    CCombineTrk dlg(keys, keysPreSel, this);
+    dlg.exec();
 
     emit sigChanged();
 }
@@ -511,7 +726,7 @@ void CGisWidget::editTrkByKey(const IGisItem::key_t& key)
     QMutexLocker lock(&IGisItem::mutexItems);
 
     CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(getItemByKey(key));
-    if(trk != 0)
+    if(nullptr != trk)
     {
         if(!trk->setReadOnlyMode(false))
         {
@@ -519,7 +734,7 @@ void CGisWidget::editTrkByKey(const IGisItem::key_t& key)
         }
 
         CCanvas * canvas = CMainWindow::self().getVisibleCanvas();
-        if(canvas != 0)
+        if(nullptr != canvas)
         {
             canvas->setMouseEditTrk(*trk);
         }
@@ -531,10 +746,10 @@ void CGisWidget::rangeTrkByKey(const IGisItem::key_t& key)
     QMutexLocker lock(&IGisItem::mutexItems);
 
     CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(getItemByKey(key));
-    if(trk != 0)
+    if(nullptr != trk)
     {
         CCanvas * canvas = CMainWindow::self().getVisibleCanvas();
-        if(canvas != 0)
+        if(nullptr != canvas)
         {
             canvas->setMouseRangeTrk(*trk);
         }
@@ -546,15 +761,15 @@ void CGisWidget::editRteByKey(const IGisItem::key_t& key)
     QMutexLocker lock(&IGisItem::mutexItems);
 
     CGisItemRte * rte = dynamic_cast<CGisItemRte*>(getItemByKey(key));
-    if(rte != 0)
+    if(nullptr != rte)
     {
         if(!rte->setReadOnlyMode(false))
         {
             return;
         }
 
-        CCanvas * canvas = CMainWindow::self().getVisibleCanvas();
-        if(canvas != 0)
+        CCanvas *canvas = CMainWindow::self().getVisibleCanvas();
+        if(nullptr != canvas)
         {
             canvas->setMouseEditRte(*rte);
         }
@@ -566,7 +781,7 @@ void CGisWidget::calcRteByKey(const IGisItem::key_t& key)
     QMutexLocker lock(&IGisItem::mutexItems);
 
     CGisItemRte * rte = dynamic_cast<CGisItemRte*>(getItemByKey(key));
-    if(rte != 0)
+    if(nullptr != rte)
     {
         rte->calc();
     }
@@ -577,7 +792,7 @@ void CGisWidget::resetRteByKey(const IGisItem::key_t& key)
     QMutexLocker lock(&IGisItem::mutexItems);
 
     CGisItemRte * rte = dynamic_cast<CGisItemRte*>(getItemByKey(key));
-    if(rte != 0)
+    if(rte != nullptr)
     {
         rte->reset();
     }
@@ -589,7 +804,7 @@ void CGisWidget::editAreaByKey(const IGisItem::key_t& key)
     QMutexLocker lock(&IGisItem::mutexItems);
 
     CGisItemOvlArea * area = dynamic_cast<CGisItemOvlArea*>(getItemByKey(key));
-    if(area != 0)
+    if(area != nullptr)
     {
         if(!area->setReadOnlyMode(false))
         {
@@ -597,13 +812,21 @@ void CGisWidget::editAreaByKey(const IGisItem::key_t& key)
         }
 
         CCanvas * canvas = CMainWindow::self().getVisibleCanvas();
-        if(canvas != 0)
+        if(canvas != nullptr)
         {
             canvas->setMouseEditArea(*area);
         }
     }
 }
 
+void CGisWidget::makeRteFromWpt(const QList<IGisItem::key_t>& keys)
+{
+    QMutexLocker lock(&IGisItem::mutexItems);
+
+    CCreateRouteFromWpt dlg(keys, this);
+    dlg.exec();
+}
+
 void CGisWidget::draw(QPainter& p, const QPolygonF& viewport, CGisDraw * gis)
 {
     QFontMetricsF fm(CMainWindow::self().getMapFont());
@@ -618,16 +841,16 @@ void CGisWidget::draw(QPainter& p, const QPolygonF& viewport, CGisDraw * gis)
             break;
         }
 
-        QTreeWidgetItem * item = treeWks->topLevelItem(i);
+        QTreeWidgetItem *item = treeWks->topLevelItem(i);
 
-        IGisProject * project = dynamic_cast<IGisProject*>(item);
-        if(project)
+        IGisProject *project = dynamic_cast<IGisProject*>(item);
+        if(nullptr != project)
         {
             project->drawItem(p, viewport, blockedAreas, gis);
             continue;
         }
-        IDevice * device = dynamic_cast<IDevice*>(item);
-        if(device)
+        IDevice *device = dynamic_cast<IDevice*>(item);
+        if(nullptr != device)
         {
             device->drawItem(p, viewport, blockedAreas, gis);
             continue;
@@ -645,13 +868,13 @@ void CGisWidget::draw(QPainter& p, const QPolygonF& viewport, CGisDraw * gis)
         QTreeWidgetItem * item = treeWks->topLevelItem(i);
 
         IGisProject * project = dynamic_cast<IGisProject*>(item);
-        if(project)
+        if(nullptr != project)
         {
             project->drawLabel(p, viewport, blockedAreas, fm, gis);
             continue;
         }
         IDevice * device = dynamic_cast<IDevice*>(item);
-        if(device)
+        if(nullptr != device)
         {
             device->drawLabel(p, viewport, blockedAreas, fm, gis);
             continue;
@@ -671,13 +894,13 @@ void CGisWidget::fastDraw(QPainter& p, const QRectF& viewport, CGisDraw *gis)
         QTreeWidgetItem * item = treeWks->topLevelItem(i);
 
         IGisProject * project = dynamic_cast<IGisProject*>(item);
-        if(project)
+        if(nullptr != project)
         {
             project->drawItem(p, viewport, gis);
             continue;
         }
         IDevice * device = dynamic_cast<IDevice*>(item);
-        if(device)
+        if(nullptr != device)
         {
             device->drawItem(p, viewport, gis);
             continue;
