@@ -24,6 +24,7 @@
 #include <QSqlDatabase>
 #include <QWidget>
 
+#include "db/IDBFolder.h"
 #include "gis/IGisItem.h"
 
 class CGisDraw;
@@ -37,6 +38,8 @@ enum event_types_e
     ,eEvtD2WShowItems   = QEvent::User + 4
     ,eEvtD2WHideItems   = QEvent::User + 5
     ,eEvtD2WUpdateLnF   = QEvent::User + 6
+    ,eEvtD2WUpdateItems = QEvent::User + 7
+    ,eEvtD2WReload      = QEvent::User + 8
 
     ,eEvtW2DAckInfo     = QEvent::User + 100
     ,eEvtW2DCreate      = QEvent::User + 101
@@ -91,6 +94,8 @@ public:
     {
     }
 
+    /// if true only the items in the list are loaded. Any other item loaded but not part of the list will be removed.
+    bool addItemsExclusively = false;
     quint64 id;
     QString db;
     QList<evt_item_t> items;
@@ -111,14 +116,28 @@ public:
 class CEvtW2DAckInfo : public QEvent
 {
 public:
-    CEvtW2DAckInfo(bool loaded, quint64 id, const QString& db) : QEvent(QEvent::Type(eEvtW2DAckInfo)), isLoaded(loaded), updateLostFound(false), id(id), db(db)
+    CEvtW2DAckInfo(Qt::CheckState checkState, quint64 id, const QString& db, const QString& host)
+        : QEvent(QEvent::Type(eEvtW2DAckInfo))
+        , checkState(checkState)
+        , id(id)
+        , db(db)
+        , host(host)
     {
     }
 
-    bool isLoaded;
-    bool updateLostFound;
+    CEvtW2DAckInfo(quint64 id, const QString& db, const QString& host)
+        : QEvent(QEvent::Type(eEvtW2DAckInfo))
+        , id(id)
+        , db(db)
+        , host(host)
+    {
+    }
+
+    Qt::CheckState checkState = Qt::Unchecked;
+    bool updateLostFound = false;
     quint64 id;
     QString db;
+    QString host;
     QSet<QString> keysChildren;
 };
 
@@ -136,14 +155,42 @@ public:
 class CEvtW2DCreate : public QEvent
 {
 public:
-    CEvtW2DCreate(const QString& name, IDBFolder::type_e type, quint64 id, const QString& db) : QEvent(QEvent::Type(eEvtW2DCreate)), name(name), type(type), idParent(id), idChild(0), db(db)
+    CEvtW2DCreate(const QString& name, IDBFolder::type_e type, quint64 id, const QString& db, const QString& host)
+        : QEvent(QEvent::Type(eEvtW2DCreate))
+        , name(name)
+        , type(type)
+        , idParent(id)
+        , db(db)
+        , host(host)
     {
     }
 
     QString name;
     IDBFolder::type_e type;
     quint64 idParent;
-    quint64 idChild;
+    quint64 idChild = 0;
+    QString db;
+    QString host;
+};
+
+class CEvtD2WUpdateItems : public QEvent
+{
+public:
+    CEvtD2WUpdateItems(quint64 id, const QString& db) : QEvent(QEvent::Type(eEvtD2WUpdateItems)), id(id), db(db)
+    {
+    }
+
+    quint64 id;
+    QString db;
+};
+
+class CEvtD2WReload : public QEvent
+{
+public:
+    CEvtD2WReload(const QString& db) : QEvent(QEvent::Type(eEvtD2WReload)), db(db)
+    {
+    }
+
     QString db;
 };
 
@@ -201,6 +248,15 @@ public:
      */
     void getItemsByPos(const QPointF& pos, QList<IGisItem *> &items);
 
+    /**
+       @brief Get items matching the given area
+
+       @param area      a rectangle in screen pixel coordinates
+       @param flags     flag field with IGisItem::selection_e flags set
+       @param items     a list to receive the temporary pointers to the found items
+     */
+    void getItemsByArea(const QRectF& area, IGisItem::selflags_t flags, QList<IGisItem *> &items);
+
     /**
        @brief Find first item with matching key
        @param key       the item's key as it is returned from IGisItem::getKey()
@@ -208,6 +264,7 @@ public:
      */
     IGisItem * getItemByKey(const IGisItem::key_t &key);
 
+    void getItemsByKeys(const QList<IGisItem::key_t>& keys, QList<IGisItem*>& items);
     /**
        @brief Delete all items with matching key from workspace
 
@@ -215,6 +272,8 @@ public:
      */
     void delItemByKey(const IGisItem::key_t &key);
 
+    void delItemsByKey(const QList<IGisItem::key_t> &keys);
+
     /**
        @brief Edit / view item details
        @param key       the item's key as it is returned from IGisItem::getKey()
@@ -227,6 +286,12 @@ public:
      */
     void copyItemByKey(const IGisItem::key_t &key);
 
+    /**
+       @brief Select a project and add a copy of all items in the list
+       @param keys      a list of item keys to copy
+     */
+    void copyItemsByKey(const QList<IGisItem::key_t> &keys);
+
     /**
        @brief Clone waypoint and move clone
        @param key       the item's key as it is returned from IGisItem::getKey()
@@ -239,6 +304,12 @@ public:
      */
     void moveWptByKey(const IGisItem::key_t &key);
 
+    /**
+       @brief Add a new waypoint by Position
+       @param pt    the position in []
+     */
+    void addWptByPos(QPointF pt) const;
+
     void toggleWptBubble(const IGisItem::key_t &key);
 
     /**
@@ -250,6 +321,8 @@ public:
 
     void focusRteByKey(bool yes, const IGisItem::key_t &key);
 
+    void convertRouteToTrack(const IGisItem::key_t &key);
+
 
     void cutTrkByKey(const IGisItem::key_t &key);
 
@@ -257,9 +330,22 @@ public:
 
     void reverseTrkByKey(const IGisItem::key_t &key);
 
-    void combineTrkByKey(const IGisItem::key_t &key);
+    /**
+       @brief Combine all tracks in a given track's project
 
-    void combineTrkByKey(const QList<IGisItem::key_t>& keys);
+       This will collect all tracks in a project and pass them
+       to the track combine dialog.
+
+       @param keyTrk    the key of the first track
+     */
+    void combineTrkByKey(const IGisItem::key_t &keyTrk);
+
+    /**
+       @brief Combine al tracks in the given list of keys.
+
+       @param keys  a list of GIS item keys
+     */
+    void combineTrkByKey(const QList<IGisItem::key_t>& keys, const QList<IGisItem::key_t> &keysPreSel);
 
     void rangeTrkByKey(const IGisItem::key_t &key);
 
@@ -271,6 +357,8 @@ public:
 
     void editAreaByKey(const IGisItem::key_t &key);
 
+    void makeRteFromWpt(const QList<IGisItem::key_t>& keys);
+
     /**
        @brief Select a project via dialog
 
