@@ -22,6 +22,7 @@
 #include <QTreeWidgetItem>
 
 #include <QColor>
+#include <QCoreApplication>
 #include <QDateTime>
 #include <QDomNode>
 #include <QMap>
@@ -42,11 +43,13 @@ class IGisProject;
 
 class IGisItem : public QTreeWidgetItem
 {
+    Q_DECLARE_TR_FUNCTIONS(IGisItem)
 public:
     struct history_event_t
     {
         QDateTime time;
         QString hash;
+        QString who = "QMapShack";
         QString icon;
         QString comment;
         QByteArray data;
@@ -58,6 +61,13 @@ public:
         {
         }
 
+        void reset()
+        {
+            histIdxInitial = NOIDX;
+            histIdxCurrent = NOIDX;
+            events.clear();
+        }
+
         qint32 histIdxInitial;
         qint32 histIdxCurrent;
         QList<history_event_t> events;
@@ -126,8 +136,23 @@ public:
     {
         eMarkNone      = 0
         ,eMarkChanged   = 0x00000001
+        ,eMarkNotPart   = 0x00000002
+        ,eMarkNotInDB   = 0x00000004
     };
 
+    enum selection_e
+    {
+        eSelectionNone          = 0
+        , eSelectionExact       = 0x00000001
+        , eSelectionIntersect   = 0x00000002
+        , eSelectionTrk         = 0x80000000
+        , eSelectionWpt         = 0x40000000
+        , eSelectionRte         = 0x20000000
+        , eSelectionOvl         = 0x10000000
+    };
+
+    using selflags_t = quint32;
+
     struct key_t
     {
         bool operator==(const key_t& k) const
@@ -155,12 +180,17 @@ public:
     /// this mutex has to be locked when ever the item list is accessed.
     static QMutex mutexItems;
 
+    /**
+       @brief If the item is part of a database project it will update itself with the database content
+     */
+    virtual void updateFromDB(quint64 id, QSqlDatabase& db);
+
     /**
        @brief Update the visual representation of the QTreeWidgetItem
        @param enable
        @param disable
      */
-    virtual void updateDecoration(mark_e enable, mark_e disable);
+    virtual void updateDecoration(quint32 enable, quint32 disable);
 
     /**
        @brief Save the item's data into a GPX structure
@@ -172,7 +202,7 @@ public:
        @brief Get key string to identify object
        @return
      */
-    const key_t& getKey();
+    const key_t& getKey() const;
 
     /**
        @brief Get a hash over the items data.
@@ -186,6 +216,18 @@ public:
      */
     const QString& getHash();
 
+    /**
+       @brief Get the hash stored in the database when the item was loaded
+
+       @return The hash as a string
+     */
+    const QString& getLastDatabaseHash();
+
+    /**
+       @brief Read the hash stored in the database
+     */
+    void setLastDatabaseHash(quint64 id, QSqlDatabase& db);
+
     /**
        @brief Get the icon attached to object
        @return
@@ -249,9 +291,9 @@ public:
        @param mouse     a pointer to the mouse object initiating the action
        @return A null pointer is returned if no screen option are available
      */
-    virtual IScrOpt * getScreenOptions(const QPoint& origin, IMouse * mouse)
+    virtual IScrOpt* getScreenOptions(const QPoint& origin, IMouse * mouse)
     {
-        return 0;
+        return nullptr;
     }
 
     /**
@@ -272,6 +314,8 @@ public:
      */
     virtual bool isCloseTo(const QPointF& pos) = 0;
 
+    virtual bool isWithin(const QRectF& area, selflags_t mode) = 0;
+
     /**
        @brief Receive the current mouse position
 
@@ -307,9 +351,9 @@ public:
 
     /**
        @brief Check if item is on a GPS device
-       @return True if the item is stored on a device
+       @return The device type (IDevice::type_e). IDevice::eTypeNone if the item is not stored on a device.
      */
-    bool isOnDevice() const;
+    qint32 isOnDevice() const;
 
     /**
        @brief Check if there are any pending unsaved changes
@@ -389,6 +433,18 @@ public:
      */
     void cutHistory();
 
+    /**
+       @brief Create a clone of itself and pass back the pointer
+
+       Add the cloned item to the project with the same index as the original
+
+       @return The pointer of the cloned item
+     */
+    virtual IGisItem * createClone() = 0;
+
+
+    IGisProject * getParentProject() const;
+
     /**
        @brief Remove all HTML tags from a string
        @param str the string
@@ -433,6 +489,18 @@ public:
      */
     static QString toLink(bool isReadOnly, const QString& href, const QString& str, const QString& key);
 
+    /**
+       @brief Unified handler to get a new item name and a pointer to the traget project
+
+       @param name      a reference to a string object with the default name and to receive the name
+       @param project   a reference to a IGisProject pointer. On success it will point to the project instance
+       @param itemtype  a string to be used for the item type in the dialogs
+
+
+       @return Returns true on success. Otherwise false.
+     */
+    static bool getNameAndProject(QString &name, IGisProject *&project, const QString &itemtype);
+
     /// a no key value that can be used to nullify references.
     const static QString noKey;
 
@@ -446,6 +514,7 @@ public:
     };
 
     static const color_t colorMap[];
+    static const size_t colorMapSize;
 
 protected:
     /// set icon of QTreeWidgetItem
@@ -455,11 +524,11 @@ protected:
     /// write waypoint data to an XML snippet
     void writeWpt(QDomElement &xml, const wpt_t &wpt);
     /// generate a unique key from item's data
-    virtual void genKey();
+    virtual void genKey() const;
     /// setup the history structure right after the creation of the item
     void setupHistory();
     /// update current history entry (e.g. to save the flags)
-    void updateHistory();
+    virtual void updateHistory();
     /// convert a color string from GPX to a QT color
     QColor str2color(const QString& name);
     /// convert a QT color to a string to be used in a GPX file
@@ -469,17 +538,33 @@ protected:
     /// call when ever you make a change to the item's data
     virtual void changed(const QString& what, const QString& icon);
 
-    virtual void loadFromDb(quint64 id, QSqlDatabase& db);
-
+    void loadFromDb(quint64 id, QSqlDatabase& db);
     bool isVisible(const QRectF& rect, const QPolygonF& viewport, CGisDraw * gis);
     bool isVisible(const QPointF& point, const QPolygonF& viewport, CGisDraw * gis);
+    bool isWithin(const QRectF& area, selflags_t flags, const QPolygonF& points);
 
+    /**
+       @brief Converts a string with HTML tags to a string without HTML depending on the device
+
+       Some devices e.g. Garmin can not handle HTML.
+
+       @param str   a string
+       @return A string with HTML removed depending on the device
+     */
+    QString html2Dev(const QString& str);
+
+    /// see flags_e for possible flags
     quint32 flags = 0;
-    key_t key;
+    /// the item's unique key
+    mutable key_t key;
+    /// each item has an icon for the tree widget
     QPixmap icon;
+    /// the dimensions of the item
     QRectF boundingRect;
-
+    /// that's where the real data is. An item is completely defined by it's history
     history_t history;
+    /// the hash in the database when the item was loaded/saved
+    QString lastDatabaseHash;
 
     enum flags_e
     {
@@ -491,7 +576,7 @@ protected:
 
     static inline bool isBlocked(const QRectF& rect, const QList<QRectF> &blockedAreas)
     {
-        foreach(const QRectF &r, blockedAreas)
+        for(const QRectF &r : blockedAreas)
         {
             if(rect.intersects(r))
             {
