@@ -19,6 +19,7 @@
 #ifndef CDBITEM_H
 #define CDBITEM_H
 
+#include <QCoreApplication>
 #include <QTreeWidgetItem>
 
 class IDBFolder;
@@ -26,6 +27,7 @@ class QSqlDatabase;
 
 class CDBItem : public QTreeWidgetItem
 {
+    Q_DECLARE_TR_FUNCTIONS(CDBItem)
 public:
     CDBItem(QSqlDatabase& db, quint64 id, IDBFolder * parent);
     virtual ~CDBItem();
@@ -38,14 +40,24 @@ public:
     {
         return key;
     }
+
+    /**
+       @brief Send show/hide events to the workspace
+     */
     void toggle();
+
+    /**
+       @brief Delete the folder/item relation in the database
+     */
     void remove();
 
+    void updateAge();
+
 private:
     QSqlDatabase& db;
     quint64 id;
 
-    int type;
+    int type = 0;
     QString key;
 };
 
