@@ -21,26 +21,37 @@
 
 #include "gis/prj/IGisProject.h"
 
-#include <QDateTime>
-#include <QDomElement>
-#include <QUrl>
-
 class CGisListWks;
 class CGisDraw;
 
 class CGpxProject : public IGisProject
 {
+    Q_DECLARE_TR_FUNCTIONS(CGpxProject)
 public:
     CGpxProject(const QString &filename, CGisListWks * parent);
     CGpxProject(const QString &filename, IDevice * parent);
     CGpxProject(const QString &filename, const IGisProject * project, IDevice * parent);
     virtual ~CGpxProject();
 
-    bool save();
-    bool saveAs();
+    const QString getFileDialogFilter() const override
+    {
+        return IGisProject::filedialogFilterGPX;
+    }
+
+    const QString getFileExtension() const override
+    {
+        return "gpx";
+    }
+
+    bool canSave() const override
+    {
+        return true;
+    }
 
     static bool saveAs(const QString& fn, IGisProject& project);
 
+    static void loadGpx(const QString &filename, CGpxProject *project);
+
 private:
     void loadGpx(const QString& filename);
 };
