@@ -21,10 +21,12 @@
 #include "gis/prj/CDetailsPrj.h"
 #include "gis/prj/IGisProject.h"
 #include "gis/rte/CGisItemRte.h"
+#include "gis/trk/CActivityTrk.h"
 #include "gis/trk/CGisItemTrk.h"
 #include "gis/wpt/CGisItemWpt.h"
 #include "helpers/CLinksDialog.h"
 #include "helpers/CProgressDialog.h"
+#include "helpers/Signals.h"
 #include "plot/CPlotProfile.h"
 #include "plot/CPlotTrack.h"
 #include "widgets/CTextEditWidget.h"
@@ -34,27 +36,48 @@
 
 CDetailsPrj::CDetailsPrj(IGisProject &prj, QWidget *parent)
     : QWidget(parent)
+    , INotifyTrk(CGisItemTrk::eVisualProject)
     , prj(prj)
 {
     setupUi(this);
 
-    connect(labelKeywords, SIGNAL(linkActivated(QString)), this, SLOT(slotLinkActivated(QString)));
-    connect(textDesc, SIGNAL(anchorClicked(QUrl)), this, SLOT(slotLinkActivated(QUrl)));
-    connect(toolPrint, SIGNAL(clicked()), this, SLOT(slotPrint()));
-    connect(toolReload, SIGNAL(clicked()), this, SLOT(slotSetupGui()));
-    connect(comboSort, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSortMode(int)));
-    connect(toolLock, SIGNAL(clicked(bool)), this, SLOT(slotLock(bool)));
+    const int N = prj.childCount();
+    for(int i = 0; i < N; i++)
+    {
+        CGisItemTrk *trk = dynamic_cast<CGisItemTrk*>(prj.child(i));
+        if(nullptr != trk)
+        {
+            trk->registerVisual(this);
+        }
+    }
+
+    connect(labelKeywords, &QLabel::linkActivated,          this, static_cast<void (CDetailsPrj::*)(const QString&)>(&CDetailsPrj::slotLinkActivated));
+    connect(textDesc,      &QTextBrowser::anchorClicked,    this, static_cast<void (CDetailsPrj::*)(const QUrl&)   >(&CDetailsPrj::slotLinkActivated));
+    connect(toolPrint,     &QToolButton::clicked,           this, &CDetailsPrj::slotPrint);
+    connect(toolReload,    &QToolButton::clicked,           this, &CDetailsPrj::slotSetupGui);
+    connect(toolLock,      &QToolButton::clicked,           this, &CDetailsPrj::slotLock);
+
+    connect(comboSort,     static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &CDetailsPrj::slotSortMode);
 
     timerUpdateTime = new QTimer(this);
     timerUpdateTime->setSingleShot(true);
     timerUpdateTime->setInterval(20);
-    connect(timerUpdateTime, SIGNAL(timeout()), this, SLOT(slotSetupGui()));
+    connect(timerUpdateTime, &QTimer::timeout, this, &CDetailsPrj::slotSetupGui);
 
     timerUpdateTime->start();
 }
 
 CDetailsPrj::~CDetailsPrj()
 {
+    const int N = prj.childCount();
+    for(int i = 0; i < N; i++)
+    {
+        CGisItemTrk *trk = dynamic_cast<CGisItemTrk*>(prj.child(i));
+        if(nullptr != trk)
+        {
+            trk->unregisterVisual(this);
+        }
+    }
 }
 
 void CDetailsPrj::resizeEvent(QResizeEvent * e)
@@ -65,7 +88,7 @@ void CDetailsPrj::resizeEvent(QResizeEvent * e)
 
 void CDetailsPrj::getTrackProfile(CGisItemTrk * trk, QImage& image)
 {
-    CPlotProfile plot(trk, IPlot::eModeIcon, this);
+    CPlotProfile plot(trk, trk->limitsGraph1, IPlot::eModeIcon, this);
     plot.setSolid(true);
     plot.save(image);
 }
@@ -94,10 +117,12 @@ void CDetailsPrj::slotSetupGui()
         return;
     }
 
-    comboSort->blockSignals(true);
+    X______________BlockAllSignals______________X(this);
     comboSort->setCurrentIndex(prj.getSorting());
     if((prj.getSorting() > IGisProject::eSortTime) && !prj.doCorrelation())
     {
+        X_____________UnBlockAllSignals_____________X(this);
+
         QString msg = tr("You want to sort waypoints along a track, but you switched off track and waypoint correlation. Do you want to switch it on again?");
         int res = QMessageBox::question(this, tr("Correlation..."), msg, QMessageBox::Yes|QMessageBox::No, QMessageBox::Yes);
         if(res == QMessageBox::Yes)
@@ -109,12 +134,11 @@ void CDetailsPrj::slotSetupGui()
             comboSort->setCurrentIndex(IGisProject::eSortNone);
         }
         timerUpdateTime->start();
+
         mutex.unlock();
         return;
     }
-    comboSort->blockSignals(false);
 
-    toolLock->blockSignals(true);
     const int N = prj.childCount();
     if(N == 0)
     {
@@ -135,22 +159,21 @@ void CDetailsPrj::slotSetupGui()
             }
         }
     }
-    toolLock->blockSignals(false);
+    X_____________UnBlockAllSignals_____________X(this);
 
     textDesc->document()->setTextWidth(textDesc->size().width() - 20);
     draw(*textDesc->document(), false);
 
-    QTabWidget * tabWidget = dynamic_cast<QTabWidget*>(parentWidget() ? parentWidget()->parentWidget() : 0);
+    QTabWidget * tabWidget = dynamic_cast<QTabWidget*>(parentWidget() ? parentWidget()->parentWidget() : nullptr);
     if(tabWidget)
     {
         int idx = tabWidget->indexOf(this);
         if(idx != NOIDX)
         {
             setObjectName(prj.getName());
-            tabWidget->setTabText(idx, prj.getName());
+            tabWidget->setTabText(idx, prj.getName().replace("&", "&&"));
         }
     }
-
     mutex.unlock();
 }
 
@@ -224,8 +247,8 @@ void CDetailsPrj::draw(QTextDocument& doc, bool printable)
     fmtTableHidden.setBottomMargin(0);
 
     QVector<QTextLength> constraints2;
-    constraints2 << QTextLength(QTextLength::PercentageLength, 70);
-    constraints2 << QTextLength(QTextLength::PercentageLength, 30);
+    constraints2 << QTextLength(QTextLength::PercentageLength, 50);
+    constraints2 << QTextLength(QTextLength::PercentageLength, 50);
     fmtTableHidden.setColumnWidthConstraints(constraints2);
 
     fmtTableInfo.setBorder(0);
@@ -264,32 +287,32 @@ void CDetailsPrj::draw(QTextDocument& doc, bool printable)
     const int N = prj.childCount();
     for(int i = 0; i < N; i++)
     {
-        CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(prj.child(i));
-        if(trk != 0)
+        CGisItemTrk *trk = dynamic_cast<CGisItemTrk*>(prj.child(i));
+        if(nullptr != trk)
         {
             trks << trk;
             nItems++;
             continue;
         }
 
-        CGisItemRte * rte = dynamic_cast<CGisItemRte*>(prj.child(i));
-        if(rte != 0)
+        CGisItemRte *rte = dynamic_cast<CGisItemRte*>(prj.child(i));
+        if(nullptr != rte)
         {
             rtes << rte;
             nItems++;
             continue;
         }
 
-        CGisItemWpt * wpt = dynamic_cast<CGisItemWpt*>(prj.child(i));
-        if(wpt != 0)
+        CGisItemWpt *wpt = dynamic_cast<CGisItemWpt*>(prj.child(i));
+        if(nullptr != wpt)
         {
             wpts << wpt;
             nItems++;
             continue;
         }
 
-        CGisItemOvlArea * area = dynamic_cast<CGisItemOvlArea*>(prj.child(i));
-        if(area != 0)
+        CGisItemOvlArea *area = dynamic_cast<CGisItemOvlArea*>(prj.child(i));
+        if(nullptr != area)
         {
             areas << area;
             nItems++;
@@ -356,7 +379,7 @@ void CDetailsPrj::drawTrackSummary(QTextCursor& cursor, const QList<CGisItemTrk*
 {
     quint32 flags = 0;
     QVector<CActivityTrk::activity_summary_t> summaries(CGisItemTrk::trkpt_t::eActMaxNum + 1);
-    foreach(const CGisItemTrk* trk, trks)
+    for(const CGisItemTrk* trk : trks)
     {
         const CActivityTrk& activities = trk->getActivities();
         flags |= activities.getAllFlags();
@@ -379,6 +402,34 @@ void CDetailsPrj::drawTrackSummary(QTextCursor& cursor, const QList<CGisItemTrk*
     cursor1.insertHtml(str);
 }
 
+void CDetailsPrj::addIcon(QTextTable * table, int col, int row, IGisItem * item, bool printable)
+{
+    table->cellAt(row,col).firstCursorPosition().insertImage(item->getIcon().toImage().scaledToWidth(16, Qt::SmoothTransformation));
+
+    CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(item);
+    if(trk)
+    {
+        QSet<QString> icons;
+        for(const CActivityTrk::activity_range_t& range : trk->getActivities().getActivityRanges())
+        {
+            icons << range.icon;
+        }
+
+        for(const QString &icon : icons)
+        {
+            if(!icon.isEmpty())
+            {
+                table->cellAt(row,col).lastCursorPosition().insertHtml(QString("<p><br/><img src='%1'/></p>").arg(icon));
+            }
+        }
+    }
+
+    if(!(printable||item->isReadOnly()))
+    {
+        table->cellAt(row,col).lastCursorPosition().insertHtml(QString("<p><a href='edit?key=%1'><img src='://icons/16x16/EditDetails.png'/></a></p>").arg(item->getKey().item));
+    }
+}
+
 
 void CDetailsPrj::drawByGroup(QTextCursor &cursor, QList<CGisItemTrk*>& trks, QList<CGisItemWpt*>& wpts, CProgressDialog& progress, int& n, bool printable)
 {
@@ -403,11 +454,11 @@ void CDetailsPrj::drawByGroup(QTextCursor &cursor, QList<CGisItemTrk*>& trks, QL
         table->cellAt(0,eComment1).firstCursorPosition().insertText(tr("Comment"));
 
         cnt = 1;
-        foreach(CGisItemWpt * wpt, wpts)
+        for(CGisItemWpt * wpt : wpts)
         {
             PROGRESS(n++, return );
 
-            table->cellAt(cnt,eSym1).firstCursorPosition().insertImage(wpt->getIcon().toImage().scaledToWidth(16, Qt::SmoothTransformation));
+            addIcon(table, eSym1, cnt, wpt, printable);
             table->cellAt(cnt,eInfo1).firstCursorPosition().insertHtml(wpt->getInfo());
             table->cellAt(cnt,eComment1).firstCursorPosition().insertHtml(IGisItem::createText(wpt->isReadOnly()||printable, wpt->getComment(), wpt->getDescription(), wpt->getLinks(), wpt->getKey().item));
             cnt++;
@@ -430,11 +481,11 @@ void CDetailsPrj::drawByGroup(QTextCursor &cursor, QList<CGisItemTrk*>& trks, QL
 
         cnt = 1;
 
-        foreach(CGisItemTrk * trk, trks)
+        for(CGisItemTrk * trk : trks)
         {
             PROGRESS(n++, return );
 
-            table->cellAt(cnt,eSym1).firstCursorPosition().insertImage(trk->getIcon().toImage().scaledToWidth(16, Qt::SmoothTransformation));
+            addIcon(table, eSym1, cnt, trk, printable);
 
             int w1 = qRound(w/3.5 > 300 ? 300 : w/3.5);
             int h1 = qRound(w1/2.0);
@@ -481,9 +532,17 @@ void CDetailsPrj::drawByGroup(QTextCursor &cursor, QList<CGisItemTrk*>& trks, QL
 struct wpt_info_t
 {
     IGisItem::key_t key;
-    qreal distance;
-    qreal ascend;
-    qreal descend;
+    qreal distance1 = NOFLOAT;
+    qreal ascend1 = NOFLOAT;
+    qreal descend1 = NOFLOAT;
+
+    qreal distance2 = NOFLOAT;
+    qreal ascend2 = NOFLOAT;
+    qreal descend2 = NOFLOAT;
+
+    qreal distance3 = NOFLOAT;
+    qreal ascend3 = NOFLOAT;
+    qreal descend3 = NOFLOAT;
 };
 
 void CDetailsPrj::drawByTrack(QTextCursor& cursor, QList<CGisItemTrk *> &trks, QList<CGisItemWpt *> &wpts, CProgressDialog &progress, int &n, bool printable)
@@ -499,13 +558,15 @@ void CDetailsPrj::drawByTrack(QTextCursor& cursor, QList<CGisItemTrk *> &trks, Q
     const qreal h1 = qRound(w1/2.0);
 
 
-    foreach(CGisItemTrk * trk, trks)
+    for(CGisItemTrk * trk : trks)
     {
+        const CGisItemTrk::trkpt_t* lastTrkpt = nullptr;
+        wpt_info_t * lastWptInfo = nullptr;
         QList<wpt_info_t> wptInfo;
         const CGisItemTrk::trk_t& t = trk->getTrackData();
-        foreach (const CGisItemTrk::trkseg_t& seg, t.segs)
+        for(const CGisItemTrk::trkseg_t& seg : t.segs)
         {
-            foreach(const CGisItemTrk::trkpt_t& trkpt, seg.pts)
+            for(const CGisItemTrk::trkpt_t& trkpt : seg.pts)
             {
                 if((trkpt.flags & CGisItemTrk::trkpt_t::eHidden) || trkpt.keyWpt.item.isEmpty())
                 {
@@ -515,9 +576,23 @@ void CDetailsPrj::drawByTrack(QTextCursor& cursor, QList<CGisItemTrk *> &trks, Q
                 wptInfo << wpt_info_t();
                 wpt_info_t& info = wptInfo.last();
                 info.key        = trkpt.keyWpt;
-                info.distance   = trkpt.distance;
-                info.ascend     = trkpt.ascend;
-                info.descend    = trkpt.descend;
+                info.distance1  = trkpt.distance;
+                info.ascend1    = trkpt.ascend;
+                info.descend1   = trkpt.descend;
+
+                if(lastWptInfo != nullptr)
+                {
+                    lastWptInfo->distance2  = trkpt.distance - lastTrkpt->distance;
+                    lastWptInfo->ascend2    = trkpt.ascend   - lastTrkpt->ascend;
+                    lastWptInfo->descend2   = trkpt.descend  - lastTrkpt->descend;
+                }
+
+                info.distance3  = trk->getTotalDistance() - trkpt.distance;
+                info.ascend3    = trk->getTotalAscend() - trkpt.ascend;
+                info.descend3   = trk->getTotalDescend() - trkpt.descend;
+
+                lastTrkpt       = &trkpt;
+                lastWptInfo     = &wptInfo.last();
             }
         }
 
@@ -534,25 +609,53 @@ void CDetailsPrj::drawByTrack(QTextCursor& cursor, QList<CGisItemTrk *> &trks, Q
 
         cnt = 1;
 
-        foreach(const wpt_info_t &info, wptInfo)
+        for(const wpt_info_t &info : wptInfo)
         {
             PROGRESS(n++, return );
 
             CGisItemWpt * wpt = dynamic_cast<CGisItemWpt*>(prj.getItemByKey(info.key));
-            if(wpt != 0)
+            if(wpt != nullptr)
             {
-                table->cellAt(cnt,eSym2).firstCursorPosition().insertImage(wpt->getIcon().toImage().scaledToWidth(16, Qt::SmoothTransformation));
+                addIcon(table, eSym2, cnt, wpt, printable);
                 table->cellAt(cnt,eInfo2).firstCursorPosition().insertHtml(wpt->getInfo());
 
                 QTextTable * table1 = table->cellAt(cnt,eData2).lastCursorPosition().insertTable(1, 2, fmtTableInfo);
 
                 QString text, val, unit;
-                IUnit::self().meter2distance(info.distance, val, unit);
-                text += "<div>"+ tr("distance: %1%2").arg(val).arg(unit) + "</div>";
-                IUnit::self().meter2elevation(info.ascend, val, unit);
-                text += "<div>"+ tr("ascent: %1%2").arg(val).arg(unit) + "</div>";
-                IUnit::self().meter2elevation(info.descend, val, unit);
-                text += "<div>"+ tr("descend: %1%2").arg(val).arg(unit) + "</div>";
+                text += "<table sytle='border=1px;'>";
+                text += "<tr><td></td><td><nobr>&nbsp;" + tr("From Start") + "&nbsp;</nobr></td><td><nobr>&nbsp;" + tr("To Next") + "&nbsp;</nobr></td><td><nobr>&nbsp;" + tr("To End")  + "&nbsp;</nobr></td></tr>";
+
+                text += "<tr>";
+                text += "<td>" + tr("distance: ") + "</td>";
+                IUnit::self().meter2distance(info.distance1, val, unit);
+                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+                IUnit::self().meter2distance(info.distance2, val, unit);
+                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+                IUnit::self().meter2distance(info.distance3, val, unit);
+                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+                text += "</tr>";
+
+                text += "<tr>";
+                text += "<td>" + tr("ascent: ") + "</td>";
+                IUnit::self().meter2elevation(info.ascend1, val, unit);
+                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+                IUnit::self().meter2elevation(info.ascend2, val, unit);
+                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+                IUnit::self().meter2elevation(info.ascend3, val, unit);
+                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+                text += "</tr>";
+
+                text += "<tr>";
+                text += "<td>" + tr("descend: ") + "</td>";
+                IUnit::self().meter2elevation(info.descend1, val, unit);
+                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+                IUnit::self().meter2elevation(info.descend2, val, unit);
+                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+                IUnit::self().meter2elevation(info.descend3, val, unit);
+                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+                text += "</tr>";
+
+                text += "</table>";
 
                 table1->cellAt(0,0).firstCursorPosition().insertHtml(text);
 
@@ -587,7 +690,7 @@ void CDetailsPrj::drawByTrack(QTextCursor& cursor, QList<CGisItemTrk *> &trks, Q
             cnt++;
         }
 
-        table->cellAt(cnt,eSym2).firstCursorPosition().insertImage(trk->getIcon().toImage().scaledToWidth(16, Qt::SmoothTransformation));
+        addIcon(table, eSym1, cnt, trk, printable);
         table->cellAt(cnt,eInfo2).firstCursorPosition().insertHtml(trk->getInfo());
 
         QTextTable * table1 = table->cellAt(cnt,eData2).lastCursorPosition().insertTable(1, 2, fmtTableInfo);
@@ -623,11 +726,11 @@ void CDetailsPrj::drawArea(QTextCursor& cursor, QList<CGisItemOvlArea *> &areas,
     table->cellAt(0,eComment1).firstCursorPosition().insertText(tr("Comment"));
 
     int cnt = 1;
-    foreach(CGisItemOvlArea * area, areas)
+    for(CGisItemOvlArea * area : areas)
     {
         PROGRESS(n++, return );
 
-        table->cellAt(cnt,eSym1).firstCursorPosition().insertImage(area->getIcon().toImage().scaledToWidth(16, Qt::SmoothTransformation));
+        addIcon(table, eSym1, cnt, area, printable);
         table->cellAt(cnt,eInfo1).firstCursorPosition().insertHtml(area->getInfo());
         table->cellAt(cnt,eComment1).firstCursorPosition().insertHtml(IGisItem::createText(area->isReadOnly()||printable, area->getComment(), area->getDescription(), area->getLinks(), area->getKey().item));
         cnt++;
@@ -653,11 +756,11 @@ void CDetailsPrj::drawRoute(QTextCursor& cursor, QList<CGisItemRte *> &rtes, CPr
     table->cellAt(0,eComment1).firstCursorPosition().insertText(tr("Comment"));
 
     int cnt = 1;
-    foreach(CGisItemRte * rte, rtes)
+    for(CGisItemRte * rte : rtes)
     {
         PROGRESS(n++, return );
 
-        table->cellAt(cnt,eSym1).firstCursorPosition().insertImage(rte->getIcon().toImage().scaledToWidth(16, Qt::SmoothTransformation));
+        addIcon(table, eSym1, cnt, rte, printable);
         table->cellAt(cnt,eInfo1).firstCursorPosition().insertHtml(rte->getInfo());
         table->cellAt(cnt,eComment1).firstCursorPosition().insertHtml(IGisItem::createText(rte->isReadOnly()||printable, rte->getComment(), rte->getDescription(), rte->getLinks(), rte->getKey().item));
         cnt++;
@@ -702,7 +805,6 @@ void CDetailsPrj::slotLinkActivated(const QUrl& url)
         {
             prj.setName(name);
         }
-        slotSetupGui();
     }
     else if(url.path() == "description")
     {
@@ -720,8 +822,7 @@ void CDetailsPrj::slotLinkActivated(const QUrl& url)
             IGisItem * item = prj.getItemByKey(key);
             if(item)
             {
-                CTextEditWidget dlg(this);
-                dlg.setHtml(item->getDescription());
+                CTextEditWidget dlg(item->getDescription(), this);
                 if(dlg.exec() == QDialog::Accepted)
                 {
                     item->setDescription(dlg.getHtml());
@@ -730,14 +831,12 @@ void CDetailsPrj::slotLinkActivated(const QUrl& url)
         }
         else
         {
-            CTextEditWidget dlg(0);
-            dlg.setHtml(prj.getDescription());
+            CTextEditWidget dlg(prj.getDescription(), nullptr);
             if(dlg.exec() == QDialog::Accepted)
             {
                 prj.setDescription(dlg.getHtml());
             }
         }
-        slotSetupGui();
     }
     else if(url.path() == "comment")
     {
@@ -755,15 +854,13 @@ void CDetailsPrj::slotLinkActivated(const QUrl& url)
             IGisItem * item = prj.getItemByKey(key);
             if(item)
             {
-                CTextEditWidget dlg(this);
-                dlg.setHtml(item->getComment());
+                CTextEditWidget dlg(item->getComment(), this);
                 if(dlg.exec() == QDialog::Accepted)
                 {
                     item->setComment(dlg.getHtml());
                 }
             }
         }
-        slotSetupGui();
     }
     else if(url.path() == "links")
     {
@@ -798,12 +895,30 @@ void CDetailsPrj::slotLinkActivated(const QUrl& url)
                 prj.setLinks(links);
             }
         }
-        slotSetupGui();
+    }
+    else if(url.path() == "edit")
+    {
+        IGisItem::key_t key;
+        key.project = prj.getKey();
+
+        QString query = url.query();
+        if(query.startsWith("key="))
+        {
+            key.item = query.mid(4);
+        }
+
+        IGisItem * item = prj.getItemByKey(key);
+        if(item)
+        {
+            item->edit();
+        }
     }
     else
     {
         QDesktopServices::openUrl(url);
+        return;
     }
+    slotSetupGui();
 }
 
 void CDetailsPrj::slotPrint()
@@ -851,3 +966,11 @@ void CDetailsPrj::slotSortMode(int idx)
     prj.setSorting(IGisProject::sorting_e(idx));
     slotSetupGui();
 }
+
+void CDetailsPrj::updateData()
+{
+    if(!prj.blockUpdateItems())
+    {
+        slotSetupGui();
+    }
+}
