@@ -23,18 +23,20 @@
 #include "gis/rte/CGisItemRte.h"
 #include "gis/trk/CGisItemTrk.h"
 #include "gis/wpt/CGisItemWpt.h"
+#include "helpers/CLimit.h"
+#include "helpers/CValue.h"
 
 #include <QtWidgets>
 
-#define VER_TRK         quint8(2)
+#define VER_TRK         quint8(5)
 #define VER_WPT         quint8(2)
-#define VER_RTE         quint8(2)
+#define VER_RTE         quint8(3)
 #define VER_AREA        quint8(1)
 #define VER_LINK        quint8(1)
 #define VER_TRKSEG      quint8(1)
 #define VER_TRKPT       quint8(2)
 #define VER_RTEPT       quint8(2)
-#define VER_RTESUBPT    quint8(1)
+#define VER_RTESUBPT    quint8(2)
 #define VER_WPT_T       quint8(1)
 #define VER_GC_T        quint8(1)
 #define VER_GCLOG_T     quint8(1)
@@ -43,8 +45,10 @@
 #define VER_COPYRIGHT   quint8(1)
 #define VER_PERSON      quint8(1)
 #define VER_HIST        quint8(1)
-#define VER_HIST_EVT    quint8(2)
-#define VER_ITEM        quint8(2)
+#define VER_HIST_EVT    quint8(3)
+#define VER_ITEM        quint8(3)
+#define VER_CVALUE      quint8(1)
+#define VER_CLIMIT      quint8(1)
 
 #define MAGIC_SIZE      10
 #define MAGIC_TRK       "QMTrk     "
@@ -129,6 +133,8 @@ QDataStream& operator<<(QDataStream& stream, const IGisItem::history_event_t& e)
     stream << e.comment;
     stream << e.data;
     stream << e.hash;
+    stream << e.who;
+
     return stream;
 }
 
@@ -144,6 +150,10 @@ QDataStream& operator>>(QDataStream& stream, IGisItem::history_event_t& e)
     {
         stream >> e.hash;
     }
+    if(version > 2)
+    {
+        stream >> e.who;
+    }
 
     return stream;
 }
@@ -369,6 +379,7 @@ QDataStream& operator<<(QDataStream& stream, const CGisItemRte::subpt_t& pt)
     stream << pt.instruction;
     stream << pt.distance;
     stream << pt.time;
+    stream << pt.ele;
 
     return stream;
 }
@@ -388,7 +399,16 @@ QDataStream& operator>>(QDataStream& stream, CGisItemRte::subpt_t& pt)
 
     stream >> pt.instruction;
     stream >> pt.distance;
-    stream >> pt.time;
+    if(version < 2)
+    {
+        quint32 time;
+        stream >> time;
+    }
+    else
+    {
+        stream >> pt.time;
+        stream >> pt.ele;
+    }
 
     return stream;
 }
@@ -442,6 +462,39 @@ QDataStream& operator>>(QDataStream& stream, IGisProject::person_t& p)
     return stream;
 }
 
+QDataStream& operator<<(QDataStream& stream, const CValue& v)
+{
+    stream << VER_CVALUE << quint8(v.mode) << v.valUser;
+    return stream;
+}
+
+QDataStream& operator>>(QDataStream& stream, CValue& v)
+{
+    quint8 version, mode;
+    stream >> version >> mode >> v.valUser;
+    // lame trick to update object on mode correctly without
+    // triggering a changed mark
+    v.mode = CValue::mode_e(mode);
+    v.setMode(CValue::mode_e(mode));
+    return stream;
+}
+
+QDataStream& operator<<(QDataStream& stream, const CLimit& l)
+{
+    stream << VER_CLIMIT << quint8(l.mode) << l.source << l.minUser << l.maxUser;
+    return stream;
+}
+
+QDataStream& operator>>(QDataStream& stream, CLimit& l)
+{
+    quint8 version, mode;
+
+    stream >> version >> mode >> l.source >> l.minUser >> l.maxUser;
+    l.mode = CLimit::mode_e(mode);
+
+    return stream;
+}
+
 
 // ---------------- main objects ---------------------------------
 
@@ -463,10 +516,13 @@ QDataStream& CGisItemTrk::operator>>(QDataStream& stream) const
     out << trk.type;
     out << trk.color;
 
-    out << colorSource;
-    out << limitLow;
-    out << limitHigh;
+    out << colorSourceLimit;
+    out << lineScale;
+    out << showArrows;
 
+    out << limitsGraph1;
+    out << limitsGraph2;
+    out << limitsGraph3;
     out << trk.segs;
 
     stream.writeRawData(MAGIC_TRK, MAGIC_SIZE);
@@ -510,13 +566,36 @@ QDataStream& CGisItemTrk::operator<<(QDataStream& stream)
     in >> trk.type;
     in >> trk.color;
 
-    // versions >= 2 contain colorized tracks
-    if(version >= 2)
+    if(version > 1 && version <= 4)
     {
-        QString source;
+        QString colorSource;
         in >> colorSource;
+
+        qreal limitLow, limitHigh;
         in >> limitLow;
         in >> limitHigh;
+
+        colorSourceLimit.source     = colorSource;
+        colorSourceLimit.mode       = CLimit::eModeAuto;
+        colorSourceLimit.minUser    = limitLow;
+        colorSourceLimit.maxUser    = limitHigh;
+    }
+    else if(version > 4)
+    {
+        in >> colorSourceLimit;
+    }
+
+    if(version > 2)
+    {
+        in >> lineScale;
+        in >> showArrows;
+    }
+
+    if(version > 3)
+    {
+        in >> limitsGraph1;
+        in >> limitsGraph2;
+        in >> limitsGraph3;
     }
 
     trk.segs.clear();
@@ -524,11 +603,8 @@ QDataStream& CGisItemTrk::operator<<(QDataStream& stream)
 
     deriveSecondaryData();
     setColor(str2color(trk.color));
-    setText(CGisListWks::eColumnName, trk.name);
+    setText(   CGisListWks::eColumnName, getName());
     setToolTip(CGisListWks::eColumnName, getInfo());
-
-    notifyChange();
-
     return stream;
 }
 
@@ -569,7 +645,7 @@ QDataStream& CGisItemWpt::operator<<(QDataStream& stream)
     }
 
     setIcon();
-    setText(CGisListWks::eColumnName, wpt.name);
+    setText   (CGisListWks::eColumnName, getName());
     setToolTip(CGisListWks::eColumnName, getInfo());
 
     return stream;
@@ -634,16 +710,21 @@ QDataStream& CGisItemRte::operator<<(QDataStream& stream)
     in >> rte.pts;
     if(version > 1)
     {
-        in >> lastRoutedWith;
-        in >> lastRoutedTime;
-        in >> totalDistance;
-        in >> totalTime;
+        in >> rte.lastRoutedWith;
+        in >> rte.lastRoutedTime;
+        in >> rte.totalDistance;
+        in >> rte.totalTime;
+    }
+    if(version > 2)
+    {
+        in >> rte.ascend;
+        in >> rte.descend;
     }
 
 
     setSymbol();
     deriveSecondaryData();
-    setText(CGisListWks::eColumnName, rte.name);
+    setText   (CGisListWks::eColumnName, getName());
     setToolTip(CGisListWks::eColumnName, getInfo());
 
     return stream;
@@ -666,10 +747,12 @@ QDataStream& CGisItemRte::operator>>(QDataStream& stream) const
     out << rte.number;
     out << rte.type;
     out << rte.pts;
-    out << lastRoutedWith;
-    out << lastRoutedTime;
-    out << totalDistance;
-    out << totalTime;
+    out << rte.lastRoutedWith;
+    out << rte.lastRoutedTime;
+    out << rte.totalDistance;
+    out << rte.totalTime;
+    out << rte.ascend;
+    out << rte.descend;
 
     stream.writeRawData(MAGIC_RTE, MAGIC_SIZE);
     stream << VER_RTE;
@@ -721,7 +804,7 @@ QDataStream& CGisItemOvlArea::operator<<(QDataStream& stream)
     deriveSecondaryData();
 
     setColor(str2color(area.color));
-    setText(CGisListWks::eColumnName, area.name);
+    setText   (CGisListWks::eColumnName, getName());
     setToolTip(CGisListWks::eColumnName, getInfo());
 
     return stream;
@@ -802,6 +885,7 @@ QDataStream& IGisProject::operator<<(QDataStream& stream)
 
     while(!stream.atEnd())
     {
+        QString lastDatabaseHash;
         IGisItem::history_t history;
         quint8 changed = 0;
         quint8 version, type;
@@ -813,23 +897,29 @@ QDataStream& IGisProject::operator<<(QDataStream& stream)
             stream >> changed;
         }
 
-        IGisItem * item = 0;
+        if(version > 2)
+        {
+            stream >> lastDatabaseHash;
+        }
+
+
+        IGisItem *item = nullptr;
         switch(type)
         {
         case IGisItem::eTypeWpt:
-            item = new CGisItemWpt(history, this);
+            item = new CGisItemWpt(history, lastDatabaseHash, this);
             break;
 
         case IGisItem::eTypeTrk:
-            item = new CGisItemTrk(history, this);
+            item = new CGisItemTrk(history, lastDatabaseHash, this);
             break;
 
         case IGisItem::eTypeRte:
-            item = new CGisItemRte(history, this);
+            item = new CGisItemRte(history, lastDatabaseHash, this);
             break;
 
         case IGisItem::eTypeOvl:
-            item = new CGisItemOvlArea(history, this);
+            item = new CGisItemOvlArea(history, lastDatabaseHash, this);
             break;
 
         default:
@@ -846,7 +936,7 @@ QDataStream& IGisProject::operator<<(QDataStream& stream)
     return stream;
 }
 
-QDataStream& IGisProject::operator>>(QDataStream& stream)
+QDataStream& IGisProject::operator>>(QDataStream& stream) const
 {
     stream.writeRawData(MAGIC_PROJ, MAGIC_SIZE);
     stream << VER_PROJECT;
@@ -867,7 +957,7 @@ QDataStream& IGisProject::operator>>(QDataStream& stream)
     for(int i = 0; i < childCount(); i++)
     {
         CGisItemTrk * item = dynamic_cast<CGisItemTrk*>(child(i));
-        if(item == 0)
+        if(nullptr == item)
         {
             continue;
         }
@@ -875,11 +965,12 @@ QDataStream& IGisProject::operator>>(QDataStream& stream)
         stream << quint8(item->type());
         stream << item->getHistory();
         stream << quint8(item->data(1,Qt::UserRole).toUInt() & IGisItem::eMarkChanged);
+        stream << item->getLastDatabaseHash();
     }
     for(int i = 0; i < childCount(); i++)
     {
         CGisItemRte * item = dynamic_cast<CGisItemRte*>(child(i));
-        if(item == 0)
+        if(nullptr == item)
         {
             continue;
         }
@@ -887,11 +978,12 @@ QDataStream& IGisProject::operator>>(QDataStream& stream)
         stream << quint8(item->type());
         stream << item->getHistory();
         stream << quint8(item->data(1,Qt::UserRole).toUInt() & IGisItem::eMarkChanged);
+        stream << item->getLastDatabaseHash();
     }
     for(int i = 0; i < childCount(); i++)
     {
         CGisItemWpt * item = dynamic_cast<CGisItemWpt*>(child(i));
-        if(item == 0)
+        if(nullptr == item)
         {
             continue;
         }
@@ -899,11 +991,12 @@ QDataStream& IGisProject::operator>>(QDataStream& stream)
         stream << quint8(item->type());
         stream << item->getHistory();
         stream << quint8(item->data(1,Qt::UserRole).toUInt() & IGisItem::eMarkChanged);
+        stream << item->getLastDatabaseHash();
     }
     for(int i = 0; i < childCount(); i++)
     {
         CGisItemOvlArea * item = dynamic_cast<CGisItemOvlArea*>(child(i));
-        if(item == 0)
+        if(nullptr == item)
         {
             continue;
         }
@@ -911,6 +1004,7 @@ QDataStream& IGisProject::operator>>(QDataStream& stream)
         stream << quint8(item->type());
         stream << item->getHistory();
         stream << quint8(item->data(1,Qt::UserRole).toUInt() & IGisItem::eMarkChanged);
+        stream << item->getLastDatabaseHash();
     }
 
     return stream;
@@ -961,7 +1055,7 @@ QDataStream& CDBProject::operator<<(QDataStream& stream)
     return stream;
 }
 
-QDataStream& CDBProject::operator>>(QDataStream& stream)
+QDataStream& CDBProject::operator>>(QDataStream& stream) const
 {
     stream.writeRawData(MAGIC_PROJ, MAGIC_SIZE);
     stream << VER_PROJECT;
