@@ -34,10 +34,10 @@ CCreateRouteFromWpt::CCreateRouteFromWpt(const QList<IGisItem::key_t> &keys, QWi
 {
     setupUi(this);
 
-    foreach(const IGisItem::key_t& key, keys)
+    for(const IGisItem::key_t& key : keys)
     {
-        CGisItemWpt * wpt = dynamic_cast<CGisItemWpt*>(CGisWidget::self().getItemByKey(key));
-        if(wpt == 0)
+        CGisItemWpt *wpt = dynamic_cast<CGisItemWpt*>(CGisWidget::self().getItemByKey(key));
+        if(nullptr == wpt)
         {
             continue;
         }
@@ -49,9 +49,9 @@ CCreateRouteFromWpt::CCreateRouteFromWpt(const QList<IGisItem::key_t> &keys, QWi
         item->setData(Qt::UserRole + 0, QPointF(wpt->getPosition()*DEG_TO_RAD));
     }
 
-    connect(listWidget, SIGNAL(itemSelectionChanged()), this, SLOT(slotSelectionChanged()));
-    connect(toolUp, SIGNAL(clicked()), this, SLOT(slotUp()));
-    connect(toolDown, SIGNAL(clicked()), this, SLOT(slotDown()));
+    connect(listWidget, &QListWidget::itemSelectionChanged, this, &CCreateRouteFromWpt::slotSelectionChanged);
+    connect(toolUp,     &QToolButton::clicked,              this, &CCreateRouteFromWpt::slotUp);
+    connect(toolDown,   &QToolButton::clicked,              this, &CCreateRouteFromWpt::slotDown);
 }
 
 CCreateRouteFromWpt::~CCreateRouteFromWpt()
@@ -62,14 +62,10 @@ void CCreateRouteFromWpt::accept()
 {
     QDialog::accept();
 
-    QString name = QInputDialog::getText(CMainWindow::getBestWidgetForParent(), QObject::tr("Edit name..."), QObject::tr("Enter new route name."), QLineEdit::Normal, "");
-    if(name.isEmpty())
-    {
-        return;
-    }
+    QString name;
+    IGisProject *project = nullptr;
 
-    IGisProject * project = CGisWidget::self().selectProject();
-    if(project == 0)
+    if(!IGisItem::getNameAndProject(name, project, tr("route")))
     {
         return;
     }
@@ -88,7 +84,7 @@ void CCreateRouteFromWpt::accept()
 void CCreateRouteFromWpt::slotSelectionChanged()
 {
     QListWidgetItem * item = listWidget->currentItem();
-    if(item != 0)
+    if(item != nullptr)
     {
         int row = listWidget->row(item);
         toolUp->setEnabled(row != 0);
