@@ -27,7 +27,9 @@
 #include "gis/rte/CDetailsRte.h"
 #include "gis/rte/CGisItemRte.h"
 #include "gis/rte/CScrOptRte.h"
+#include "gis/trk/CGisItemTrk.h"
 #include "helpers/CDraw.h"
+#include "units/IUnit.h"
 
 #include <QtWidgets>
 #include <QtXml>
@@ -60,7 +62,7 @@ CGisItemRte::CGisItemRte(const CGisItemRte& parentRte, IGisProject * project, in
 
     if(clone)
     {
-        rte.name += QObject::tr("_Clone");
+        rte.name += tr("_Clone");
         key.clear();
         history.events.clear();
     }
@@ -98,12 +100,16 @@ CGisItemRte::CGisItemRte(const QDomNode& xml, IGisProject *parent)
     updateDecoration(eMarkNone, eMarkNone);
 }
 
-CGisItemRte::CGisItemRte(const history_t& hist, IGisProject * project)
+CGisItemRte::CGisItemRte(const history_t& hist, const QString &dbHash, IGisProject * project)
     : IGisItem(project, eTypeRte, project->childCount())
 {
     history = hist;
     loadHistory(hist.histIdxCurrent);
     deriveSecondaryData();
+    if(!dbHash.isEmpty())
+    {
+        lastDatabaseHash = dbHash;
+    }
 }
 
 CGisItemRte::CGisItemRte(quint64 id, QSqlDatabase& db, IGisProject * project)
@@ -124,6 +130,18 @@ CGisItemRte::CGisItemRte(const SGisLine &l, const QString &name, IGisProject *pr
     updateDecoration(eMarkChanged, eMarkNone);
 }
 
+CGisItemRte::CGisItemRte(CFitStream& stream, IGisProject * project)
+    : IGisItem(project, eTypeRte, project->childCount())
+{
+    // --- start read and process data ----
+    readRteFromFit(stream);
+    // --- stop read and process data ----
+
+    setupHistory();
+    deriveSecondaryData();
+    updateDecoration(eMarkNone, eMarkNone);
+}
+
 CGisItemRte::~CGisItemRte()
 {
     // reset user focus if focused on this track
@@ -133,10 +151,22 @@ CGisItemRte::~CGisItemRte()
     }
 }
 
+IGisItem * CGisItemRte::createClone()
+{
+    int idx = -1;
+    IGisProject * project = getParentProject();
+    if(project)
+    {
+        idx = project->indexOfChild(this);
+    }
+    return new CGisItemRte(*this, project, idx, true);
+}
+
+
 bool CGisItemRte::isCalculated()
 {
     bool yes = true;
-    foreach(const rtept_t &pt, rte.pts)
+    for(const rtept_t &pt : rte.pts)
     {
         if((pt.fakeSubpt.lat == NOFLOAT) || (pt.fakeSubpt.lon == NOFLOAT))
         {
@@ -147,58 +177,95 @@ bool CGisItemRte::isCalculated()
     return yes;
 }
 
+void CGisItemRte::setElevation(qreal ele, subpt_t& subpt, qreal& lastEle)
+{
+    if(ele == NOFLOAT)
+    {
+        subpt.ele = NOINT;
+        return;
+    }
+
+    subpt.ele = qRound(ele);
+
+    if(lastEle != NOFLOAT)
+    {
+        qreal delta   = subpt.ele - lastEle;
+
+        if(qAbs(delta) > ASCEND_THRESHOLD)
+        {
+            if(delta > 0)
+            {
+                rte.ascend  += delta;
+            }
+            else
+            {
+                rte.descend -= delta;
+            }
+            lastEle = subpt.ele;
+        }
+    }
+    else
+    {
+        lastEle = subpt.ele;
+    }
+}
+
 void CGisItemRte::deriveSecondaryData()
 {
+    QPolygonF pos;
+    QPolygonF ele;
     qreal north = -90;
     qreal east  = -180;
     qreal south =  90;
     qreal west  =  180;
 
-    //foreach(const rtept_t &rtept, rte.pts)
-    const int N = rte.pts.count();
-
-    for(int i = 0; i < N; i++)
+    for(rtept_t &rtept : rte.pts)
     {
-        rtept_t &rtept = rte.pts[i];
-        if(rtept.lon < west)
-        {
-            west    = rtept.lon;
-        }
-        if(rtept.lon > east)
-        {
-            east    = rtept.lon;
-        }
-        if(rtept.lat < south)
-        {
-            south   = rtept.lat;
-        }
-        if(rtept.lat > north)
-        {
-            north   = rtept.lat;
-        }
+        west  = qMin(west,  rtept.lon);
+        east  = qMax(east,  rtept.lon);
+        south = qMin(south, rtept.lat);
+        north = qMax(north, rtept.lat);
 
-        foreach(const subpt_t &subpt, rtept.subpts)
+        pos << (QPointF(rtept.lon, rtept.lat) * DEG_TO_RAD);
+        rtept.ele = NOINT;
+        rtept.fakeSubpt.ele = NOINT;
+
+        for(subpt_t &subpt : rtept.subpts)
         {
-            if(subpt.lon < west)
-            {
-                west    = subpt.lon;
-            }
-            if(subpt.lon > east)
-            {
-                east    = subpt.lon;
-            }
-            if(subpt.lat < south)
-            {
-                south   = subpt.lat;
-            }
-            if(subpt.lat > north)
-            {
-                north   = subpt.lat;
-            }
+            west  = qMin(west,  subpt.lon);
+            east  = qMax(east,  subpt.lon);
+            south = qMin(south, subpt.lat);
+            north = qMax(north, subpt.lat);
+
+            pos << (QPointF(subpt.lon, subpt.lat) * DEG_TO_RAD);
+            subpt.ele = NOINT;
         }
         rtept.updateIcon();
     }
 
+    ele.resize(pos.size());
+    ele.fill(NOPOINTF);
+    CMainWindow::self().getElevationAt(pos, ele);
+
+    if(!ele.isEmpty())
+    {
+        qreal lastEle = NOFLOAT;
+        int i = 0;
+        rte.descend = 0;
+        rte.ascend = 0;
+
+        for(rtept_t &rtept : rte.pts)
+        {
+            setElevation(ele[i++].y(), rtept.fakeSubpt, lastEle);
+            rtept.ele = rtept.fakeSubpt.ele;
+
+            for(subpt_t &subpt : rtept.subpts)
+            {
+                setElevation(ele[i++].y(), subpt, lastEle);
+            }
+        }
+    }
+
     boundingRect = QRectF(QPointF(west * DEG_TO_RAD, north * DEG_TO_RAD), QPointF(east * DEG_TO_RAD,south * DEG_TO_RAD));
 }
 
@@ -208,6 +275,28 @@ void CGisItemRte::edit()
     dlg.exec();
 }
 
+void CGisItemRte::toTrack()
+{
+    QString name;
+    IGisProject * project;
+
+    if(!getNameAndProject(name, project, tr("track")))
+    {
+        return;
+    }
+
+    SGisLine line;
+    getPolylineFromData(line);
+
+    CCanvas * canvas = CMainWindow::self().getVisibleCanvas();
+    if(canvas)
+    {
+        canvas->getElevationAt(line);
+    }
+
+    new CGisItemTrk(line, name, project, -1);
+}
+
 void CGisItemRte::setSymbol()
 {
     icon = QPixmap("://icons/32x32/Route.png").scaled(22,22, Qt::KeepAspectRatio, Qt::SmoothTransformation);
@@ -218,71 +307,76 @@ void CGisItemRte::setName(const QString& str)
 {
     setText(CGisListWks::eColumnName, str);
     rte.name = str;
-    changed(QObject::tr("Changed name."), "://icons/48x48/EditText.png");
+    changed(tr("Changed name."), "://icons/48x48/EditText.png");
 }
 
 void CGisItemRte::setComment(const QString& str)
 {
     rte.cmt = str;
-    changed(QObject::tr("Changed comment"), "://icons/48x48/EditText.png");
+    changed(tr("Changed comment"), "://icons/48x48/EditText.png");
 }
 
 void CGisItemRte::setDescription(const QString& str)
 {
     rte.desc = str;
-    changed(QObject::tr("Changed description"), "://icons/48x48/EditText.png");
+    changed(tr("Changed description"), "://icons/48x48/EditText.png");
 }
 
 void CGisItemRte::setLinks(const QList<link_t>& links)
 {
     rte.links = links;
-    changed(QObject::tr("Changed links"), "://icons/48x48/Link.png");
+    changed(tr("Changed links"), "://icons/48x48/Link.png");
 }
 
 
 
-QString CGisItemRte::getInfo(bool allowEdit) const
+QString CGisItemRte::getInfo(bool showName) const
 {
     QString val1, unit1, val2, unit2;
     QString str = "<div>";
 
-    if(allowEdit)
+    if(showName)
     {
-        str += "<b>" + toLink(isReadOnly(), "name", getName(), "") + "</b>";
+        str += "<b>" + getName() + "</b><br />";
+    }
+
+    if(rte.totalDistance != NOFLOAT)
+    {
+        IUnit::self().meter2distance(rte.totalDistance, val1, unit1);
+        str += tr("Length: %1 %2").arg(val1).arg(unit1);
     }
     else
     {
-        str += "<div style='font-weight: bold;'>" + getName() + "</div>";
+        str += tr("Length: -");
     }
 
     str += "<br/>\n";
-    if(totalDistance != NOFLOAT)
+    if(rte.totalTime != 0)
     {
-        IUnit::self().meter2distance(totalDistance, val1, unit1);
-        str += QObject::tr("Length: %1 %2").arg(val1).arg(unit1);
+        IUnit::self().seconds2time(rte.totalTime, val1, unit1);
+        str += tr("Time: %1 %2").arg(val1).arg(unit1);
     }
     else
     {
-        str += QObject::tr("Length: -");
+        str += tr("Time: -");
     }
 
-    str += "<br/>\n";
-    if(totalTime != 0)
+    if((rte.ascend != NOFLOAT) && (rte.descend != NOFLOAT))
     {
-        IUnit::self().seconds2time(totalTime, val1, unit1);
-        str += QObject::tr("Time: %1 %2").arg(val1).arg(unit1);
-    }
-    else
-    {
-        str += QObject::tr("Time: -");
+        str += "<br/>\n";
+        QString val1, val2, unit1, unit2;
+        IUnit::self().meter2elevation(rte.ascend,  val1, unit1);
+        IUnit::self().meter2elevation(rte.descend, val2, unit2);
+
+        str += tr("%1%2 %3, %4%5 %6").arg(QChar(0x2197)).arg(val1).arg(unit1).arg(QChar(0x2198)).arg(val2).arg(unit2);
     }
 
-    if(!lastRoutedWith.isEmpty())
+    if(!rte.lastRoutedWith.isEmpty())
     {
         str += "<br/>\n";
-        str += QObject::tr("Last time routed:<br/>%1").arg(IUnit::datetime2string(lastRoutedTime, false, boundingRect.center()));
+        str += tr("Last time routed:<br/>%1").arg(IUnit::datetime2string(rte.lastRoutedTime, false, boundingRect.center()));
         str += "<br/>\n";
-        str += QObject::tr("with %1").arg(lastRoutedWith);
+        str += tr("with %1").arg(rte.lastRoutedWith);
     }
     return str;
 }
@@ -302,7 +396,7 @@ QPointF CGisItemRte::getPointCloseBy(const QPoint& screenPos)
 
     qint32 d    = NOINT;
     QPointF pt  = NOPOINTF;
-    foreach(const QPointF &point, line)
+    for(const QPointF &point : line)
     {
         int tmp = (screenPos - point).manhattanLength();
         if(tmp < d)
@@ -325,6 +419,12 @@ bool CGisItemRte::isCloseTo(const QPointF& pos)
     return dist < 20;
 }
 
+bool CGisItemRte::isWithin(const QRectF& area, selflags_t flags)
+{
+    return (flags & eSelectionRte) ? IGisItem::isWithin(area, flags, line) : false;
+}
+
+
 void CGisItemRte::gainUserFocus(bool yes)
 {
     keyUserFocus = yes ? key : key_t();
@@ -361,7 +461,7 @@ void CGisItemRte::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>
     QVector<QPixmap> icons;
     QVector<QPointF> focus;
 
-    foreach(const rtept_t &rtept, rte.pts)
+    for(const rtept_t &rtept : rte.pts)
     {
         QPointF pt(rtept.lon * DEG_TO_RAD, rtept.lat * DEG_TO_RAD);
 
@@ -373,7 +473,7 @@ void CGisItemRte::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>
         focus << rtept.focus;
 
         blockedAreas << QRectF(pt - rtept.focus, rtept.icon.size());
-        foreach(const subpt_t &subpt, rtept.subpts)
+        for(const subpt_t &subpt : rtept.subpts)
         {
             QPointF pt(subpt.lon * DEG_TO_RAD, subpt.lat * DEG_TO_RAD);
             gis->convertRad2Px(pt);
@@ -410,7 +510,7 @@ void CGisItemRte::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>
 
     p.setPen(hasUserFocus() ? penForegroundFocus : penForeground);
     p.setBrush(hasUserFocus() ? penForegroundFocus.color() : penForeground.color());
-    CDraw::arrows(line, extViewport, p, 10, 80);
+    CDraw::arrows(line, extViewport, p, 10, 80, 1.0);
     p.drawPolyline(line);
 
     p.setPen(Qt::NoPen);
@@ -444,6 +544,8 @@ void CGisItemRte::drawItem(QPainter& p, const QRectF& viewport, CGisDraw * gis)
         return;
     }
 
+    QDateTime startTime = rte.pts.first().fakeSubpt.time;
+
     if(hasUserFocus() && mouseMoveFocus && mouseMoveFocus->lon != NOFLOAT && mouseMoveFocus->lat != NOFLOAT)
     {
         QPointF anchor(mouseMoveFocus->lon, mouseMoveFocus->lat);
@@ -452,10 +554,10 @@ void CGisItemRte::drawItem(QPainter& p, const QRectF& viewport, CGisDraw * gis)
         p.drawEllipse(anchor, 5, 5);
 
         QString str, val, unit;
-        IUnit::self().seconds2time(mouseMoveFocus->time, val, unit);
-        str += QObject::tr("Time: %1 %2").arg(val).arg(unit) + " ";
+        IUnit::self().seconds2time((mouseMoveFocus->time.toTime_t() - startTime.toTime_t()), val, unit);
+        str += tr("Time: %1 %2").arg(val).arg(unit) + " ";
         IUnit::self().meter2distance(mouseMoveFocus->distance, val, unit);
-        str += QObject::tr("Distance: %1 %2").arg(val).arg(unit);
+        str += tr("Distance: %1 %2").arg(val).arg(unit);
         str += "\n" + mouseMoveFocus->instruction;
 
         // calculate bounding box of text
@@ -485,7 +587,7 @@ void CGisItemRte::drawLabel(QPainter& p, const QPolygonF& viewport, QList<QRectF
     }
 
 
-    foreach(const rtept_t &rtept, rte.pts)
+    for(const rtept_t &rtept : rte.pts)
     {
         QPointF pt(rtept.lon * DEG_TO_RAD, rtept.lat * DEG_TO_RAD);
 
@@ -564,26 +666,26 @@ void CGisItemRte::readRouteDataFromGisLine(const SGisLine &l)
 void CGisItemRte::setDataFromPolyline(const SGisLine &l)
 {
     QMutexLocker lock(&mutexItems);
-    mouseMoveFocus = 0;
+    mouseMoveFocus = nullptr;
 
     readRouteDataFromGisLine(l);
 
     flags |= eFlagTainted;
-    changed(QObject::tr("Changed route points."), "://icons/48x48/LineMove.png");
+    changed(tr("Changed route points."), "://icons/48x48/LineMove.png");
 }
 
 void CGisItemRte::getPolylineFromData(SGisLine& l)
 {
     QMutexLocker lock(&mutexItems);
     l.clear();
-    foreach(const rtept_t &rtept, rte.pts)
+    for(const rtept_t &rtept : rte.pts)
     {
         l << point_t(QPointF(rtept.lon * DEG_TO_RAD, rtept.lat * DEG_TO_RAD));
 
         point_t& pt = l.last();
 
         pt.subpts.clear();
-        foreach(const subpt_t &subpt, rtept.subpts)
+        for(const subpt_t &subpt : rtept.subpts)
         {
             pt.subpts << IGisLine::subpt_t(QPointF(subpt.lon * DEG_TO_RAD, subpt.lat * DEG_TO_RAD));
         }
@@ -593,7 +695,7 @@ void CGisItemRte::getPolylineFromData(SGisLine& l)
 void CGisItemRte::calc()
 {
     QMutexLocker lock(&mutexItems);
-    mouseMoveFocus = 0;
+    mouseMoveFocus = nullptr;
     for(int i = 0; i < rte.pts.size(); i++)
     {
         rte.pts[i].subpts.clear();
@@ -611,11 +713,11 @@ void CGisItemRte::reset()
         pt.fakeSubpt = subpt_t();
     }
 
-    mouseMoveFocus  = 0;
-    totalDistance   = NOFLOAT;
-    totalTime       = 0;
-    lastRoutedTime  = QDateTime();
-    lastRoutedWith  = "";
+    mouseMoveFocus  = nullptr;
+    rte.totalDistance   = NOFLOAT;
+    rte.totalTime       = 0;
+    rte.lastRoutedTime  = QDateTime();
+    rte.lastRoutedWith  = "";
 
     deriveSecondaryData();
     updateHistory();
@@ -631,7 +733,7 @@ QPointF CGisItemRte::setMouseFocusByPoint(const QPoint& pt, focusmode_e fmode, c
 {
     QMutexLocker lock(&mutexItems);
 
-    const subpt_t * newPointOfFocus = 0;
+    const subpt_t * newPointOfFocus = nullptr;
     quint32 idx = 0;
 
     if(pt != NOPOINT && GPS_Math_DistPointPolyline(line, pt) < MIN_DIST_FOCUS)
@@ -639,7 +741,7 @@ QPointF CGisItemRte::setMouseFocusByPoint(const QPoint& pt, focusmode_e fmode, c
         quint32 i = 0;
         qint32 d1 = NOINT;
 
-        foreach(const QPointF &point, line)
+        for(const QPointF &point : line)
         {
             int tmp = (pt - point).manhattanLength();
             if(tmp <= d1)
@@ -655,7 +757,7 @@ QPointF CGisItemRte::setMouseFocusByPoint(const QPoint& pt, focusmode_e fmode, c
 
     if(newPointOfFocus && (newPointOfFocus->type == subpt_t::eTypeNone))
     {
-        newPointOfFocus = 0;
+        newPointOfFocus = nullptr;
     }
 
 
@@ -667,14 +769,14 @@ QPointF CGisItemRte::setMouseFocusByPoint(const QPoint& pt, focusmode_e fmode, c
 const CGisItemRte::subpt_t * CGisItemRte::getSubPtByIndex(quint32 idx)
 {
     quint32 cnt = 0;
-    foreach(const rtept_t &rtept, rte.pts)
+    for(const rtept_t &rtept : rte.pts)
     {
         if(cnt == idx)
         {
             return &rtept.fakeSubpt;
         }
 
-        foreach(const subpt_t &subpt, rtept.subpts)
+        for(const subpt_t &subpt : rtept.subpts)
         {
             cnt++;
             if(cnt == idx)
@@ -685,7 +787,7 @@ const CGisItemRte::subpt_t * CGisItemRte::getSubPtByIndex(quint32 idx)
         cnt++;
     }
 
-    return 0;
+    return nullptr;
 }
 
 void CGisItemRte::setResult(Routino_Output * route, const QString& options)
@@ -693,7 +795,9 @@ void CGisItemRte::setResult(Routino_Output * route, const QString& options)
     QMutexLocker lock(&mutexItems);
 
     qint32 idxRtept = -1;
-    rtept_t * rtept = 0;
+    rtept_t * rtept = nullptr;
+
+    QDateTime time = QDateTime::currentDateTimeUtc();
 
     Routino_Output * next = route;
     while(next)
@@ -709,11 +813,12 @@ void CGisItemRte::setResult(Routino_Output * route, const QString& options)
             rtept->fakeSubpt.turn      = next->turn;
             rtept->fakeSubpt.bearing   = next->bearing;
             rtept->fakeSubpt.distance  = next->dist * 1000;
-            rtept->fakeSubpt.time      = next->time * 60;
+            rtept->fakeSubpt.time      = time.addSecs(next->time * 60);
+
             rtept->fakeSubpt.type      = subpt_t::eTypeWpt;
             rtept->fakeSubpt.instruction = QString(next->desc1) + ".\n" + QString(next->desc2) + ".";
         }
-        else if(rtept != 0)
+        else if(rtept != nullptr)
         {
             rtept->subpts << subpt_t();
             subpt_t& subpt  = rtept->subpts.last();
@@ -723,7 +828,7 @@ void CGisItemRte::setResult(Routino_Output * route, const QString& options)
             subpt.turn      = next->turn;
             subpt.bearing   = next->bearing;
             subpt.distance  = next->dist * 1000;
-            subpt.time      = next->time * 60;
+            subpt.time      = time.addSecs(next->time * 60);
 
             if(next->name != 0)
             {
@@ -739,16 +844,16 @@ void CGisItemRte::setResult(Routino_Output * route, const QString& options)
                 subpt.type = subpt_t::eTypeNone;
             }
 
-            totalDistance = subpt.distance;
-            totalTime     = subpt.time;
+            rte.totalDistance = subpt.distance;
+            rte.totalTime     = subpt.time.toTime_t() - time.toTime_t();
             subpt.instruction = QString(next->desc1) + ".\n" + QString(next->desc2) + ".";
         }
 
         next = next->next;
     }
 
-    lastRoutedTime = QDateTime::currentDateTimeUtc();
-    lastRoutedWith = "Routino, " + options;
+    rte.lastRoutedTime = QDateTime::currentDateTimeUtc();
+    rte.lastRoutedWith = "Routino, " + options;
 
     deriveSecondaryData();
     updateHistory();
@@ -782,12 +887,14 @@ void CGisItemRte::setResult(const QDomDocument& xml, const QString &options)
 {
     QMutexLocker lock(&mutexItems);
 
+    QDateTime localtime = QDateTime::currentDateTimeUtc();
+
     QDomElement response    = xml.firstChildElement("response");
     QDomElement route       = response.firstChildElement("route");
 
     // get time of travel
     QDomElement xmlTime     = route.firstChildElement("time");
-    totalTime = xmlTime.text().toUInt();
+    rte.totalTime = xmlTime.text().toUInt();
 
 
     // build list of maneuvers
@@ -867,7 +974,7 @@ void CGisItemRte::setResult(const QDomDocument& xml, const QString &options)
         subpt.type              = subpt_t::eTypeJunct;
         subpt.instruction       = maneuver.instruction;
 
-        subpt.time              = time;
+        subpt.time              = localtime.addSecs(time);
         time += maneuver.time;
 
         subpt.distance          = dist;
@@ -894,9 +1001,9 @@ void CGisItemRte::setResult(const QDomDocument& xml, const QString &options)
     rtept.fakeSubpt.lon = rtept.lon;
     rtept.fakeSubpt.lat = rtept.lat;
 
-    totalDistance  = dist;
-    lastRoutedTime = QDateTime::currentDateTimeUtc();
-    lastRoutedWith = "MapQuest" + options;
+    rte.totalDistance  = dist;
+    rte.lastRoutedTime = QDateTime::currentDateTimeUtc();
+    rte.lastRoutedWith = "MapQuest" + options;
 
     deriveSecondaryData();
     updateHistory();
