@@ -23,6 +23,7 @@
 #include "gis/IGisLine.h"
 #include <routino.h>
 
+#include "gis/fit/CFitStream.h"
 #include <QPen>
 #include <QPointer>
 
@@ -33,6 +34,7 @@ class CScrOptRte;
 
 class CGisItemRte : public IGisItem, public IGisLine
 {
+    Q_DECLARE_TR_FUNCTIONS(CGisItemRte)
 public:
 
     enum focusmode_e
@@ -60,7 +62,9 @@ public:
 
         QString instruction;
         qreal distance = 0;
-        quint32 time = 0;
+        QDateTime time;
+
+        qint32 ele = NOINT;
     };
 
     struct rtept_t : public wpt_t
@@ -91,33 +95,45 @@ public:
         QString type;
         QVector<rtept_t> pts;
         // -- all gpx tags - stop
+
+        QString lastRoutedWith;
+        QDateTime lastRoutedTime;
+
+        qreal totalDistance = NOFLOAT;
+        quint32 totalTime = 0;
+        qreal ascend  = NOFLOAT;
+        qreal descend = NOFLOAT;
     };
 
     CGisItemRte(const QDomNode &xml, IGisProject *parent);
     CGisItemRte(const CGisItemRte& parentRte, IGisProject *project, int idx, bool clone);
-    CGisItemRte(const history_t& hist, IGisProject * project);
+    CGisItemRte(const history_t& hist, const QString& dbHash, IGisProject * project);
     CGisItemRte(quint64 id, QSqlDatabase& db, IGisProject * project);
     CGisItemRte(const CQlgtRoute& rte1);
     CGisItemRte(const SGisLine& l, const QString &name, IGisProject *project, int idx);
+    CGisItemRte(CFitStream& stream, IGisProject * project);
     virtual ~CGisItemRte();
 
-    QDataStream& operator<<(QDataStream& stream);
-    QDataStream& operator>>(QDataStream& stream) const;
+    IGisItem * createClone() override;
 
-    const QString& getName() const
+    QDataStream& operator<<(QDataStream& stream) override;
+    QDataStream& operator>>(QDataStream& stream) const override;
+
+    const QString& getName() const override
     {
         return rte.name.isEmpty() ? noName : rte.name;
     }
 
-    QString getInfo(bool allowEdit = false) const;
-    IScrOpt * getScreenOptions(const QPoint &origin, IMouse * mouse);
-    QPointF getPointCloseBy(const QPoint& screenPos);
-    void drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>& blockedAreas, CGisDraw * gis);
-    void drawItem(QPainter& p, const QRectF& viewport, CGisDraw * gis);
-    void drawLabel(QPainter& p, const QPolygonF& viewport, QList<QRectF>& blockedAreas, const QFontMetricsF& fm, CGisDraw * gis);
-    void drawHighlight(QPainter& p);
-    void save(QDomNode& gpx);
-    bool isCloseTo(const QPointF& pos);
+    QString getInfo(bool showName = true) const override;
+    IScrOpt * getScreenOptions(const QPoint &origin, IMouse * mouse) override;
+    QPointF getPointCloseBy(const QPoint& screenPos) override;
+    void drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>& blockedAreas, CGisDraw * gis) override;
+    void drawItem(QPainter& p, const QRectF& viewport, CGisDraw * gis) override;
+    void drawLabel(QPainter& p, const QPolygonF& viewport, QList<QRectF>& blockedAreas, const QFontMetricsF& fm, CGisDraw * gis) override;
+    void drawHighlight(QPainter& p) override;
+    void save(QDomNode& gpx) override;
+    bool isCloseTo(const QPointF& pos) override;
+    bool isWithin(const QRectF& area, selflags_t flags) override;
     /**
        @brief Switch user focus on and off.
 
@@ -125,7 +141,7 @@ public:
 
        @param yes   set true to gain focus.
      */
-    void gainUserFocus(bool yes);
+    void gainUserFocus(bool yes) override;
     /**
        @brief Make sure the route has lost focus.
 
@@ -138,7 +154,7 @@ public:
 
        @return True if the route has user focus
      */
-    bool hasUserFocus() const
+    bool hasUserFocus() const override
     {
         return key == keyUserFocus;
     }
@@ -153,19 +169,19 @@ public:
         return keyUserFocus;
     }
 
-    void setDataFromPolyline(const SGisLine& l);
+    void setDataFromPolyline(const SGisLine& l) override;
 
-    void getPolylineFromData(SGisLine &l);
+    void getPolylineFromData(SGisLine &l) override;
 
-    const QString& getComment() const
+    const QString& getComment() const override
     {
         return rte.cmt;
     }
-    const QString& getDescription() const
+    const QString& getDescription() const override
     {
         return rte.desc;
     }
-    const QList<link_t>& getLinks() const
+    const QList<link_t>& getLinks() const override
     {
         return rte.links;
     }
@@ -176,15 +192,15 @@ public:
     }
 
     void setName(const QString& str);
-    void setComment(const QString& str);
-    void setDescription(const QString& str);
-    void setLinks(const QList<link_t>& links);
+    void setComment(const QString& str)       override;
+    void setDescription(const QString& str)   override;
+    void setLinks(const QList<link_t>& links) override;
 
     void calc();
-
     void reset();
+    void edit() override;
+    void toTrack();
 
-    void edit();
 
     QPointF setMouseFocusByPoint(const QPoint& pt, focusmode_e fmode, const QString &owner);
 
@@ -195,8 +211,10 @@ public:
 
 private:
     void deriveSecondaryData();
-    void setSymbol();
+    void setElevation(qreal ele, subpt_t &subpt, qreal &lastEle);
+    void setSymbol() override;
     void readRte(const QDomNode& xml, rte_t& rte);
+    void readRteFromFit(CFitStream &stream);
     void readRouteDataFromGisLine(const SGisLine &l);
     const subpt_t * getSubPtByIndex(quint32 idx);
 
@@ -209,13 +227,8 @@ private:
     rte_t rte;
     QPolygonF line;
 
-    QString lastRoutedWith;
-    QDateTime lastRoutedTime;
 
-    qreal totalDistance = NOFLOAT;
-    quint32 totalTime = 0;
-
-    const subpt_t * mouseMoveFocus = 0;
+    const subpt_t * mouseMoveFocus = nullptr;
 
     QPointer<CScrOptRte>  scrOpt;
 };
