@@ -84,7 +84,7 @@ bool CTwoNavProject::save()
         QFile fileKey(dir.absoluteFilePath(QString("%1.key").arg(getKey())));
         if(!fileKey.open(QIODevice::WriteOnly))
         {
-            QMessageBox::critical(CMainWindow::getBestWidgetForParent(), QObject::tr("Error..."), QObject::tr("Failed to open %1.").arg(fileKey.fileName()), QMessageBox::Abort);
+            QMessageBox::critical(CMainWindow::getBestWidgetForParent(), tr("Error..."), tr("Failed to open %1.").arg(fileKey.fileName()), QMessageBox::Abort);
             throw -1;
         }
         fileKey.close();
@@ -156,7 +156,7 @@ bool CTwoNavProject::saveAs()
     QString path = cfg.value("Paths/lastGisPath", QDir::homePath()).toString();
 
     QString filter = filedialogFilterGPX;
-    QString fn = QFileDialog::getSaveFileName(CMainWindow::getBestWidgetForParent(), QObject::tr("Save GIS data to..."), path, filedialogSaveFilters, &filter);
+    QString fn = QFileDialog::getSaveFileName(CMainWindow::getBestWidgetForParent(), tr("Save GIS data to..."), path, filedialogSaveFilters, &filter);
 
     if(fn.isEmpty())
     {
@@ -188,7 +188,7 @@ bool CTwoNavProject::saveWpts(QList<CGisItemWpt*>& wpts, const QString& filename
     QFile file(filename);
     if(!file.open(QIODevice::WriteOnly))
     {
-        QMessageBox::critical(CMainWindow::getBestWidgetForParent(), QObject::tr("Error..."), QObject::tr("Failed to open %1.").arg(filename), QMessageBox::Abort);
+        QMessageBox::critical(CMainWindow::getBestWidgetForParent(), tr("Error..."), tr("Failed to open %1.").arg(filename), QMessageBox::Abort);
         return false;
     }
     QTextStream out(&file);
@@ -199,7 +199,7 @@ bool CTwoNavProject::saveWpts(QList<CGisItemWpt*>& wpts, const QString& filename
     qreal west = 180.0;
     qreal east = -180.0;
 
-    foreach(CGisItemWpt * wpt, wpts)
+    for(CGisItemWpt * wpt : wpts)
     {
         QPointF pt = wpt->getPosition();
 
@@ -227,7 +227,7 @@ bool CTwoNavProject::saveWpts(QList<CGisItemWpt*>& wpts, const QString& filename
     out << "U  1" << endl;
     out << "z " << west << ", " << south << ", " << east << ", " << north << endl;
 
-    foreach(CGisItemWpt * wpt, wpts)
+    for(CGisItemWpt * wpt : wpts)
     {
         wpt->saveTwoNav(out, dir);
     }
@@ -244,7 +244,7 @@ bool CTwoNavProject::load(const QString& filename)
     QDir dir(filename);
 
     QStringList entries = dir.entryList(QDir::NoDotAndDotDot|QDir::Dirs|QDir::Files);
-    foreach(const QString &entry, entries)
+    for(const QString &entry : entries)
     {
         QFileInfo fi(entry);
 
@@ -255,7 +255,7 @@ bool CTwoNavProject::load(const QString& filename)
         }
     }
 
-    foreach(const QString &entry, entries)
+    for(const QString &entry : entries)
     {
         QFileInfo fi(entry);
 
