@@ -22,32 +22,33 @@
 #include "gis/IGisItem.h"
 #include "gis/IGisLine.h"
 #include "gis/trk/CActivityTrk.h"
-#include "helpers/INotifiable.h"
+#include "helpers/CLimit.h"
+#include "helpers/CValue.h"
 
+#include <QDebug>
 #include <QPen>
 #include <QPointer>
 #include <functional>
 
 class QDomNode;
 class IGisProject;
-class IPlot;
+class INotifyTrk;
 class CDetailsTrk;
 class CScrOptTrk;
 class QSqlDatabase;
 class CQlgtTrack;
 class IQlgtOverlay;
-class QDir;
 class CProgressDialog;
 class CPropertyTrk;
+class CFitStream;
 
 #define TRK_N_COLORS          17
 #define ASCEND_THRESHOLD       5
-
-#include <QDebug>
-
+#define MIN_WIDTH_INFO_BOX   300
 
 class CGisItemTrk : public IGisItem, public IGisLine
 {
+    Q_DECLARE_TR_FUNCTIONS(CGisItemTrk)
 public:
     struct trk_t;
     struct trkpt_t;
@@ -64,71 +65,81 @@ public:
         , eModeRange
     };
 
-    /**
-       @brief Used to create a new track from a part of an existing track
-       @param name
-       @param idx1
-       @param idx2
-       @param srctrk
-       @param project
-     */
+    enum visual_e
+    {
+        eVisualNone          = 0
+        , eVisualColorLegend = 0x01
+        , eVisualPlot        = 0x02
+        , eVisualDetails     = 0x04
+        , eVisualProject     = 0x08
+        , eVisualColorAct    = 0x10
+        , eVisualTrkTable    = 0x20
+        , eVisualAll         = -1
+    };
+
+    /** @brief Used to create a new track from a part of an existing track */
     CGisItemTrk(const QString& name, qint32 idx1, qint32 idx2, const trk_t &srctrk, IGisProject *project);
-    /**
-       @brief Used to create a copy of track with new parent
-       @param parentTrk
-       @param project
-       @param idx
-       @param clone
-     */
+
+    /** @brief Used to create a copy of track with new parent */
     CGisItemTrk(const CGisItemTrk& parentTrk, IGisProject * project, int idx, bool clone);
-    /**
-       @brief Used to restore a track from a line of coordinates
-       @param l
-       @param name
-       @param project
-       @param idx
-     */
+
+    /** @brief Used to restore a track from a line of coordinates */
     CGisItemTrk(const SGisLine &l, const QString &name, IGisProject *project, int idx);
-    /**
-       @brief Used to create track from GPX file
-       @param xml
-       @param project
-     */
+
+    /** @brief Used to create track from GPX file */
     CGisItemTrk(const QDomNode &xml, IGisProject *project);
-    /**
-       @brief Used to restore track from history structure
-       @param hist
-       @param project
-     */
-    CGisItemTrk(const history_t& hist, IGisProject * project);
-    /**
-       @brief Used to restore track from database
-       @param id
-       @param db
-       @param project
-     */
+
+    /** @brief Used to restore track from history structure */
+    CGisItemTrk(const history_t& hist, const QString& dbHash, IGisProject * project);
+
+    /** @brief Used to restore track from database */
     CGisItemTrk(quint64 id, QSqlDatabase& db, IGisProject * project);
-    /**
-       @brief Clone QLandkarte GT track
-       @param trk1
-     */
+
+    /** @brief Clone QLandkarte GT track */
     CGisItemTrk(const CQlgtTrack& trk1);
-    /**
-       @brief Load track from file (e.g. TwoNav *trk)
-       @param filename
-       @param project
-     */
+
+    /** @brief Load track from file (e.g. TwoNav *trk) */
     CGisItemTrk(const QString& filename, IGisProject * project);
 
     CGisItemTrk(const IQlgtOverlay& ovl);
 
+    /**
+       @brief Creates a new track via provided trkdata
+       @param trkdata  The track's new data (will be moved, don't use your "copy" after construction!
+       @param project  The project this track belongs to
+     */
+    CGisItemTrk(trk_t& trkdata, IGisProject *project);
+
+    CGisItemTrk(CFitStream& stream, IGisProject * project);
+
     virtual ~CGisItemTrk();
 
+    /**
+       @brief Overide IGisItem::updateHistory() method
+
+        same as changed();
+
+     */
+    void updateHistory(quint32 visuals);
+
+    /**
+       @brief Update all registered visuals via the INotifyTrk interface
+       @param a bit field of visuals to be updated
+       @param who a string for debug purposes
+     */
+    void updateVisuals(quint32 visuals, const QString &who);
+
+    /**
+       @brief Create a cloned copy of this track
+       @return The cloned item a pointer
+     */
+    IGisItem * createClone() override;
+
     /**
        @brief Save track to GPX tree
        @param gpx   The <gpx> node to append by the track
      */
-    void save(QDomNode& gpx);
+    void save(QDomNode& gpx) override;
     /**
        @brief Save track to TwoNav track file
        @param dir   the path to store the file
@@ -139,16 +150,16 @@ public:
        @param stream  the data stream to read from
        @return A reference to the stream
      */
-    QDataStream& operator<<(QDataStream& stream);
+    QDataStream& operator<<(QDataStream& stream) override;
     /**
        @brief Serialize track into a binary data stream
        @param stream  the data stream to write to.
        @return A reference to the stream
      */
-    QDataStream& operator>>(QDataStream& stream) const;
+    QDataStream& operator>>(QDataStream& stream) const override;
 
     /// get name of track
-    const QString& getName() const
+    const QString& getName() const override
     {
         return trk.name.isEmpty() ? noName : trk.name;
     }
@@ -165,28 +176,27 @@ public:
         return color;
     }
 
-
     /**
        @brief get a summary of the track
-       @param allowEdit if true the track name is a link to allow interactions like edit
+       @param showName  if true the track name is shown
        @return
      */
-    QString getInfo(bool allowEdit = false) const;
+    QString getInfo(bool showName = true) const override;
     /// get a summary of a selected range
-    QString getInfoRange();
+    QString getInfoRange() const;
     /// get a summary of a selected range defined by two track points
-    QString getInfoRange(const trkpt_t& pt1, const trkpt_t& pt2);
+    QString getInfoRange(const trkpt_t& pt1, const trkpt_t& pt2) const;
     /// get a summary for a track point
-    QString getInfoTrkPt(const trkpt_t& pt);
+    QString getInfoTrkPt(const trkpt_t& pt) const;
     /// get a progress summary for a selected track point
-    QString getInfoProgress(const trkpt_t& pt);
+    QString getInfoProgress(const trkpt_t& pt) const;
 
     quint32 getTotalElapsedSeconds() const
     {
         return totalElapsedSeconds;
     }
 
-    quint32 getTotalElapsedSecondsMoving()
+    quint32 getTotalElapsedSecondsMoving() const
     {
         return totalElapsedSecondsMoving;
     }
@@ -206,22 +216,29 @@ public:
         return totalDistance;
     }
 
-    const QString& getComment() const
+    const QString& getComment() const override
     {
         return trk.cmt;
     }
-    const QString& getDescription() const
+    const QString& getDescription() const override
     {
         return trk.desc;
     }
-    const QList<link_t>& getLinks() const
+    const QList<link_t>& getLinks() const override
     {
         return trk.links;
     }
+
+    qint32 getCntTotalPoints()
+    {
+        return cntTotalPoints;
+    }
+
+
     /// get the track as a simple coordinate polyline
-    void getPolylineFromData(QPolygonF &l);
+    void getPolylineFromData(QPolygonF &l) const;
     /// get the track as polyline with elevation, pixel and GIS coordinates.
-    void getPolylineFromData(SGisLine& l);
+    void getPolylineFromData(SGisLine& l) override;
 
     const QDateTime& getTimeStart() const
     {
@@ -242,6 +259,24 @@ public:
     {
         return propHandler;
     }
+
+    const trkpt_t * getMouseMoveFocusPoint() const
+    {
+        return mouseMoveFocus;
+    }
+
+    quint32 getAllValidFlags() const
+    {
+        return allValidFlags;
+    }
+
+    /**
+       @brief Get the elevation of a track point
+       @param idx   The total index of the point
+       @return The elevation or NOINT if the index is invalid, or the track point has no elevation value.
+     */
+    qint32 getElevation(qint32 idx) const;
+
     /** @defgroup ColorSource Stuff related to coloring tracks using data from different sources
 
         @{
@@ -259,9 +294,9 @@ public:
 
         @return  The new source to use.
      */
-    QString getColorizeSource()
+    QString getColorizeSource() const
     {
-        return colorSource;
+        return colorSourceLimit.getSource();
     }
 
     QStringList getExistingDataSources() const;
@@ -269,30 +304,29 @@ public:
     void setColorizeLimitLow(qreal limit);
     qreal getColorizeLimitLow() const
     {
-        return limitLow;
+        return colorSourceLimit.getMin();
     }
 
     void setColorizeLimitHigh(qreal limit);
     qreal getColorizeLimitHigh() const
     {
-        return limitHigh;
+        return colorSourceLimit.getMax();
     }
 
-    const QString getColorizeUnit() const;
+    QString getColorizeUnit() const;
 
-    void getExtrema(qreal &min, qreal &max, const QString &source) const;
+    qreal getMin(const QString& source) const;
+    qreal getMax(const QString& source) const;
 
 private:
-    QString colorSource  = "";
-
-    // the low and high limit for (slope-)colored drawing of tracks
-    qreal limitLow  = -10;
-    qreal limitHigh =  10;
-
-    void drawColorized(QPainter &p);
+    void drawColorized(QPainter &p) const;
+    void drawColorizedByActivity(QPainter& p) const;
+    void setPen(QPainter& p, QPen& pen, quint32 flag) const;
     /**@}*/
 
 
+
+public:
     /**
        @brief Get the indices of visible points for a selected range
 
@@ -301,16 +335,22 @@ private:
        @param idx1 a reference to receive the first index
        @param idx2 a reference to receive the second index
      */
-public:
-    void getSelectedVisiblePoints(qint32& idx1, qint32& idx2);
+    void getSelectedVisiblePoints(qint32& idx1, qint32& idx2) const;
 
     void setName(const QString& str);
     void setColor(int idx);
+    /// set the width of the inner track line by factor
     bool setMode(mode_e m, const QString &owner);
-    void setComment(const QString& str);
-    void setDescription(const QString& str);
-    void setLinks(const QList<link_t>& links);
-    void setDataFromPolyline(const SGisLine &l);
+    void setComment(const QString& str)         override;
+    void setDescription(const QString& str)     override;
+    void setLinks(const QList<link_t>& links)   override;
+    void setDataFromPolyline(const SGisLine &l) override;
+    /**
+       @brief Manually set the elevation value of a single track point
+       @param idx   the total index of the track point
+       @param ele   the new elevation value
+     */
+    void setElevation(qint32 idx, qint32 ele);
 
     /**
        @brief display the track screen options
@@ -319,24 +359,26 @@ public:
        @param mouse     the mouse object causing the request
        @return          a pointer to the screen option widget
      */
-    IScrOpt * getScreenOptions(const QPoint &origin, IMouse * mouse);
+    IScrOpt * getScreenOptions(const QPoint &origin, IMouse * mouse) override;
     /**
        @brief Get a screen pixel of the track close to the given position on the screen
        @param screenPos Screen position as pixel coordinate
        @return The screen coordinates as pixel of a track point close by
      */
-    QPointF getPointCloseBy(const QPoint& screenPos);
+    QPointF getPointCloseBy(const QPoint& screenPos) override;
     /**
        @brief isCloseTo
        @param pos Screen position as pixel coordinate
        @return True if point is considered close enough
      */
-    bool isCloseTo(const QPointF& pos);
+    bool isCloseTo(const QPointF& pos) override;
 
-    void drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>& blockedAreas, CGisDraw * gis);
-    void drawItem(QPainter& p, const QRectF& viewport, CGisDraw * gis);
-    void drawLabel(QPainter& p, const QPolygonF& viewport, QList<QRectF>& blockedAreas, const QFontMetricsF& fm, CGisDraw * gis);
-    void drawHighlight(QPainter& p);
+    bool isWithin(const QRectF& area, selflags_t flags) override;
+
+    void drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>& blockedAreas, CGisDraw * gis) override;
+    void drawItem(QPainter& p, const QRectF& viewport, CGisDraw * gis) override;
+    void drawLabel(QPainter&, const QPolygonF&, QList<QRectF>&, const QFontMetricsF&, CGisDraw*) override;
+    void drawHighlight(QPainter& p) override;
     void drawRange(QPainter& p);
 
     /**
@@ -346,7 +388,7 @@ public:
 
        @param yes   set true to gain focus.
      */
-    void gainUserFocus(bool yes);
+    void gainUserFocus(bool yes) override;
     /**
        @brief Make sure the track has lost focus.
 
@@ -358,7 +400,7 @@ public:
     /**
        @brief Make sure a CDetailsTrk widget is registered with the main tab widget
      */
-    void edit();
+    void edit() override;
 
     /**
        @brief Cut track at mouseClickFocus
@@ -381,7 +423,7 @@ public:
        Handle the complete process of selecting tracks, choosing the order and
        the final name with dialogs.
      */
-    void combine(const QList<key_t> &keysPreSel);
+    void combine(const QList<key_t> &keys);
 
     /**
        @brief Set the trkpt_t::eHidden flag
@@ -425,14 +467,14 @@ public:
        including the hidden (trkpt_t::eHidden) ones.
 
      */
-    void copySelectedPoints();
+    void copySelectedPoints() const;
 
     /**
        @brief Check for user focus
 
        @return True if the track has user focus
      */
-    bool hasUserFocus() const
+    bool hasUserFocus() const override
     {
         return key == keyUserFocus;
     }
@@ -454,7 +496,7 @@ public:
 
        @param plot
      */
-    void registerPlot(IPlot * plot);
+    void registerVisual(INotifyTrk * visual);
 
     /**
        @brief Each plot widget that operates on the track must unregister during it's destruction
@@ -463,13 +505,12 @@ public:
 
        @param plot
      */
-    void unregisterPlot(IPlot * plot);
+    void unregisterVisual(INotifyTrk * visual);
 
     /**
        @brief Use point with the distance from start matching best the given distance.
 
        @param dist      the distance in [m]
-       @param initiator a pointer to an initiating IPlot object, or 0
      */
     bool setMouseFocusByDistance(qreal dist, focusmode_e fmode, const QString& owner);
 
@@ -477,7 +518,6 @@ public:
        @brief Use point with time from start matching best the given time delta
 
        @param time      a time delta in [s] relative to the start time
-       @param initiator a pointer to an initiating IPlot object, or 0
      */
     bool setMouseFocusByTime(quint32 time, focusmode_e fmode, const QString& owner);
 
@@ -494,78 +534,45 @@ public:
      */
     bool setMouseFocusByTotalIndex(qint32 idx, focusmode_e fmode, const QString& owner);
 
+
+    /** @defgroup Filter All filters implemented by CGisItemTrks.
+
+        @note All filter implementations are found in src/gis/trk/filter/filter.cpp
+
+        @{
+     */
     /**
        @brief Reduce the amount of visible track points with the help of the Douglas Peuker algorithm
 
-       @note All filter implementations are found in src/gis/trk/filter/filter.cpp
-
        @param dist the Douglas Peuker distance in meters
      */
     void filterReducePoints(qreal dist);
 
-    /**
-       @brief Remove track points without valid location at the beginning of the track
+    /** @brief Remove track points without valid location at the beginning of the track */
+    void filterRemoveInvalidPoints();
 
-       @note All filter implementations are found in src/gis/trk/filter/filter.cpp
-     */
-    void filterRemoveNullPoints();
-    /**
-       @brief filterReset
-
-       @note All filter implementations are found in src/gis/trk/filter/filter.cpp
-     */
-    void filterReset();
-    /**
-       @brief filterDelete
-
-       @note All filter implementations are found in src/gis/trk/filter/filter.cpp
-     */
-    void filterDelete();
-    /**
-       @brief filterSmoothProfile
-
-       @note All filter implementations are found in src/gis/trk/filter/filter.cpp
-
-       @param points  size of Median filter
-     */
+    /** @param points  size of Median filter */
     void filterSmoothProfile(int points);
-    /**
-       @brief filterReplaceElevation
-     */
-    void filterReplaceElevation();
-    /**
-       @brief filterOffsetElevation
 
-       @note All filter implementations are found in src/gis/trk/filter/filter.cpp
-
-       @param offset elevation offset in meters
-     */
+    /** @param offset elevation offset in meters */
     void filterOffsetElevation(int offset);
-    /**
-       @brief filterNewDate
 
-       @note All filter implementations are found in src/gis/trk/filter/filter.cpp
-
-       @param date new date for start of track
-     */
+    /** @param date new date for start of track */
     void filterNewDate(const QDateTime& date);
-    /**
-       @brief filterObscureDate
 
-       @note All filter implementations are found in src/gis/trk/filter/filter.cpp
-
-       @param delta intervall to increase timestamps in seconds
-     */
+    /** @param delta interval to increase timestamps in seconds a*/
     void filterObscureDate(int delta);
-    /**
-       @brief filterSpeed
 
-       @note All filter implementations are found in src/gis/trk/filter/filter.cpp
-
-       @param speed speed in meter per seconds
-     */
+    /** @param speed speed in meter per seconds */
     void filterSpeed(qreal speed);
 
+    void filterReplaceElevation();
+    void filterReset();
+    void filterDelete();
+    void filterSplitSegment();
+    void filterDeleteExtension(const QString &ext);
+    /** @} */
+
     /**
        @brief Correlate waypoints with the track points
 
@@ -578,7 +585,13 @@ public:
     void findWaypointsCloseBy(CProgressDialog &progress, quint32 &current);
 
 private:
-    void setSymbol();
+    /// no don't really use it, use CGisItemTrk(quint32 visuals) instead
+    void updateHistory() override
+    {
+        updateHistory(eVisualAll);
+    }
+
+    void setSymbol() override;
     /**
        @brief Read track data from section in GPX file
        @param xml   The XML <trk> section
@@ -591,6 +604,12 @@ private:
        @param filename
      */
     bool readTwoNav(const QString& filename);
+    /**
+       @brief Read serialized track data from a FIT file stream
+       @param stream
+     */
+    void readTrkFromFit(CFitStream &stream);
+
     /**
        @brief Derive secondary data from the track data
 
@@ -598,8 +617,14 @@ private:
      */
     void deriveSecondaryData();
 
+    /**
+     * @brief Reset internal data like range selection and details dialog
+     */
+    void resetInternalData();
 
-    /** @defgroup ExtremaExtensions Stuff related to calculation of extremas/extensions
+    void verifyTrkPt(trkpt_t *&last, trkpt_t& trkpt);
+
+    /** @defgroup ExtremaExtensions Stuff related to calculation of extrema/extensions
 
         @{
      */
@@ -616,6 +641,10 @@ private:
     QHash<QString, limits_t> extrema;
     void updateExtremaAndExtensions();
 
+    const trkpt_t* getTrkPtByCondition(std::function<bool(const trkpt_t&)> cond) const;
+
+    trkpt_t* getTrkPtByCondition(std::function<bool(const trkpt_t&)> cond);
+
     /**
        @brief Try to get access Nth visible point matching the idx
 
@@ -625,7 +654,7 @@ private:
        @param idx The index into all visible points
        @return A null pointer of no point is found.
      */
-    const trkpt_t *getTrkPtByVisibleIndex(qint32 idx);
+    const trkpt_t *getTrkPtByVisibleIndex(qint32 idx) const;
     /**
        @brief Try to get access Nth point
 
@@ -635,21 +664,21 @@ private:
        @param idx The index into all points
        @return A null pointer of no point is found.
      */
-    const trkpt_t *getTrkPtByTotalIndex(qint32 idx);
+    const trkpt_t *getTrkPtByTotalIndex(qint32 idx) const;
 
     /**
        @brief Check if the track point at index it the last one visible
        @param idxTotal  The point's index
        @return True if it is the last one visible
      */
-    bool isTrkPtLastVisible(qint32 idxTotal);
+    bool isTrkPtLastVisible(qint32 idxTotal) const;
 
     /**
        @brief Check if the track point at index it the first one visible
        @param idxTotal  The point's index
        @return True if it is the first one visible
      */
-    bool isTrkPtFirstVisible(qint32 idxTotal);
+    bool isTrkPtFirstVisible(qint32 idxTotal) const;
     /**
        @brief Tell the point of focus to all plots and the detail dialog
 
@@ -660,6 +689,8 @@ private:
     bool publishMouseFocus(const trkpt_t * pt, focusmode_e fmode, const QString &owner);
     void publishMouseFocusNormalMode(const trkpt_t * pt, focusmode_e fmode);
     void publishMouseFocusRangeMode(const trkpt_t * pt, focusmode_e fmode);
+    void resetMouseRange();
+
     /**
        @brief Replace all trackpoints by the coordinates stored in the polyline
 
@@ -669,7 +700,7 @@ private:
      */
     void readTrackDataFromGisLine(const SGisLine &l);
     /**
-       @brief Overide IGisItem::changed() method
+       @brief Override IGisItem::changed() method
 
        As the CDetailsTrk is no modal dialog that blocks the GUI from any other input the track
        can be changed while the widget is visible. Therefore it needs some feedback to update the
@@ -680,13 +711,17 @@ private:
        @param what  The reason string
        @param icon  An icon string
      */
-    void changed(const QString& what, const QString& icon);
+    void changed(const QString& what, const QString& icon) override;
 
     /// setup colorIdx, color, bullet and icon
     void setColor(const QColor& c);
     /// setup track icon by color
     void setIcon(const QString& iconColor);
 
+    void setMouseFocusVisuals(const trkpt_t * pt);
+    void setMouseRangeFocusVisuals(const trkpt_t * pt1, const CGisItemTrk::trkpt_t * pt2);
+    void setMouseClickFocusVisuals(const trkpt_t * pt);
+
 public:
     struct trkpt_t : public wpt_t
     {
@@ -711,9 +746,8 @@ public:
 
         enum flag_e
         {
-            eHidden     = 0x00000004      ///< mark point as deleted
-
-                          // activity flags
+            eHidden         = 0x00000004      ///< mark point as deleted
+                              // activity flags
             ,eActNone   = 0x00000000
             ,eActFoot   = 0x80000000
             ,eActCycle  = 0x40000000
@@ -723,38 +757,73 @@ public:
             ,eActSwim   = 0x04000000
             ,eActShip   = 0x02000000
             ,eActAero   = 0x01000000
-            ,eActMask   = 0xFF000000    ///< mask for activity flags
-            ,eActMaxNum = 8             ///< maximum number of activity flags. this is defined by the mask
+            ,eActSki    = 0x00800000
+            ,eActMask   = 0xFF800000    ///< mask for activity flags
+            ,eActMaxNum = 9             ///< maximum number of activity flags. this is defined by the mask
         };
 
-        quint32 flags = 0;
-        /// index within the complete track
-        qint32 idxTotal;
-        /// offset into lineSimple
-        qint32 idxVisible;
+        enum valid_e
+        {
+            eValidTime     = 0x00000001
+            ,eValidEle      = 0x00000002
+            ,eValidPos      = 0x00000004
+            ,eValidMask     = 0x0000FFFF
+        };
 
-        /// the distance to the last point
-        qreal deltaDistance;
-        /// the distance from the start of the track
-        qreal distance;
-        /// the ascend from the start of the track
-        qreal ascend;
-        /// the descend from the start of the track
-        qreal descend;
-        /// the slope [°] over several points close by
-        qreal slope1;
-        /// the slope [%] over several points close by
-        qreal slope2;
-        /// the speed over several points close by
-        qreal speed;
-        /// the seconds since the start of the track
-        qreal elapsedSeconds;
-        /// the seconds since the start of the track with moving speed
-        qreal elapsedSecondsMoving;
-        /// the key of an attached waypoint
-        key_t keyWpt;
-        /// track point extensions
-        QHash<QString,QVariant> extensions;
+        enum invalid_e
+        {
+            eInvalidTime    = eValidTime << 16
+            ,eInvalidEle    = eValidEle  << 16
+            ,eInvalidPos    = eValidPos  << 16
+            ,eInvalidMask   = 0xFFFF0000
+        };
+
+
+        inline bool isHidden() const
+        {
+            return hasFlag(trkpt_t::eHidden);
+        }
+
+        inline bool hasFlag(enum flag_e flag) const
+        {
+            return flags & flag;
+        }
+
+        inline void setFlag(enum flag_e flag)
+        {
+            flags |= flag;
+        }
+
+        inline void unsetFlag(enum flag_e flag)
+        {
+            flags &= ~flag;
+        }
+
+        inline bool isValid(valid_e flag) const
+        {
+            return (valid & flag) != 0;
+        }
+
+        inline bool isInvalid(invalid_e flag) const
+        {
+            return (valid & flag) != 0;
+        }
+
+        quint32 flags = 0;
+        quint32 valid = 0;
+        qint32 idxTotal = NOIDX;            //< index within the complete track
+        qint32 idxVisible;                  //< offset into lineSimple
+        qreal deltaDistance;                //< the distance to the last point
+        qreal distance;                     //< the distance from the start of the track
+        qreal ascend;                       //< the ascend from the start of the track
+        qreal descend;                      //< the descend from the start of the track
+        qreal slope1;                       //< the slope [°] over several points close by
+        qreal slope2;                       //< the slope [%] over several points close by
+        qreal speed;                        //< the speed over several points close by
+        qreal elapsedSeconds;               //< the seconds since the start of the track
+        qreal elapsedSecondsMoving;         //< the seconds since the start of the track with moving speed
+        key_t keyWpt;                       //< the key of an attached waypoint
+        QHash<QString,QVariant> extensions; //< track point extensions
     };
 
     struct trkseg_t
@@ -790,8 +859,49 @@ public:
         return trk;
     }
 
-    void registerNotification(INotifiable *obj);
-    void unregisterNotification(INotifiable *obj);
+    void updateFromDB(quint64 id, QSqlDatabase& db) override;
+
+private:
+    fGetLimit _getMin = [this](const QString& source)
+                        {
+                            return getMin(source);
+                        };
+
+    fGetLimit _getMax = [this](const QString& source)
+                        {
+                            return getMax(source);
+                        };
+
+    qreal   getMinProp(const QString& source) const;
+    qreal   getMaxProp(const QString& source) const;
+    QString getUnitProp(const QString& source) const;
+
+    fGetLimit _getMinProp = [this](const QString& source)
+                            {
+                                return getMinProp(source);
+                            };
+
+    fGetLimit _getMaxProp = [this](const QString& source)
+                            {
+                                return getMaxProp(source);
+                            };
+
+    fGetUnit _getUnitProp = [this](const QString& source)
+                            {
+                                return getUnitProp(source);
+                            };
+
+    fMarkChanged _markChanged = [this]()
+                                {
+                                    updateHistory(eVisualNone);
+                                };
+
+public:
+    CLimit limitsGraph1 {"TrackDetails/Graph1", _getMin, _getMax, _getMinProp, _getMaxProp, _getUnitProp, _markChanged};
+    CLimit limitsGraph2 {"TrackDetails/Graph2", _getMin, _getMax, _getMinProp, _getMaxProp, _getUnitProp, _markChanged};
+    CLimit limitsGraph3 {"TrackDetails/Graph3", _getMin, _getMax, _getMinProp, _getMaxProp, _getUnitProp, _markChanged};
+
+    CLimit colorSourceLimit {"TrackDetails/Style", _getMin, _getMax, _getMin, _getMax, _getUnitProp, _markChanged};
 
 private:
     /// this is the GPX structure oriented data of the track
@@ -799,8 +909,6 @@ private:
 
     /// the key of the track having the user focus.
     static key_t keyUserFocus;
-    /// background (border) color of all tracks
-    static const QPen penBackground;
     /// drawing and mouse interaction is dependent on the mode
     mode_e mode = eModeNormal;
 
@@ -808,6 +916,7 @@ private:
        \defgroup TrackStatistics Some statistical values over the complete track
      */
     /**@{*/
+    quint32 allValidFlags = 0;
     qint32 cntTotalPoints   = 0;
     qint32 cntVisiblePoints = 0;
     QDateTime timeStart;
@@ -823,23 +932,47 @@ private:
         \defgroup DrawUtilies Objects used to draw the track
      */
     /**@{*/
-    /// the track line color by index
-    unsigned colorIdx = 4;
-    /// the track line color
-    QColor color;
+
+    unsigned colorIdx = 4;  //< the track line color by index
+    QColor color;           //< the track line color
+
+    QPixmap bullet;         //< the trackpoint bullet icon
+    QPolygonF lineSimple;   //< the current track line as screen pixel coordinates
+    QPolygonF lineFull;     //< visible and invisible points
+
+    qint32 penWidthFg = 3;  //< inner trackline width
+    qint32 penWidthBg = 5;  //< outer trackline width
+    qint32 penWidthHi = 11; //< highlighted trackline width
+
+    qint32 widthInfoBox = MIN_WIDTH_INFO_BOX;
+
     /// the pen with the actual track color
-    QPen penForeground {Qt::blue, 3, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin};
-    /// the trackpoint bullet icon
-    QPixmap bullet;
-    /// the current track line as screen pixel coordinates
-    QPolygonF lineSimple;
-    /// visible and invisible points
-    QPolygonF lineFull;
+    QPen penForeground {Qt::blue, qreal(penWidthFg), Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin};
+    /// background (border) color of all tracks
+    QPen penBackground {Qt::white, qreal(penWidthBg), Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin};
+
+
+    fValueOnChange onChange = [this](const QVariant& val)
+                              {
+                                  int w = qRound(3.0 * val.toDouble());
+
+                                  penWidthFg = w;
+                                  penWidthBg = w + 2;
+                                  penWidthHi = w + 8;
+
+                                  penForeground.setWidth(penWidthFg);
+                                  penBackground.setWidth(penWidthBg);
+                              };
+
+public:
+    CValue lineScale     {"TrackDetails/lineScale", 1.0, _markChanged, onChange};
+    CValue showArrows    {"TrackDetails/showArrows", true, _markChanged};
+private:
     /**@}*/
 
 
     /**
-        A list of plot objects that need to get informed on any change in data.
+        A list of INotifyTrk objects that need to get informed on any change in data.
 
         @note This is necessary because QTreeWidgetItem is not derived from QObject.
               Thus no signals and slots can be handled. Probably this is because the
@@ -847,20 +980,21 @@ private:
               amount of items.
 
               Anyway we need some kind of signaling between the track object and the
-              plot objects displaying the data. And we have to keep in mind that
+              INotifyTrk objects displaying the data. And we have to keep in mind that
               the track can be delete by the user at any time. That is why no other
               object is allowed to save a pointer to the track. It must store the
               key. But accessing the track via key is expensive.
 
               That is why we make an exception here. As the track will delete all
-              registered plot objects upon destruction, it should be ok to store
-              the track object in the plot object, too. By that plot and track can
+              registered INotifyTrk objects upon destruction, it should be ok to store
+              the track object in the INotifyTrk object, too. By that INotifyTrk and track can
               easily communicate with each other.
-     */
-    QSet<IPlot*> registeredPlots;
 
-    QSet<INotifiable*> notifyOnChange;
-    void notifyChange();
+        @note CDetailsTrk is an INotifyTrk, too. But it is a bit special as it has to be destroyed
+              right after all other INotifyTrk have been destroyed. That is why it is not part of
+              that set.
+     */
+    QSet<INotifyTrk*> registeredVisuals;
 
     /**
         \defgroup FocusRange Variables to handle mouse focus and range selection
@@ -886,20 +1020,14 @@ private:
      */
     QString mouseFocusOwner;
 
-    /// the current track point selected by mouse movement
-    const trkpt_t * mouseMoveFocus = 0;
-    /// the last track point the user clicked on
-    const trkpt_t * mouseClickFocus = 0;
-    /// the first point of a range selection
-    const trkpt_t * mouseRange1 = 0;
-    /// the second point of a range selection
-    const trkpt_t * mouseRange2 = 0;
+    const trkpt_t *mouseMoveFocus  = nullptr; //< the current track point selected by mouse movement
+    const trkpt_t *mouseClickFocus = nullptr; //< the last track point the user clicked on
+    const trkpt_t *mouseRange1     = nullptr; //< the first point of a range selection
+    const trkpt_t *mouseRange2     = nullptr; //< the second point of a range selection
     /**@}*/
 
-    /// the track's details dialog if any
-    QPointer<CDetailsTrk> dlgDetails;
-    /// the track's screen option if visible
-    QPointer<CScrOptTrk>  scrOpt;
+    QPointer<CDetailsTrk> dlgDetails; //< the track's details dialog if any
+    QPointer<CScrOptTrk>  scrOpt;     //< the track's screen option if visible
 
     /// all function concerning track activities have been moved to CActivityTrk
     CActivityTrk activities = {this};
@@ -908,6 +1036,20 @@ private:
     CPropertyTrk * propHandler = nullptr;
 };
 
+class INotifyTrk
+{
+public:
+    INotifyTrk(CGisItemTrk::visual_e mask) : mask(mask){}
+    virtual ~INotifyTrk() = default;
+
+    virtual void updateData() = 0;
+    virtual void setMouseFocus(const CGisItemTrk::trkpt_t * pt) = 0;
+    virtual void setMouseRangeFocus(const CGisItemTrk::trkpt_t * pt1, const CGisItemTrk::trkpt_t * pt2) = 0;
+    virtual void setMouseClickFocus(const CGisItemTrk::trkpt_t * pt) = 0;
+
+    const CGisItemTrk::visual_e mask;
+};
+
 using fTrkPtGetVal = std::function<qreal(const CGisItemTrk::trkpt_t&)>;
 
 #endif //CGISITEMTRK_H
