@@ -26,11 +26,7 @@ CFilterReplaceElevation::CFilterReplaceElevation(CGisItemTrk &trk, QWidget *pare
 {
     setupUi(this);
 
-    connect(toolApply, SIGNAL(clicked()), this, SLOT(slotApply()));
-}
-
-CFilterReplaceElevation::~CFilterReplaceElevation()
-{
+    connect(toolApply, &QToolButton::clicked, this, &CFilterReplaceElevation::slotApply);
 }
 
 void CFilterReplaceElevation::slotApply()
