@@ -27,11 +27,7 @@ CFilterReset::CFilterReset(CGisItemTrk &trk, QWidget *parent)
 {
     setupUi(this);
 
-    connect(toolApply, SIGNAL(clicked()), this, SLOT(slotApply()));
-}
-
-CFilterReset::~CFilterReset()
-{
+    connect(toolApply, &QToolButton::clicked, this, &CFilterReset::slotApply);
 }
 
 void CFilterReset::slotApply()
