@@ -18,7 +18,10 @@
 
 #include "CMainWindow.h"
 #include "GeoMath.h"
+#include "gis/CGisWidget.h"
 #include "gis/trk/CGisItemTrk.h"
+#include "gis/trk/CKnownExtension.h"
+#include "gis/trk/CPropertyTrk.h"
 
 #include <QtMath>
 #include <proj_api.h>
@@ -28,9 +31,9 @@ void CGisItemTrk::filterReducePoints(qreal dist)
     QVector<pointDP> line;
     bool nothingDone = true;
 
-    foreach (const trkseg_t &seg, trk.segs)
+    for(const trkseg_t &seg : trk.segs)
     {
-        foreach(const trkpt_t &pt, seg.pts)
+        for(const trkpt_t &pt : seg.pts)
         {
             pointDP dp;
             dp.x = pt.lon * DEG_TO_RAD;
@@ -88,7 +91,6 @@ void CGisItemTrk::filterReducePoints(qreal dist)
                     nothingDone = false;
                     pt.flags |=  trkpt_t::eHidden;
                 }
-
             }
 
             cnt++;
@@ -103,32 +105,30 @@ void CGisItemTrk::filterReducePoints(qreal dist)
     deriveSecondaryData();
     QString val, unit;
     IUnit::self().meter2distance(dist, val, unit);
-    changed(QObject::tr("Hide points by Douglas Peuker algorithm (%1%2)").arg(val).arg(unit), "://icons/48x48/PointHide.png");
+    changed(tr("Hide points by Douglas Peuker algorithm (%1%2)").arg(val).arg(unit), "://icons/48x48/PointHide.png");
 }
 
-void CGisItemTrk::filterRemoveNullPoints()
+void CGisItemTrk::filterRemoveInvalidPoints()
 {
     bool nothingDone    = true;
-    bool done           = false;
 
-    for(int i = 0; i < trk.segs.size() && !done; i++)
+    // use all valid flags as invalid mask. By that only
+    // invalid flags for properties with valid points count
+    quint32 invalidMask = (getAllValidFlags() & trkpt_t::eValidMask) << 16;
+
+    for(int i = 0; i < trk.segs.size(); i++)
     {
         trkseg_t& seg = trk.segs[i];
 
-        for(int n = 0; n < seg.pts.size() && !done; n++)
+        for(int n = 0; n < seg.pts.size(); n++)
         {
             trkpt_t& pt = seg.pts[n];
 
-            if( (NOFLOAT == pt.lat || 0. == pt.lat)
-             && (NOFLOAT == pt.lon || 0. == pt.lon) )
+            if(pt.isInvalid(trkpt_t::invalid_e(invalidMask)))
             {
                 pt.flags |= trkpt_t::eHidden;
                 nothingDone = false;
             }
-            else
-            {
-                done = true;
-            }
         }
     }
 
@@ -138,7 +138,7 @@ void CGisItemTrk::filterRemoveNullPoints()
     }
 
     deriveSecondaryData();
-    changed(QObject::tr("Hide points with invalid coordinates at the beginning of the track"), "://icons/48x48/PointHide.png");
+    changed(tr("Hide points with invalid data."), "://icons/48x48/PointHide.png");
 }
 
 void CGisItemTrk::filterReset()
@@ -154,7 +154,7 @@ void CGisItemTrk::filterReset()
         }
     }
     deriveSecondaryData();
-    changed(QObject::tr("Reset all hidden track points to visible"), "://icons/48x48/PointHide.png");
+    changed(tr("Reset all hidden track points to visible"), "://icons/48x48/PointHide.png");
 }
 
 void CGisItemTrk::filterDelete()
@@ -188,7 +188,7 @@ void CGisItemTrk::filterDelete()
     }
 
     deriveSecondaryData();
-    changed(QObject::tr("Permanently removed all hidden track points"), "://icons/48x48/PointHide.png");
+    changed(tr("Permanently removed all hidden track points"), "://icons/48x48/PointHide.png");
 }
 
 void CGisItemTrk::filterSmoothProfile(int points)
@@ -238,7 +238,7 @@ void CGisItemTrk::filterSmoothProfile(int points)
         }
     }
     deriveSecondaryData();
-    changed(QObject::tr("Smoothed profile with a Median filter of size %1").arg(points), "://icons/48x48/SetEle.png");
+    changed(tr("Smoothed profile with a Median filter of size %1").arg(points), "://icons/48x48/SetEle.png");
 }
 
 void CGisItemTrk::filterReplaceElevation()
@@ -258,7 +258,7 @@ void CGisItemTrk::filterReplaceElevation()
     }
 
     QPolygonF ele(line.size());
-    CMainWindow::self().getEelevationAt(line, ele);
+    CMainWindow::self().getElevationAt(line, ele);
 
     int cnt = 0;
     for(int i = 0; i < trk.segs.size(); i++)
@@ -274,7 +274,7 @@ void CGisItemTrk::filterReplaceElevation()
     }
 
     deriveSecondaryData();
-    changed(QObject::tr("Replaced elevation data with data from DEM files."), "://icons/48x48/SetEle.png");
+    changed(tr("Replaced elevation data with data from DEM files."), "://icons/48x48/SetEle.png");
 }
 
 void CGisItemTrk::filterOffsetElevation(int offset)
@@ -297,7 +297,7 @@ void CGisItemTrk::filterOffsetElevation(int offset)
     QString val, unit;
     IUnit::self().meter2elevation(offset, val, unit);
     deriveSecondaryData();
-    changed(QObject::tr("Offset elevation data by %1%2.").arg(val).arg(unit), "://icons/48x48/SetEle.png");
+    changed(tr("Offset elevation data by %1%2.").arg(val).arg(unit), "://icons/48x48/SetEle.png");
 }
 
 void CGisItemTrk::filterNewDate(const QDateTime& date)
@@ -316,7 +316,7 @@ void CGisItemTrk::filterNewDate(const QDateTime& date)
     }
 
     deriveSecondaryData();
-    changed(QObject::tr("Changed start of track to %1.").arg(date.toString()), "://icons/48x48/Time.png");
+    changed(tr("Changed start of track to %1.").arg(date.toString()), "://icons/48x48/Time.png");
 }
 
 void CGisItemTrk::filterObscureDate(int delta)
@@ -335,7 +335,7 @@ void CGisItemTrk::filterObscureDate(int delta)
         }
 
         deriveSecondaryData();
-        changed(QObject::tr("Remove timestamps."), "://icons/48x48/Time.png");
+        changed(tr("Remove timestamps."), "://icons/48x48/Time.png");
     }
     else
     {
@@ -358,7 +358,7 @@ void CGisItemTrk::filterObscureDate(int delta)
         }
 
         deriveSecondaryData();
-        changed(QObject::tr("Set artificial timestamps with delta of %1 sec.").arg(delta), "://icons/48x48/Time.png");
+        changed(tr("Set artificial timestamps with delta of %1 sec.").arg(delta), "://icons/48x48/Time.png");
     }
 }
 
@@ -382,8 +382,7 @@ void CGisItemTrk::filterSpeed(qreal speed)
                 continue;
             }
 
-            qreal dmsec = 1000 * pt.deltaDistance/speed;
-            timestamp   = timestamp.addMSecs(qRound(dmsec));
+            timestamp   = speed == 0 ? QDateTime() : timestamp.addMSecs(qRound(1000 * pt.deltaDistance/speed));
             pt.time     = timestamp;
         }
     }
@@ -391,5 +390,47 @@ void CGisItemTrk::filterSpeed(qreal speed)
     deriveSecondaryData();
     QString val, unit;
     IUnit::self().meter2speed(speed, val, unit);
-    changed(QObject::tr("Changed speed to %1%2.").arg(val).arg(unit), "://icons/48x48/Time.png");
+    changed(tr("Changed speed to %1%2.").arg(val).arg(unit), "://icons/48x48/Time.png");
+}
+
+void CGisItemTrk::filterSplitSegment()
+{
+    IGisProject * project = CGisWidget::self().selectProject();
+    if(nullptr == project)
+    {
+        return;
+    }
+
+    int part = 0;
+    for(const trkseg_t &seg : trk.segs)
+    {
+        if(0 < seg.pts.count())
+        {
+            qint32 idx1 = seg.pts[                  0].idxTotal;
+            qint32 idx2 = seg.pts[seg.pts.count() - 1].idxTotal;
+
+            new CGisItemTrk(tr("%1 (Segment %2)").arg(trk.name).arg(part), idx1, idx2, trk, project);
+            part++;
+        }
+    }
+}
+
+void CGisItemTrk::filterDeleteExtension(const QString &extStr)
+{
+    for(int i = 0; i < trk.segs.size(); i++)
+    {
+        trkseg_t& seg = trk.segs[i];
+
+        for(int n = 0; n < seg.pts.size(); n++)
+        {
+            seg.pts[n].extensions.remove(extStr);
+        }
+    }
+
+    extrema.remove(extStr);
+    existingExtensions.remove(extStr);
+    propHandler->setupData();
+
+    const CKnownExtension &ext = CKnownExtension::get(extStr);
+    changed(tr("Removed extension %1 from all Track Points").arg(ext.name), "://icons/48x48/FilterModifyExtension.png");
 }
