@@ -18,21 +18,25 @@
 
 #include "CMainWindow.h"
 #include "canvas/CCanvas.h"
+#include "helpers/CDraw.h"
 #include "helpers/CSettings.h"
 #include "map/CMapDraw.h"
 #include "map/CMapItem.h"
 #include "map/CMapList.h"
 #include "map/CMapPathSetup.h"
 #include "map/IMap.h"
+#include "map/cache/CDiskCache.h"
+#include "setup/IAppSetup.h"
 
 #include <QtGui>
 #include <QtWidgets>
 
 
+
 QList<CMapDraw*> CMapDraw::maps;
-QString CMapDraw::cachePath = QDir::home().absoluteFilePath(".QMapShack/");
+QString CMapDraw::cachePath = "";
 QStringList CMapDraw::mapPaths;
-QStringList CMapDraw::supportedFormats = QString("*.vrt|*.jnx|*.img|*.rmap|*.wmts|*.tms").split('|');
+QStringList CMapDraw::supportedFormats = QString("*.vrt|*.jnx|*.img|*.rmap|*.wmts|*.tms|*.gemf").split('|');
 
 
 CMapDraw::CMapDraw(CCanvas *parent)
@@ -40,8 +44,8 @@ CMapDraw::CMapDraw(CCanvas *parent)
 {
     mapList = new CMapList(canvas);
     CMainWindow::self().addMapList(mapList, canvas->objectName());
-    connect(canvas, SIGNAL(destroyed()), mapList, SLOT(deleteLater()));
-    connect(mapList, SIGNAL(sigChanged()), this, SLOT(emitSigCanvasUpdate()));
+    connect(canvas,  &CCanvas::destroyed,   mapList, &CMapList::deleteLater);
+    connect(mapList, &CMapList::sigChanged, this,    &CMapDraw::emitSigCanvasUpdate);
 
     buildMapList();
 
@@ -53,7 +57,7 @@ CMapDraw::~CMapDraw()
     maps.removeOne(this);
 }
 
-void CMapDraw::setProjection(const QString& proj)
+void CMapDraw::setProjection(const QString& proj) /* override */
 {
     // --- save the active maps
     QStringList keys;
@@ -69,6 +73,10 @@ void CMapDraw::setProjection(const QString& proj)
 void CMapDraw::setupMapPath()
 {
     QStringList paths = mapPaths;
+    if(cachePath.isEmpty())
+    {
+        cachePath =  IAppSetup::getPlatformInstance()->defaultCachePath();
+    }
     CMapPathSetup dlg(paths, cachePath);
     if(dlg.exec() != QDialog::Accepted)
     {
@@ -94,7 +102,7 @@ void CMapDraw::setupMapPath(const QStringList& paths)
 {
     mapPaths = paths;
 
-    foreach(CMapDraw * map, maps)
+    for(CMapDraw * map : maps)
     {
         QStringList keys;
         map->saveActiveMapsList(keys);
@@ -111,8 +119,13 @@ void CMapDraw::saveMapPath(QSettings& cfg)
 
 void CMapDraw::loadMapPath(QSettings& cfg)
 {
-    mapPaths    = cfg.value("mapPath", mapPaths).toStringList();
-    cachePath   = cfg.value("cachePath", cachePath).toString();
+    mapPaths  = cfg.value("mapPath", mapPaths).toStringList();
+    cachePath = cfg.value("cachePath", cachePath).toString();
+
+    if(cachePath.isEmpty())
+    {
+        cachePath = IAppSetup::getPlatformInstance()->defaultCachePath();
+    }
 }
 
 
@@ -204,7 +217,7 @@ bool CMapDraw::findPolylineCloseBy(const QPointF& pt1, const QPointF& pt2, qint3
     return res;
 }
 
-void CMapDraw::saveConfig(QSettings& cfg)
+void CMapDraw::saveConfig(QSettings& cfg) /* override */
 {
     // store group context for later use
     cfgGroup = cfg.group();
@@ -217,7 +230,7 @@ void CMapDraw::saveConfig(QSettings& cfg)
     cfg.endGroup();
 }
 
-void CMapDraw::loadConfig(QSettings& cfg)
+void CMapDraw::loadConfig(QSettings& cfg) /* override */
 {
     // store group context for later use
     cfgGroup = cfg.group();
@@ -246,17 +259,22 @@ void CMapDraw::buildMapList()
     QMutexLocker lock(&CMapItem::mutexActiveMaps);
     mapList->clear();
 
-    foreach(const QString &path, mapPaths)
+    QSet<QString> maps;
+
+    for(const QString &path : mapPaths)
     {
         QDir dir(path);
+
         // find available maps
-        foreach(const QString &filename, dir.entryList(supportedFormats, QDir::Files|QDir::Readable, QDir::Name))
+        for(const QString &filename : dir.entryList(supportedFormats, QDir::Files|QDir::Readable, QDir::Name))
         {
             QFileInfo fi(filename);
 
             CMapItem * item = new CMapItem(*mapList, this);
 
-            item->setText(0,fi.baseName().replace("_", " "));
+            maps.insert(fi.baseName());
+
+            item->setText(0, fi.baseName().replace("_", " "));
             item->filename = dir.absoluteFilePath(filename);
             item->updateIcon();
 
@@ -269,6 +287,9 @@ void CMapDraw::buildMapList()
             f.close();
         }
     }
+
+    CDiskCache::cleanupRemovedMaps(maps);
+
     mapList->updateHelpText();
 }
 
@@ -316,7 +337,7 @@ void CMapDraw::restoreActiveMapsList(const QStringList& keys)
 {
     QMutexLocker lock(&CMapItem::mutexActiveMaps);
 
-    foreach(const QString &key, keys)
+    for(const QString &key : keys)
     {
         for(int i = 0; i < mapList->count(); i++)
         {
@@ -341,7 +362,7 @@ void CMapDraw::restoreActiveMapsList(const QStringList& keys, QSettings& cfg)
 {
     QMutexLocker lock(&CMapItem::mutexActiveMaps);
 
-    foreach(const QString &key, keys)
+    for(const QString &key : keys)
     {
         for(int i = 0; i < mapList->count(); i++)
         {
@@ -367,11 +388,11 @@ void CMapDraw::reportStatusToCanvas(const QString& key, const QString& msg)
     canvas->reportStatus(key, msg);
 }
 
-void CMapDraw::drawt(IDrawContext::buffer_t& currentBuffer)
+void CMapDraw::drawt(IDrawContext::buffer_t& currentBuffer) /* override */
 {
     // iterate over all active maps and call the draw method
     CMapItem::mutexActiveMaps.lock();
-    if(mapList)
+    if(mapList && (mapList->count() != 0))
     {
         for(int i = 0; i < mapList->count(); i++)
         {
@@ -388,6 +409,33 @@ void CMapDraw::drawt(IDrawContext::buffer_t& currentBuffer)
             item->mapfile->draw(currentBuffer);
         }
     }
+    else
+    {
+        const int offMargin = currentBuffer.image.size().width()*0.1;
+        const int offTop    = currentBuffer.image.size().height()/3;
+        QPainter p(&currentBuffer.image);
+        p.setPen(Qt::black);
+        p.translate(offMargin,offTop);
+
+        QString msg = tr(
+            "There are no maps right now. "
+            "QMapShack is no fun without maps. "
+            "You can install maps by pressing the 'Help! I want maps!' button in the 'Maps' dock window. "
+            "Or you can press the F1 key to open the online documentation that tells you how to use QMapShack. "
+            "\n\n"
+            "If it's no fun, why don't you provide maps? Well to host maps ready for download and installation "
+            "requires a good server. And this is not a free service. The project lacks the money. Additionally "
+            "map and DEM data has a copyright. Therefore the copyright holder has to be asked prior to package "
+            "the data. This is not that easy as it might sound and for some data you have to pay royalties. "
+            "The project simply lacks resources to do this. And we think installing the stuff yourself is not "
+            "that much to ask from you. After all the software is ditributed without a fee."
+            );
+
+        QTextDocument doc;
+        doc.setPlainText(msg);
+        doc.setTextWidth(currentBuffer.image.width() - offMargin*2);
+        doc.drawContents(&p);
+    }
     CMapItem::mutexActiveMaps.unlock();
 }
 
