@@ -190,7 +190,7 @@ CMapWMTS::CMapWMTS(const QString &filename, CMapDraw *parent)
         QString str = xmlTileMatrixSet.namedItem("SupportedCRS").toElement().text();
 
         char * ptr1 = (char*)malloc(str.toLatin1().size() + 1);
-        char * ptr2 = 0;
+        char * ptr2 = nullptr;
 
         strncpy(ptr1,str.toLatin1().data(), str.toLatin1().size() + 1);
         OGRSpatialReference oSRS;
@@ -212,7 +212,7 @@ CMapWMTS::CMapWMTS(const QString &filename, CMapDraw *parent)
         free(ptr1);
         free(ptr2);
 
-        if(tileset.pjsrc == 0)
+        if(tileset.pjsrc == nullptr)
         {
             QMessageBox::warning(CMainWindow::getBestWidgetForParent(), tr("Error..."), tr("No georeference information found."));
             return;
@@ -247,8 +247,8 @@ CMapWMTS::CMapWMTS(const QString &filename, CMapDraw *parent)
     slotSetCachePath(QDir(CMapDraw::getCacheRoot()).absoluteFilePath(fi.baseName()));
 
     accessManager   = new QNetworkAccessManager(parent->thread());
-    connect(this, SIGNAL(sigQueueChanged()), this, SLOT(slotQueueChanged()));
-    connect(accessManager,SIGNAL(finished(QNetworkReply*)),this,SLOT(slotRequestFinished(QNetworkReply*)));
+    connect(this,          &CMapWMTS::sigQueueChanged,       this, &CMapWMTS::slotQueueChanged);
+    connect(accessManager, &QNetworkAccessManager::finished, this, &CMapWMTS::slotRequestFinished);
 
     name = fi.baseName().replace("_", " ");
 
@@ -271,17 +271,17 @@ void CMapWMTS::getLayers(QListWidget& list)
     }
 
     int i = 0;
-    foreach(const layer_t &layer, layers)
+    for(const layer_t &layer : layers)
     {
         QListWidgetItem * item = new QListWidgetItem(layer.title, &list);
         item->setCheckState(layer.enabled ? Qt::Checked : Qt::Unchecked);
         item->setData(Qt::UserRole, i++);
     }
 
-    connect(&list, SIGNAL(itemChanged(QListWidgetItem*)), this, SLOT(slotLayersChanged(QListWidgetItem*)));
+    connect(&list, &QListWidget::itemChanged, this, &CMapWMTS::slotLayersChanged);
 }
 
-void CMapWMTS::saveConfig(QSettings& cfg)
+void CMapWMTS::saveConfig(QSettings& cfg) /* override */
 {
     QMutexLocker lock(&mutex);
 
@@ -303,7 +303,7 @@ void CMapWMTS::saveConfig(QSettings& cfg)
     cfg.setValue("enabledLayers", enabled);
 }
 
-void CMapWMTS::loadConfig(QSettings& cfg)
+void CMapWMTS::loadConfig(QSettings& cfg) /* override */
 {
     QMutexLocker lock(&mutex);
 
@@ -323,7 +323,7 @@ void CMapWMTS::loadConfig(QSettings& cfg)
 
     // enable layers stored in configuration
     enabled = cfg.value("enabledLayers", enabled).toStringList();
-    foreach(const QString &str, enabled)
+    for(const QString &str : enabled)
     {
         int idx = str.toInt();
         if(idx < layers.size())
@@ -451,7 +451,7 @@ void CMapWMTS::slotRequestFinished(QNetworkReply* reply)
     slotQueueChanged();
 }
 
-void CMapWMTS::draw(IDrawContext::buffer_t& buf)
+void CMapWMTS::draw(IDrawContext::buffer_t& buf) /* override */
 {
     QMutexLocker lock(&mutex);
 
@@ -502,7 +502,7 @@ void CMapWMTS::draw(IDrawContext::buffer_t& buf)
     QRectF viewport(QPointF(x1,y1) * RAD_TO_DEG, QPointF(x2,y2) * RAD_TO_DEG);
 
     // draw layers
-    foreach(const layer_t &layer, layers)
+    for(const layer_t &layer : layers)
     {
         if(!layer.boundingBox.intersects(viewport) || !layer.enabled)
         {
@@ -529,7 +529,7 @@ void CMapWMTS::draw(IDrawContext::buffer_t& buf)
         QString tileMatrixId;
         QPointF s1 = (pt2 - pt1)/QPointF(buf.image.width(), buf.image.height());
         qreal d = NOFLOAT;
-        foreach(const QString &key, tileset.tilematrix.keys())
+        for(const QString &key : tileset.tilematrix.keys())
         {
             const tilematrix_t& tilematrix = tileset.tilematrix[key];
             qreal s2 = tilematrix.scale * 0.28e-3;
