@@ -56,7 +56,7 @@ void CMouseEditArea::drawLine(const QPolygonF &l, const QColor color, int width,
     p.drawPolygon(l);
 }
 
-IGisLine * CMouseEditArea::getGisLine()
+IGisLine * CMouseEditArea::getGisLine() const
 {
     return dynamic_cast<CGisItemOvlArea*>(CGisWidget::self().getItemByKey(key));
 }
@@ -84,22 +84,16 @@ void CMouseEditArea::slotCopyToNew()
         return;
     }
 
-    IGisProject * project = CGisWidget::self().selectProject();
-    if(project == 0)
-    {
-        return;
-    }
+    IGisProject * project = nullptr;
 
     QString name;
     CGisItemOvlArea * area = dynamic_cast<CGisItemOvlArea*>(CGisWidget::self().getItemByKey(key));
-    if(area != 0)
+    if(area != nullptr)
     {
         name = area->getName();
     }
 
-    name = QInputDialog::getText(CMainWindow::getBestWidgetForParent(), QObject::tr("Edit name..."), QObject::tr("Enter new area name."), QLineEdit::Normal, name);
-
-    if(name.isEmpty())
+    if(!IGisItem::getNameAndProject(name, project, tr("area")))
     {
         return;
     }
