@@ -65,7 +65,7 @@ void CMouseEditRte::mousePressEvent(QMouseEvent * e)
     IMouseEditLine::mousePressEvent(e);
 }
 
-IGisLine * CMouseEditRte::getGisLine()
+IGisLine * CMouseEditRte::getGisLine() const
 {
     return dynamic_cast<CGisItemRte*>(CGisWidget::self().getItemByKey(key));
 }
@@ -94,27 +94,22 @@ void CMouseEditRte::slotCopyToNew()
         return;
     }
 
-    IGisProject * project = CGisWidget::self().selectProject();
-    if(project == 0)
-    {
-        return;
-    }
 
-    /// @todo make this independent from track
+    IGisProject * project = nullptr;
+
     QString name;
     CGisItemRte * rte = dynamic_cast<CGisItemRte*>(CGisWidget::self().getItemByKey(key));
-    if(rte != 0)
+    if(rte != nullptr)
     {
         name = rte->getName();
     }
 
-    name = QInputDialog::getText(CMainWindow::getBestWidgetForParent(), QObject::tr("Edit name..."), QObject::tr("Enter new route name."), QLineEdit::Normal, name);
-
-    if(name.isEmpty())
+    if(!IGisItem::getNameAndProject(name, project, tr("route")))
     {
         return;
     }
 
+
     new CGisItemRte(points,name, project, NOIDX);
 
     canvas->resetMouse();
