@@ -60,26 +60,26 @@ void CMouseEditTrk::mousePressEvent(QMouseEvent * e)
     IMouseEditLine::mousePressEvent(e);
 }
 
-IGisLine * CMouseEditTrk::getGisLine()
+IGisLine * CMouseEditTrk::getGisLine() const
 {
     return dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
 }
 
 void CMouseEditTrk::slotAbort()
 {
-    canvas->reportStatus(key.item,"");
+    canvas->reportStatus(key.item, "");
     IMouseEditLine::slotAbortEx(false);
 }
 
 void CMouseEditTrk::slotCopyToOrig()
 {
-    canvas->reportStatus(key.item,"");
+    canvas->reportStatus(key.item, "");
 
     if(!isNewLine)
     {
-        int res = QMessageBox::warning(canvas, tr("Warning!"), tr("This will replace all data of the original by a simple line of coordinates. All other data will be lost permanently."), QMessageBox::Ok|QMessageBox::Abort, QMessageBox::Ok);
+        QMessageBox::StandardButton button = QMessageBox::warning(canvas, tr("Warning!"), tr("This will replace all data of the original by a simple line of coordinates. All other data will be lost permanently."), QMessageBox::Ok|QMessageBox::Abort, QMessageBox::Ok);
 
-        if(res != QMessageBox::Ok)
+        if(button != QMessageBox::Ok)
         {
             return;
         }
@@ -99,30 +99,25 @@ void CMouseEditTrk::slotCopyToNew()
         return;
     }
 
-    IGisProject * project = CGisWidget::self().selectProject();
-    if(project == 0)
-    {
-        return;
-    }
 
-    /// @todo make this independent from track
+    IGisProject * project = nullptr;
+
     QString name;
     CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
-    if(trk != 0)
+    if(trk != nullptr)
     {
         name = trk->getName();
     }
 
-    name = QInputDialog::getText(CMainWindow::getBestWidgetForParent(), QObject::tr("Edit name..."), QObject::tr("Enter new track name."), QLineEdit::Normal, name);
-
-    if(name.isEmpty())
+    if(!IGisItem::getNameAndProject(name, project, tr("track")))
     {
         return;
     }
 
-    CMainWindow::self().getEelevationAt(points);
 
-    new CGisItemTrk(points,name, project, NOIDX);
+    CMainWindow::self().getElevationAt(points);
+
+    new CGisItemTrk(points, name, project, NOIDX);
 
     canvas->resetMouse();
     canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
