@@ -36,11 +36,12 @@ CMouseNormal::CMouseNormal(CGisDraw *gis, CCanvas *canvas)
     screenUnclutter = new CScrOptUnclutter(this);
 
     menu = new QMenu(canvas);
-    menu->addAction(QIcon("://icons/32x32/AddWpt.png"), tr("Add Waypoint"), this, SLOT(slotAddWpt()));
-    menu->addAction(QIcon("://icons/32x32/AddTrk.png"), tr("Add Track"), this, SLOT(slotAddTrk()));
-    menu->addAction(QIcon("://icons/32x32/AddRte.png"), tr("Add Route"), this, SLOT(slotAddRte()));
-    menu->addAction(QIcon("://icons/32x32/AddArea.png"), tr("Add Area"), this, SLOT(slotAddArea()));
-
+    menu->addAction(QIcon("://icons/32x32/AddWpt.png"),  tr("Add Waypoint"), this, SLOT(slotAddWpt()));
+    menu->addAction(QIcon("://icons/32x32/AddTrk.png"),  tr("Add Track"),    this, SLOT(slotAddTrk()));
+    menu->addAction(QIcon("://icons/32x32/AddRte.png"),  tr("Add Route"),    this, SLOT(slotAddRte()));
+    menu->addAction(QIcon("://icons/32x32/AddArea.png"), tr("Add Area"),     this, SLOT(slotAddArea()));
+    menu->addSeparator();
+    menu->addAction(QIcon("://icons/32x32/SelectArea.png"), tr("Select Items On Map"), this, SLOT(slotSelectArea()));
     menu->addSeparator();
     menu->addAction(QIcon("://icons/32x32/Copy.png"), tr("Copy position"), this, SLOT(slotCopyPosition()));
     menu->addAction(QIcon("://icons/32x32/Copy.png"), tr("Copy position (Grid)"), this, SLOT(slotCopyPositionGrid()));
@@ -50,13 +51,13 @@ CMouseNormal::~CMouseNormal()
 {
 }
 
-void CMouseNormal::stopTracking()
+void CMouseNormal::stopTracking() const
 {
     const IGisItem::key_t& key = CGisItemTrk::getKeyUserFocus();
     if(!key.item.isEmpty())
     {
         CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
-        if(trk != 0)
+        if(trk != nullptr)
         {
             trk->setMouseFocusByPoint(NOPOINT, CGisItemTrk::eFocusMouseMove, "CMouseNormal");
         }
@@ -109,6 +110,7 @@ void CMouseNormal::mouseMoveEvent(QMouseEvent * e)
         case eStateIdle:
             CGisWidget::self().mouseMove(point);
 
+        //break; skip break intentionally
         case eStateHooverSingle:
         case eStateHooverMultiple:
         {
@@ -116,7 +118,7 @@ void CMouseNormal::mouseMoveEvent(QMouseEvent * e)
             if(!keyTrk.item.isEmpty())
             {
                 CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(keyTrk));
-                if(trk != 0)
+                if(trk != nullptr)
                 {
                     trk->setMouseFocusByPoint(point, CGisItemTrk::eFocusMouseMove, "CMouseNormal");
                 }
@@ -126,7 +128,7 @@ void CMouseNormal::mouseMoveEvent(QMouseEvent * e)
             if(!keyRte.item.isEmpty())
             {
                 CGisItemRte * rte = dynamic_cast<CGisItemRte*>(CGisWidget::self().getItemByKey(keyRte));
-                if(rte != 0)
+                if(rte != nullptr)
                 {
                     rte->setMouseFocusByPoint(point, CGisItemRte::eFocusMouseMove, "CMouseNormal");
                 }
@@ -157,7 +159,7 @@ void CMouseNormal::mouseReleaseEvent(QMouseEvent *e)
                 stateItemSel = eStateIdle;
 
                 IGisItem * item = CGisWidget::self().getItemByKey(screenUnclutter->getItemKey());
-                if(item)
+                if(nullptr != item)
                 {
                     item->treeWidget()->collapseAll();
                     item->treeWidget()->setCurrentItem(item);
@@ -183,11 +185,11 @@ void CMouseNormal::mouseReleaseEvent(QMouseEvent *e)
             case eStateUnclutterMultiple:
             {
                 const CScrOptUnclutter::item_t * scrOpt = screenUnclutter->selectItem(point);
-                if(scrOpt != 0)
+                if(scrOpt != nullptr)
                 {
                     IGisItem * item = CGisWidget::self().getItemByKey(scrOpt->key);
                     screenUnclutter->clear(); // CAUTION!! this will delete the object scrOpt is pointing to.
-                    scrOpt = 0;
+                    scrOpt = nullptr;
                     if(item)
                     {
                         item->treeWidget()->collapseAll();
@@ -210,9 +212,6 @@ void CMouseNormal::mouseReleaseEvent(QMouseEvent *e)
                 resetState();
                 break;
             }
-
-            default:
-                ;
             }
 
             canvas->update();
@@ -308,7 +307,7 @@ void CMouseNormal::draw(QPainter& p, CCanvas::redraw_e needsRedraw, const QRect
             break;
         }
 
-        foreach(IGisItem * item, items)
+        for(IGisItem * item : items)
         {
             item->drawHighlight(p);
             screenUnclutter->addItem(item);
@@ -346,34 +345,17 @@ void CMouseNormal::draw(QPainter& p, CCanvas::redraw_e needsRedraw, const QRect
 }
 
 
-void CMouseNormal::slotAddWpt()
+void CMouseNormal::slotAddWpt() const
 {
-    QString name;
-    QString icon;
     QPointF pt = point;
     gis->convertPx2Rad(pt);
     pt *= RAD_TO_DEG;
 
-    if(!CGisItemWpt::getNewWptData(pt, icon, name))
-    {
-        return;
-    }
-
-    IGisProject * project = CGisWidget::self().selectProject();
-    if(project == 0)
-    {
-        return;
-    }
-
-    QMutexLocker lock(&IGisItem::mutexItems);
-    CGisItemWpt * wpt = new CGisItemWpt(pt, name, icon, project);
-    wpt->edit();
-
-
+    CGisWidget::self().addWptByPos(pt);
     canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
 }
 
-void CMouseNormal::slotAddTrk()
+void CMouseNormal::slotAddTrk() const
 {
     QPointF pt = point;
     gis->convertPx2Rad(pt);
@@ -382,7 +364,7 @@ void CMouseNormal::slotAddTrk()
     canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
 }
 
-void CMouseNormal::slotAddRte()
+void CMouseNormal::slotAddRte() const
 {
     QPointF pt = point;
     gis->convertPx2Rad(pt);
@@ -391,7 +373,7 @@ void CMouseNormal::slotAddRte()
     canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
 }
 
-void CMouseNormal::slotAddArea()
+void CMouseNormal::slotAddArea() const
 {
     QPointF pt = point;
     gis->convertPx2Rad(pt);
@@ -400,7 +382,7 @@ void CMouseNormal::slotAddArea()
     canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
 }
 
-void CMouseNormal::slotCopyPosition()
+void CMouseNormal::slotCopyPosition() const
 {
     QPointF pt = point;
     gis->convertPx2Rad(pt);
@@ -412,7 +394,7 @@ void CMouseNormal::slotCopyPosition()
     clipboard->setText(position);
 }
 
-void CMouseNormal::slotCopyPositionGrid()
+void CMouseNormal::slotCopyPositionGrid() const
 {
     QString position;
     QPointF pt = point;
@@ -423,3 +405,8 @@ void CMouseNormal::slotCopyPositionGrid()
     QClipboard *clipboard = QApplication::clipboard();
     clipboard->setText(position);
 }
+
+void CMouseNormal::slotSelectArea() const
+{
+    canvas->setMouseSelect();
+}
