@@ -26,7 +26,7 @@
 CLineOpAddPoint::CLineOpAddPoint(SGisLine& points, CGisDraw *gis, CCanvas * canvas, IMouseEditLine * parent)
     : ILineOp(points, gis, canvas, parent)
 {
-    cursor = QCursor(QPixmap(":/cursors/cursorAdd.png"),0,0);
+    cursor = QCursor(QPixmap(":/cursors/cursorAdd.png"), 0, 0);
 }
 
 CLineOpAddPoint::~CLineOpAddPoint()
@@ -174,7 +174,7 @@ void CLineOpAddPoint::mouseMoveEventEx(QMouseEvent * e)
 
             if((idxFocus == 0) || (idxFocus == (points.size() - 1)))
             {
-                isPoint  = true;
+                isPoint = true;
             }
         }
     }
@@ -222,7 +222,7 @@ void CLineOpAddPoint::drawFg(QPainter& p)
         else
         {
             line << pt1.pixel;
-            foreach(const IGisLine::subpt_t& pt, pt1.subpts)
+            for(const IGisLine::subpt_t& pt : pt1.subpts)
             {
                 line << pt.pixel;
             }
