@@ -63,10 +63,10 @@ void CLineOpSelectRange::mousePressEventEx(QMouseEvent * e)
             }
 
             scrOptRangeLine = new CScrOptRangeLine(points[idx2nd].pixel, parentHandler, canvas);
-            connect(scrOptRangeLine->toolDelete, SIGNAL(clicked()), this, SLOT(slotDelete()));
-            connect(scrOptRangeLine->toolCalcRoute, SIGNAL(clicked()), this, SLOT(slotCalc()));
-            connect(scrOptRangeLine->toolDelete, SIGNAL(clicked()), scrOptRangeLine, SLOT(hide()));
-            connect(scrOptRangeLine->toolCalcRoute, SIGNAL(clicked()), scrOptRangeLine, SLOT(hide()));
+            connect(scrOptRangeLine->toolDelete,    &QToolButton::clicked, this,            &CLineOpSelectRange::slotDelete);
+            connect(scrOptRangeLine->toolCalcRoute, &QToolButton::clicked, this,            &CLineOpSelectRange::slotCalc);
+            connect(scrOptRangeLine->toolDelete,    &QToolButton::clicked, scrOptRangeLine.data(), &CScrOptRangeLine::hide);
+            connect(scrOptRangeLine->toolCalcRoute, &QToolButton::clicked, scrOptRangeLine.data(), &CScrOptRangeLine::hide);
 
             if(d < 2)
             {
@@ -129,12 +129,18 @@ void CLineOpSelectRange::mouseMoveEventEx(QMouseEvent * e)
 
 void CLineOpSelectRange::wheelEvent(QWheelEvent * e)
 {
-    resetState();
+    if(state == eState2nd)
+    {
+        resetState();
+    }
 }
 
 void CLineOpSelectRange::keyPressEvent(QKeyEvent * e)
 {
-    resetState();
+    if(state == eState2nd)
+    {
+        resetState();
+    }
 }
 
 
@@ -174,7 +180,7 @@ void CLineOpSelectRange::drawFg(QPainter& p)
             {
                 const IGisLine::point_t& point = points[i];
                 seg << point.pixel;
-                foreach(const IGisLine::subpt_t& subpt, point.subpts)
+                for(const IGisLine::subpt_t& subpt : point.subpts)
                 {
                     seg << subpt.pixel;
                 }
@@ -191,14 +197,14 @@ void CLineOpSelectRange::drawFg(QPainter& p)
             QRectF r(0,0,7,7);
             p.setPen(QPen(Qt::darkGreen,2));
             p.setBrush(Qt::darkGreen);
-            foreach(const QPointF &pt, seg)
+            for(const QPointF &pt : seg)
             {
                 r.moveCenter(pt);
                 p.drawRect(r);
             }
             p.setPen(Qt::NoPen);
             p.setBrush(Qt::black);
-            foreach(const QPointF &pt, seg)
+            for(const QPointF &pt : seg)
             {
                 r.moveCenter(pt);
                 p.drawRect(r);
@@ -215,9 +221,9 @@ void CLineOpSelectRange::resetState()
     {
         scrOptRangeLine->deleteLater();
     }
-    idxFocus    = NOIDX;
-    idx2nd      = NOIDX;
-    state       = eStateIdle;
+    idxFocus = NOIDX;
+    idx2nd   = NOIDX;
+    state    = eStateIdle;
 }
 
 void CLineOpSelectRange::slotDelete()
