@@ -36,11 +36,12 @@ ILineOp::ILineOp(SGisLine& points, CGisDraw *gis, CCanvas *canvas, IMouseEditLin
     timerRouting = new QTimer(this);
     timerRouting->setSingleShot(true);
     timerRouting->setInterval(400);
-    connect(timerRouting, SIGNAL(timeout()), this, SLOT(slotTimeoutRouting()));
+    connect(timerRouting, &QTimer::timeout, this, &ILineOp::slotTimeoutRouting);
 }
 
 ILineOp::~ILineOp()
 {
+    canvas->reportStatus("Routino", QString());
 }
 
 void ILineOp::cancelDelayedRouting()
@@ -101,7 +102,7 @@ void ILineOp::drawSinglePointLarge(const QPointF &pt, QPainter& p)
     p.drawRect(rectPoint);
 }
 
-void ILineOp::drawLeadLine(const QPolygonF& line, QPainter& p)
+void ILineOp::drawLeadLine(const QPolygonF& line, QPainter& p) const
 {
     p.setPen(QPen(Qt::yellow, 7, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
     p.drawPolyline(line);
@@ -119,11 +120,12 @@ void ILineOp::mousePressEvent(QMouseEvent * e)
 
         if(e->button() == Qt::LeftButton)
         {
-            lastPos     = pos;
-            mapMove     = true;
-            mapDidMove  = false;
+            lastPos    = pos;
+            mapMove    = true;
+            mapDidMove = false;
         }
     }
+    showRoutingErrorMessage(QString());
 }
 
 void ILineOp::mouseMoveEvent(QMouseEvent * e)
@@ -201,6 +203,40 @@ void ILineOp::updateLeadLines(qint32 idx)
     }
 }
 
+void ILineOp::showRoutingErrorMessage(const QString &msg) const
+{
+    if(msg.isEmpty())
+    {
+        canvas->reportStatus("Routino", QString());
+    }
+    else
+    {
+        canvas->reportStatus("Routino", QString("<span style='color: red;'><b>%1</b><br />%2</span>").arg(tr("Routing")).arg(msg));
+    }
+}
+
+void ILineOp::tryRouting(IGisLine::point_t& pt1, IGisLine::point_t& pt2) const
+{
+    QPolygonF subs;
+
+    try
+    {
+        if(CRouterSetup::self().calcRoute(pt1.coord, pt2.coord, subs) >= 0)
+        {
+            pt1.subpts.clear();
+            for(const QPointF &sub : subs)
+            {
+                pt1.subpts << IGisLine::subpt_t(sub);
+            }
+        }
+        showRoutingErrorMessage(QString());
+    }
+    catch(const QString &msg)
+    {
+        showRoutingErrorMessage(msg);
+    }
+}
+
 void ILineOp::finalizeOperation(qint32 idx)
 {
     if(idx == NOIDX)
@@ -213,33 +249,13 @@ void ILineOp::finalizeOperation(qint32 idx)
         CCanvas::setOverrideCursor(Qt::WaitCursor,"ILineOp::finalizeOperation");
         if(idx > 0)
         {
-            QPolygonF subs;
-            IGisLine::point_t& pt1 = points[idx - 1];
-            IGisLine::point_t& pt2 = points[idx];
-            if(CRouterSetup::self().calcRoute(pt1.coord, pt2.coord, subs) >= 0)
-            {
-                pt1.subpts.clear();
-                foreach(const QPointF &sub, subs)
-                {
-                    pt1.subpts << IGisLine::subpt_t(sub);
-                }
-            }
+            tryRouting(points[idx - 1], points[idx]);
         }
-
         if(idx < (points.size() - 1))
         {
-            QPolygonF subs;
-            IGisLine::point_t& pt1 = points[idx];
-            IGisLine::point_t& pt2 = points[idx + 1];
-            if(CRouterSetup::self().calcRoute(pt1.coord, pt2.coord, subs) >= 0)
-            {
-                pt1.subpts.clear();
-                foreach(const QPointF &sub, subs)
-                {
-                    pt1.subpts << IGisLine::subpt_t(sub);
-                }
-            }
+            tryRouting(points[idx], points[idx + 1]);
         }
+
         CCanvas::restoreOverrideCursor("ILineOp::finalizeOperation");
     }
     else if(parentHandler->useVectorRouting())
@@ -248,7 +264,7 @@ void ILineOp::finalizeOperation(qint32 idx)
         {
             IGisLine::point_t& pt1 = points[idx - 1];
             pt1.subpts.clear();
-            foreach(const QPointF &pt, subLineCoord1)
+            for(const QPointF &pt : subLineCoord1)
             {
                 pt1.subpts << IGisLine::subpt_t(pt);
             }
@@ -258,7 +274,7 @@ void ILineOp::finalizeOperation(qint32 idx)
         {
             IGisLine::point_t& pt1 = points[idx];
             pt1.subpts.clear();
-            foreach(const QPointF &pt, subLineCoord2)
+            for(const QPointF &pt : subLineCoord2)
             {
                 pt1.subpts << IGisLine::subpt_t(pt);
             }
@@ -269,7 +285,7 @@ void ILineOp::finalizeOperation(qint32 idx)
 }
 
 
-qint32 ILineOp::isCloseTo(const QPoint& pos)
+qint32 ILineOp::isCloseTo(const QPoint& pos) const
 {
     qint32 min = 30;
     qint32 idx = NOIDX;
@@ -289,8 +305,7 @@ qint32 ILineOp::isCloseTo(const QPoint& pos)
     return idx;
 }
 
-
-qint32 ILineOp::isCloseToLine(const QPoint& pos)
+qint32 ILineOp::isCloseToLine(const QPoint& pos) const
 {
     qint32 idx = NOIDX;
     qreal dist = 60;
@@ -307,7 +322,7 @@ qint32 ILineOp::isCloseToLine(const QPoint& pos)
         }
         else
         {
-            foreach(const IGisLine::subpt_t& pt, pt1.subpts)
+            for(const IGisLine::subpt_t& pt : pt1.subpts)
             {
                 line << pt.pixel;
             }
