@@ -18,12 +18,9 @@
 
 #include "CMainWindow.h"
 #include "GeoMath.h"
-#include "GeoMath.h"
 #include "canvas/CCanvas.h"
 #include "gis/CGisDraw.h"
-#include "gis/CGisWidget.h"
 #include "gis/IGisLine.h"
-#include "gis/rte/router/CRouterSetup.h"
 #include "gis/trk/CGisItemTrk.h"
 #include "helpers/CDraw.h"
 #include "helpers/CSettings.h"
@@ -54,10 +51,7 @@ IMouseEditLine::IMouseEditLine(const IGisItem::key_t &key, const QPointF& point,
 
 IMouseEditLine::IMouseEditLine(const IGisItem::key_t &key, IGisLine &src, bool enableStatus, const QString &type, CGisDraw *gis, CCanvas *parent)
     : IMouse(gis, parent)
-    , idxHistory(NOIDX)
     , key(key)
-    , doCanvasPanning(false)
-    , lineOp(0)
     , enableStatus(enableStatus)
     , type(type)
 {
@@ -95,25 +89,34 @@ IMouseEditLine::~IMouseEditLine()
     delete scrOptEditLine;
 }
 
+void IMouseEditLine::setCanvasPanning(bool enable)
+{
+    doCanvasPanning = enable;
+
+    scrOptEditLine->toolUndo->setVisible(!enable);
+    scrOptEditLine->toolRedo->setVisible(!enable);
+}
+
+
 void IMouseEditLine::commonSetup()
 {
     // create permanent line edit on screen options
     scrOptEditLine = new CScrOptEditLine(this);
-    connect(scrOptEditLine->pushSaveOrig,    SIGNAL(clicked()), this, SLOT(slotCopyToOrig()   ));
-    connect(scrOptEditLine->pushSaveNew,     SIGNAL(clicked()), this, SLOT(slotCopyToNew()    ));
-    connect(scrOptEditLine->pushAbort,       SIGNAL(clicked()), this, SLOT(slotAbort()        ));
+    connect(scrOptEditLine->pushSaveOrig,    &QPushButton::clicked, this, &IMouseEditLine::slotCopyToOrig   );
+    connect(scrOptEditLine->pushSaveNew,     &QPushButton::clicked, this, &IMouseEditLine::slotCopyToNew    );
+    connect(scrOptEditLine->pushAbort,       &QPushButton::clicked, this, &IMouseEditLine::slotAbort        );
 
-    connect(scrOptEditLine->toolMovePoint,   SIGNAL(clicked()), this, SLOT(slotMovePoint()    ));
-    connect(scrOptEditLine->toolSelectRange, SIGNAL(clicked()), this, SLOT(slotSelectRange()  ));
-    connect(scrOptEditLine->toolAddPoint,    SIGNAL(clicked()), this, SLOT(slotAddPoint()     ));
-    connect(scrOptEditLine->toolDeletePoint, SIGNAL(clicked()), this, SLOT(slotDeletePoint()  ));
+    connect(scrOptEditLine->toolMovePoint,   &QPushButton::clicked, this, &IMouseEditLine::slotMovePoint    );
+    connect(scrOptEditLine->toolSelectRange, &QPushButton::clicked, this, &IMouseEditLine::slotSelectRange  );
+    connect(scrOptEditLine->toolAddPoint,    &QPushButton::clicked, this, &IMouseEditLine::slotAddPoint     );
+    connect(scrOptEditLine->toolDeletePoint, &QPushButton::clicked, this, &IMouseEditLine::slotDeletePoint  );
 
-    connect(scrOptEditLine->toolNoRoute,     SIGNAL(clicked()), this, SLOT(slotNoRouting()    ));
-    connect(scrOptEditLine->toolAutoRoute,   SIGNAL(clicked()), this, SLOT(slotAutoRouting()  ));
-    connect(scrOptEditLine->toolVectorRoute, SIGNAL(clicked()), this, SLOT(slotVectorRouting()));
+    connect(scrOptEditLine->toolNoRoute,     &QPushButton::clicked, this, &IMouseEditLine::slotNoRouting    );
+    connect(scrOptEditLine->toolAutoRoute,   &QPushButton::clicked, this, &IMouseEditLine::slotAutoRouting  );
+    connect(scrOptEditLine->toolVectorRoute, &QPushButton::clicked, this, &IMouseEditLine::slotVectorRouting);
 
-    connect(scrOptEditLine->toolUndo,        SIGNAL(clicked()), this, SLOT(slotUndo()         ));
-    connect(scrOptEditLine->toolRedo,        SIGNAL(clicked()), this, SLOT(slotRedo()         ));
+    connect(scrOptEditLine->toolUndo,        &QPushButton::clicked, this, &IMouseEditLine::slotUndo         );
+    connect(scrOptEditLine->toolRedo,        &QPushButton::clicked, this, &IMouseEditLine::slotRedo         );
 
     SETTINGS;
     int mode = cfg.value("Route/drawMode",0).toInt();
@@ -146,12 +149,12 @@ void IMouseEditLine::abortStep()
     }
 }
 
-bool IMouseEditLine::useAutoRouting()
+bool IMouseEditLine::useAutoRouting() const
 {
     return scrOptEditLine->toolAutoRoute->isChecked();
 }
 
-bool IMouseEditLine::useVectorRouting()
+bool IMouseEditLine::useVectorRouting() const
 {
     return scrOptEditLine->toolVectorRoute->isChecked();
 }
@@ -165,7 +168,7 @@ void IMouseEditLine::drawLine(const QPolygonF &l, const QColor color, int width,
 
 void IMouseEditLine::draw(QPainter& p, CCanvas::redraw_e needsRedraw, const QRect &rect)
 {
-    if(needsRedraw & (CCanvas::eRedrawMouse|CCanvas::eRedrawGis))
+    if(needsRedraw & (CCanvas::eRedrawMouse | CCanvas::eRedrawGis))
     {
         points.updatePixel(gis);
 
@@ -173,15 +176,13 @@ void IMouseEditLine::draw(QPainter& p, CCanvas::redraw_e needsRedraw, const QRec
         pixelPts.clear();
         pixelSubs.clear();
 
-        for(int i = 0; i < points.size(); i++)
+        for(const IGisLine::point_t &pt : points)
         {
-            IGisLine::point_t& pt = points[i];
             pixelLine << pt.pixel;
             pixelPts << pt.pixel;
 
-            for(int n = 0; n < pt.subpts.size(); n++)
+            for(const IGisLine::subpt_t &sub : pt.subpts)
             {
-                IGisLine::subpt_t& sub = pt.subpts[n];
                 pixelLine << sub.pixel;
                 pixelSubs << sub.pixel;
             }
@@ -199,12 +200,12 @@ void IMouseEditLine::draw(QPainter& p, CCanvas::redraw_e needsRedraw, const QRec
 
     // draw magenta arrows (with white background)
     p.setBrush(Qt::magenta);
-    CDraw::arrows(pixelLine, QRectF(), p, 10, 80);
+    CDraw::arrows(pixelLine, QRectF(), p, 10, 80, 1.0);
 
     p.setPen(Qt::NoPen);
     p.setBrush(Qt::white);
     QRect r1(0,0,9,9);
-    foreach(const QPointF &pt, pixelPts)
+    for(const QPointF &pt : pixelPts)
     {
         r1.moveCenter(pt.toPoint());
         p.drawRect(r1);
@@ -215,13 +216,13 @@ void IMouseEditLine::draw(QPainter& p, CCanvas::redraw_e needsRedraw, const QRec
     p.setPen(Qt::NoPen);
     p.setBrush(Qt::black);
     QRect r2(0,0,7,7);
-    foreach(const QPointF &pt, pixelPts)
+    for(const QPointF &pt : pixelPts)
     {
         r2.moveCenter(pt.toPoint());
         p.drawRect(r2);
     }
 
-    foreach(const QPointF &pt, pixelSubs)
+    for(const QPointF &pt : pixelSubs)
     {
         p.drawEllipse(pt, 2, 2);
     }
@@ -271,6 +272,9 @@ void IMouseEditLine::mouseMoveEvent(QMouseEvent * e)
     }
 
     lineOp->mouseMoveEvent(e);
+
+    canvas->displayInfo(point);
+    canvas->update();
 }
 
 void IMouseEditLine::mouseReleaseEvent(QMouseEvent *e)
@@ -303,7 +307,7 @@ void IMouseEditLine::slotDeletePoint()
     delete lineOp;
     lineOp = new CLineOpDeletePoint(points, gis, canvas, this);
     changeCursor();
-    doCanvasPanning = false;
+    setCanvasPanning(false);
 }
 
 void IMouseEditLine::slotSelectRange()
@@ -312,7 +316,7 @@ void IMouseEditLine::slotSelectRange()
     delete lineOp;
     lineOp = new CLineOpSelectRange(points, gis, canvas, this);
     changeCursor();
-    doCanvasPanning = false;
+    setCanvasPanning(false);
 }
 
 void IMouseEditLine::slotMovePoint()
@@ -321,7 +325,7 @@ void IMouseEditLine::slotMovePoint()
     delete lineOp;
     lineOp = new CLineOpMovePoint(points, gis, canvas, this);
     changeCursor();
-    doCanvasPanning = false;
+    setCanvasPanning(false);
 }
 
 void IMouseEditLine::slotAddPoint()
@@ -330,12 +334,13 @@ void IMouseEditLine::slotAddPoint()
     delete lineOp;
     lineOp = new CLineOpAddPoint(points, gis, canvas, this);
     changeCursor();
-    doCanvasPanning = false;
+    setCanvasPanning(false);
 }
 
 void IMouseEditLine::slotNoRouting()
 {
     canvas->reportStatus(key.item, tr("<b>No Routing</b><br/>All points will be connected with a straight line.<br/>"));
+    canvas->reportStatus("Routino", QString());
 }
 
 void IMouseEditLine::slotAutoRouting()
@@ -346,6 +351,7 @@ void IMouseEditLine::slotAutoRouting()
 void IMouseEditLine::slotVectorRouting()
 {
     canvas->reportStatus(key.item, tr("<b>Vector Routing</b><br/>Connect points with a line from a loaded vector map if possible.<br/>"));
+    canvas->reportStatus("Routino", QString());
 }
 
 
@@ -364,7 +370,7 @@ void IMouseEditLine::slotAbortEx(bool showMB)
     bool doAbort = ( idxHistory == 0 ) || !showMB;
     if(!doAbort)
     {
-        doAbort = (QMessageBox::Yes == QMessageBox::question(NULL, "Abort", "Do you really want to abort?\nAny modifications done will be discarded.", QMessageBox::Yes | QMessageBox::No));
+        doAbort = (QMessageBox::Yes == QMessageBox::question(nullptr, "Abort", "Do you really want to abort?\nAny modifications done will be discarded.", QMessageBox::Yes | QMessageBox::No));
     }
 
     if(doAbort)
@@ -378,11 +384,11 @@ void IMouseEditLine::slotCopyToOrig()
 {
     QMutexLocker lock(&IGisItem::mutexItems);
 
-    IGisLine * l = getGisLine();
-    if(l != 0)
+    IGisLine * line = getGisLine();
+    if(line != nullptr)
     {
-        CMainWindow::self().getEelevationAt(points);
-        l->setDataFromPolyline(points);
+        CMainWindow::self().getElevationAt(points);
+        line->setDataFromPolyline(points);
     }
 
 
@@ -420,6 +426,11 @@ void IMouseEditLine::storeToHistory(const SGisLine& line)
 
 void IMouseEditLine::slotUndo()
 {
+    if(lineOp != nullptr)
+    {
+        lineOp->abortStep();
+    }
+
     if(idxHistory > 0)
     {
         idxHistory--;
@@ -434,12 +445,17 @@ void IMouseEditLine::slotUndo()
 
 void IMouseEditLine::slotRedo()
 {
+    // abort operation
+    if(lineOp != nullptr)
+    {
+        lineOp->abortStep();
+    }
+
     if(idxHistory < (history.size() - 1))
     {
         idxHistory++;
     }
 
-
     points = history[idxHistory];
 
     scrOptEditLine->toolRedo->setEnabled(idxHistory < (history.size() - 1));
@@ -451,7 +467,7 @@ void IMouseEditLine::updateStatus()
 {
     if(!enableStatus || points.isEmpty())
     {
-        canvas->reportStatus("IMouseEditLine","");
+        canvas->reportStatus("IMouseEditLine", QString());
         return;
     }
 
@@ -462,19 +478,12 @@ void IMouseEditLine::updateStatus()
     qreal dist  = 0;
 
     qreal lastEle = points[0].ele;
-    qreal absDelta;
-    qreal delta;
-
-
     QPointF lastPos = points[0].coord;
 
-    for(int i = 0; i < points.size(); i++)
+    for(const IGisLine::point_t &pt1 : points)
     {
-        const IGisLine::point_t& pt1 = points[i];
-
-        delta       = pt1.ele - lastEle;
-        absDelta    = qAbs(delta);
-        if(absDelta > ASCEND_THRESHOLD)
+        qreal delta = pt1.ele - lastEle;
+        if(qAbs(delta) > ASCEND_THRESHOLD)
         {
             if(delta > 0)
             {
@@ -490,11 +499,10 @@ void IMouseEditLine::updateStatus()
         dist += GPS_Math_Distance(lastPos.x(), lastPos.y(), pt1.coord.x(), pt1.coord.y());
         lastPos = pt1.coord;
 
-        foreach(const IGisLine::subpt_t& pt, pt1.subpts)
+        for(const IGisLine::subpt_t& pt : pt1.subpts)
         {
-            delta       = pt.ele - lastEle;
-            absDelta    = qAbs(delta);
-            if(absDelta > ASCEND_THRESHOLD)
+            delta = pt.ele - lastEle;
+            if(qAbs(delta) > ASCEND_THRESHOLD)
             {
                 if(delta > 0)
                 {
