@@ -23,16 +23,19 @@
 #include "plot/CPlotProfile.h"
 #include "units/IUnit.h"
 
+
 #include <proj_api.h>
 
 CPlotProfile::CPlotProfile(QWidget * parent)
-    : IPlot(0, CPlotData::eAxisLinear, eModeNormal, parent)
+    : IPlot(nullptr, CPlotData::eAxisLinear, eModeNormal, parent)
 {
 }
 
-CPlotProfile::CPlotProfile(CGisItemTrk *trk, mode_e mode, QWidget *parent)
+CPlotProfile::CPlotProfile(CGisItemTrk *trk, CLimit& lim, mode_e mode, QWidget *parent)
     : IPlot(trk, CPlotData::eAxisLinear, mode, parent)
+    , limit(&lim)
 {
+    connect(limit, &CLimit::sigChanged, this, &CPlotProfile::setLimits);
     setWindowTitle(trk->getNameEx());
     updateData();
 }
@@ -41,10 +44,17 @@ CPlotProfile::~CPlotProfile()
 {
 }
 
-void CPlotProfile::setTrack(CGisItemTrk * track)
+void CPlotProfile::setTrack(CGisItemTrk * track, CLimit &lim)
 {
     trk = track;
-    trk->registerPlot(this);
+    trk->registerVisual(this);
+
+    if(limit)
+    {
+        disconnect(limit, &CLimit::sigChanged, this, &CPlotProfile::setLimits);
+    }
+    limit = &lim;
+    connect(limit, &CLimit::sigChanged, this, &CPlotProfile::setLimits);
 
     updateData();
 }
@@ -53,6 +63,11 @@ void CPlotProfile::updateData()
 {
     clear();
 
+    qreal scale;
+    QString unit;
+    IUnit::self().meter2unit(trk->getTotalDistance(), scale, unit);
+    setXTicScale(scale);
+
     if(mode == eModeIcon)
     {
         setXLabel(trk->getName());
@@ -60,7 +75,7 @@ void CPlotProfile::updateData()
     }
     else
     {
-        setXLabel(tr("distance [%1]").arg(IUnit::self().baseunit));
+        setXLabel(tr("distance [%1]").arg(unit));
         setYLabel(tr("alt. [%1]").arg(IUnit::self().baseunit));
     }
 
@@ -72,9 +87,9 @@ void CPlotProfile::updateData()
 
     qreal basefactor = IUnit::self().basefactor;
     const CGisItemTrk::trk_t& t = trk->getTrackData();
-    foreach (const CGisItemTrk::trkseg_t& seg, t.segs)
+    for(const CGisItemTrk::trkseg_t& seg : t.segs)
     {
-        foreach(const CGisItemTrk::trkpt_t& trkpt, seg.pts)
+        for(const CGisItemTrk::trkpt_t& trkpt : seg.pts)
         {
             if(trkpt.flags & CGisItemTrk::trkpt_t::eHidden)
             {
@@ -90,7 +105,7 @@ void CPlotProfile::updateData()
             coords << QPointF(trkpt.lon * DEG_TO_RAD, trkpt.lat * DEG_TO_RAD);
             lineDem << QPointF(trkpt.distance, NOFLOAT);
 
-            if(project == 0 || trkpt.keyWpt.item.isEmpty() || (mode == eModeIcon))
+            if(nullptr == project || trkpt.keyWpt.item.isEmpty() || (mode == eModeIcon))
             {
                 continue;
             }
@@ -107,7 +122,7 @@ void CPlotProfile::updateData()
         }
     }
 
-    CMainWindow::self().getEelevationAt(coords, lineDem);
+    CMainWindow::self().getElevationAt(coords, lineDem);
 
     newLine(lineEle, "GPS");
     if(!lineDem.isEmpty())
@@ -120,24 +135,34 @@ void CPlotProfile::updateData()
 
 void CPlotProfile::setMouseFocus(const CGisItemTrk::trkpt_t * ptMouseMove)
 {
-    if(ptMouseMove == 0)
+    if(nullptr == ptMouseMove)
     {
-        if(posMouse != NOPOINT)
+        if(posMouse1 != NOPOINT)
         {
-            posMouse = NOPOINT;
+            posMouse1 = NOPOINT;
             needsRedraw = true;
         }
     }
     else
     {
-        if(posMouse == NOPOINT)
+        if(posMouse1 == NOPOINT)
         {
             needsRedraw = true;
         }
 
-        posMouse.rx() = left  + data->x().val2pt(ptMouseMove->distance);
-        posMouse.ry() = top  +  data->y().val2pt(ptMouseMove->ele);
+        posMouse1.rx() = left + data->x().val2pt(ptMouseMove->distance);
+        posMouse1.ry() = top  + data->y().val2pt(ptMouseMove->ele);
     }
     update();
 }
 
+void CPlotProfile::setLimits()
+{
+    IPlot::setLimits();
+    data->ymin = limit->getMin() == NOFLOAT ? data->ymin : limit->getMin();
+    data->ymax = limit->getMax() == NOFLOAT ? data->ymax : limit->getMax();
+
+    data->y().setLimits(data->ymin, data->ymax);
+    resetZoom();
+    update();
+}
