@@ -57,20 +57,20 @@ void ITrack::setupProjection(const QRectF& boundingBox)
     if(pjsrc)
     {
         pj_free(pjsrc);
-        pjsrc = 0;
+        pjsrc = nullptr;
     }
 
     if(boundingBox.top() > (60*DEG_TO_RAD))
     {
-        pjsrc =  pj_init_plus("+init=epsg:32661");
+        pjsrc = pj_init_plus("+init=epsg:32661");
     }
     else if(boundingBox.bottom() < (-60*DEG_TO_RAD))
     {
-        pjsrc =  pj_init_plus("+init=epsg:32761");
+        pjsrc = pj_init_plus("+init=epsg:32761");
     }
     else
     {
-        pjsrc =  pj_init_plus("+init=epsg:3857");
+        pjsrc = pj_init_plus("+init=epsg:3857");
     }
 }
 
@@ -94,7 +94,7 @@ void ITrack::setTrack(const QPolygonF& track)
 
 void ITrack::updateData()
 {
-    if((pjsrc == 0) || (trk == 0 && coords.isEmpty()))
+    if((pjsrc == nullptr) || (nullptr == trk && coords.isEmpty()))
     {
         return;
     }
@@ -103,9 +103,9 @@ void ITrack::updateData()
     {
         coords.clear();
         const CGisItemTrk::trk_t& t = trk->getTrackData();
-        foreach (const CGisItemTrk::trkseg_t& seg, t.segs)
+        for(const CGisItemTrk::trkseg_t& seg : t.segs)
         {
-            foreach(const CGisItemTrk::trkpt_t& trkpt, seg.pts)
+            for(const CGisItemTrk::trkpt_t& trkpt : seg.pts)
             {
                 if(trkpt.flags & CGisItemTrk::trkpt_t::eHidden)
                 {
@@ -118,7 +118,7 @@ void ITrack::updateData()
     }
 
     line.clear();
-    foreach(const QPointF &trkpt, coords)
+    for(const QPointF &trkpt : coords)
     {
         QPointF pt(trkpt.x(), trkpt.y());
         pj_transform(pjtar, pjsrc, 1, 0, &pt.rx(), &pt.ry(), 0);
