@@ -52,7 +52,8 @@ CQmsDb::CQmsDb(const QString &filename, CImportDatabase *parent)
         gui->stdErr(tr("Remove existing file %1").arg(filename));
         QFile::remove(filename);
     }
-    valid = setupDB(filename, "qlgt2qms");
+    QString error;
+    valid = setupDB(filename, "qlgt2qms", error);
     if(!valid)
     {
         return;
@@ -127,7 +128,7 @@ void CQmsDb::addFolder(CQlgtFolder& folder)
         generate key and info text properly
      */
     CDBProject project(folder);
-    foreach(quint64 id, folder.items)
+    for(quint64 id : folder.items)
     {
         quint64 idChild = mapItemIDs[id];
         query.prepare("SELECT type FROM items WHERE id=:id");
@@ -171,9 +172,9 @@ void CQmsDb::addFolder(CQlgtFolder& folder)
     in.setVersion(QDataStream::Qt_5_2);
     project >> in;
 
-    query.prepare("INSERT INTO folders (type, key, name, comment, locked, data) VALUES (:type, :key, :name, :comment, :locked, :data)");
+    query.prepare("INSERT INTO folders (type, keyqms, name, comment, locked, data) VALUES (:type, :keyqms, :name, :comment, :locked, :data)");
     query.bindValue(":type",    mapFolderTypes[folder.type]);
-    query.bindValue(":key",     project.getKey());
+    query.bindValue(":keyqms",     project.getKey());
     query.bindValue(":name",    project.getName());
     query.bindValue(":comment", project.getInfo());
     query.bindValue(":locked",  folder.locked);
@@ -259,13 +260,14 @@ quint64 CQmsDb::store(IGisItem& item)
 
     QSqlQuery query(db);
     // item is unknown to database -> create item in database
-    query.prepare("INSERT INTO items (type, key, icon, name, comment, data) VALUES (:type, :key, :icon, :name, :comment, :data)");
+    query.prepare("INSERT INTO items (type, keyqms, icon, name, comment, data, hash) VALUES (:type, :keyqms, :icon, :name, :comment, :data, :hash)");
     query.bindValue(":type",    item.type());
-    query.bindValue(":key",     item.getKey().item);
+    query.bindValue(":keyqms",     item.getKey().item);
     query.bindValue(":icon",    buffer.data());
     query.bindValue(":name",    item.getName());
     query.bindValue(":comment", item.getInfo());
     query.bindValue(":data", data);
+    query.bindValue(":hash", item.getHash());
     QUERY_EXEC(return 0);
 
     query.prepare("SELECT last_insert_rowid() from items");
