@@ -37,7 +37,7 @@ inline qreal readFloat(float val)
 }
 
 CDBProject::CDBProject(CQlgtFolder& folder)
-    : IGisProject(eTypeDb, "", (CGisListWks*)0)
+    : IGisProject(eTypeDb, "", (CGisListWks*) nullptr)
 {
     metadata.name = folder.name;
     if(folder.diary)
@@ -47,7 +47,7 @@ CDBProject::CDBProject(CQlgtFolder& folder)
 }
 
 CGisItemWpt::CGisItemWpt(const CQlgtWpt& wpt1)
-    : IGisItem(0, eTypeWpt, NOIDX)
+    : IGisItem(nullptr, eTypeWpt, NOIDX)
 {
     qreal direction;
     QDateTime time = QDateTime::fromTime_t(wpt1.timestamp,QTimeZone("UTC"));
@@ -101,24 +101,21 @@ CGisItemWpt::CGisItemWpt(const CQlgtWpt& wpt1)
         geocache.state      = wpt1.geocache.state;
         geocache.locale     = wpt1.geocache.locale;
 
-        foreach(const CQlgtWpt::geocachelog_t& log1, wpt1.geocache.logs)
+        for(const CQlgtWpt::geocachelog_t& log1 : wpt1.geocache.logs)
         {
             CGisItemWpt::geocachelog_t log;
             log.id          = log1.id;
             IUnit::parseTimestamp(log1.date, log.date);
             log.type        = log1.type;
-            ;
             log.finderId    = log1.finderId;
-            ;
             log.finder      = log1.finder;
-            ;
             log.textIsHtml  = log1.text.contains("</");
             log.text        = log1.text;
             geocache.logs << log;
         }
     }
 
-    foreach(const CQlgtWpt::image_t& image1, wpt1.images)
+    for(const CQlgtWpt::image_t& image1 : wpt1.images)
     {
         CGisItemWpt::image_t image;
 
@@ -137,7 +134,7 @@ CGisItemWpt::CGisItemWpt(const CQlgtWpt& wpt1)
 
 
 CGisItemTrk::CGisItemTrk(const CQlgtTrack &trk1)
-    : IGisItem(0, eTypeTrk, NOIDX)
+    : IGisItem(nullptr, eTypeTrk, NOIDX)
     , activities(this)
 {
     trk.name        = trk1.name;
@@ -148,7 +145,7 @@ CGisItemTrk::CGisItemTrk(const CQlgtTrack &trk1)
 
     bool hasExtData = trk1.hasExt1Data();
     trkseg_t seg;
-    foreach(const CQlgtTrack::pt_t& pt1, trk1.track)
+    for(const CQlgtTrack::pt_t& pt1 : trk1.track)
     {
         trkpt_t pt;
         QDateTime time = QDateTime::fromTime_t(pt1._timestamp,QTimeZone("UTC"));
@@ -195,7 +192,7 @@ CGisItemTrk::CGisItemTrk(const CQlgtTrack &trk1)
         history.events << history_event_t();
         history_event_t& event = history.events.last();
         event.time      = QDateTime::currentDateTimeUtc();
-        event.comment   = QObject::tr("Copy flag information from QLandkarte GT track");
+        event.comment   = tr("Copy flag information from QLandkarte GT track");
         event.icon      = "://icons/48x48/PointHide.png";
 
         QDataStream stream(&event.data, QIODevice::WriteOnly);
@@ -222,7 +219,7 @@ CGisItemTrk::CGisItemTrk(const IQlgtOverlay& ovl)
     trk.color   = ovl.color.name();
 
     trkseg_t seg;
-    foreach(const IQlgtOverlay::pt_t& pt1, ovl.points)
+    for(const IQlgtOverlay::pt_t& pt1 : ovl.points)
     {
         trkpt_t pt;
         pt.lon = pt1.u * RAD_TO_DEG;
@@ -257,7 +254,7 @@ CGisItemOvlArea::CGisItemOvlArea(const IQlgtOverlay& ovl)
     area.style      = ovl.style;
     area.opacity    = ovl.opacity != 255;
 
-    foreach(const IQlgtOverlay::pt_t& pt1, ovl.points)
+    for(const IQlgtOverlay::pt_t& pt1 : ovl.points)
     {
         pt_t pt;
         pt.lon = pt1.u * RAD_TO_DEG;
@@ -284,7 +281,7 @@ CGisItemRte::CGisItemRte(const CQlgtRoute& rte1)
     QPointF focus;
     QPixmap icon = getWptIconByName(rte1.iconString, focus);
 
-    foreach (const CQlgtRoute::pt_t& pt1, rte1.priRoute)
+    for(const CQlgtRoute::pt_t& pt1 : rte1.priRoute)
     {
         rtept_t pt;
         pt.lon      = pt1.lon;
