@@ -27,9 +27,9 @@ CMapVrtBuilder::CMapVrtBuilder(QWidget *parent)
     setupUi(this);
     setObjectName(tr("Build GDAL VRT"));
 
-    connect(toolSourceFiles, SIGNAL(clicked()), this, SLOT(slotSelectSourceFiles()));
-    connect(toolTargetFile, SIGNAL(clicked()), this, SLOT(slotSelectTargetFile()));
-    connect(pushStart, SIGNAL(clicked()), this, SLOT(slotStart()));
+    connect(toolSourceFiles, &QToolButton::clicked, this, &CMapVrtBuilder::slotSelectSourceFiles);
+    connect(toolTargetFile,  &QToolButton::clicked, this, &CMapVrtBuilder::slotSelectTargetFile);
+    connect(pushStart,       &QPushButton::clicked, this, &CMapVrtBuilder::slotStart);
 
     pushStart->setDisabled(true);
 }
@@ -54,7 +54,7 @@ void CMapVrtBuilder::slotSelectSourceFiles()
     cfg.setValue("VrtBuilder/sourcePath", path);
 
     listWidget->clear();
-    foreach(const QString &file, files)
+    for(const QString &file : files)
     {
         new QListWidgetItem(QIcon("://icons/32x32/Map.png"), file, listWidget);
     }
@@ -67,7 +67,7 @@ void CMapVrtBuilder::slotSelectTargetFile()
     SETTINGS;
     QString path = cfg.value("VrtBuilder/targetPath",QDir::homePath()).toString();
 
-    QString file = QFileDialog::getSaveFileName(this, tr("Select target file..."), path);
+    QString file = QFileDialog::getSaveFileName(this, tr("Select target file..."), path, "GDAL vrt (*.vrt)");
     if(file.isEmpty())
     {
         return;
@@ -99,7 +99,7 @@ void CMapVrtBuilder::slotStart()
     QStringList args;
     args << labelTargetFilename->text();
 
-    foreach(const QListWidgetItem * item, listWidget->findItems("*", Qt::MatchWildcard))
+    for(const QListWidgetItem * item : listWidget->findItems("*", Qt::MatchWildcard))
     {
         args << item->text();
     }
@@ -113,5 +113,4 @@ void CMapVrtBuilder::finished(int exitCode, QProcess::ExitStatus status)
     textBrowser->setTextColor(Qt::darkGreen);
     textBrowser->append(tr("!!! done !!!\n"));
     pushStart->setEnabled(true);
-    return;
 }
