@@ -24,7 +24,7 @@
 
 #include <QtWidgets>
 #include <proj_api.h>
-IUnit * IUnit::m_self = 0;
+const IUnit * IUnit::m_self = nullptr;
 
 const QPointF NOPOINTF(NOFLOAT, NOFLOAT);
 const QPoint NOPOINT (NOINT, NOINT);
@@ -421,15 +421,15 @@ const char * IUnit::tblTimezone[] =
 
 const int N_TIMEZONES = sizeof(IUnit::tblTimezone)/sizeof(const char*);
 
-QRegExp IUnit::reCoord1("^\\s*([N|S]){1}\\W*([0-9]+)\\W*([0-9]+\\.[0-9]+)\\s+([E|W|O]){1}\\W*([0-9]+)\\W*([0-9]+\\.[0-9]+)\\s*$");
+const QRegExp IUnit::reCoord1("^\\s*([N|S]){1}\\W*([0-9]+)\\W*([0-9]+\\.[0-9]+)\\s+([E|W|O]){1}\\W*([0-9]+)\\W*([0-9]+\\.[0-9]+)\\s*$");
 
-QRegExp IUnit::reCoord2("^\\s*([N|S]){1}\\s*([0-9]+\\.[0-9]+)\\W*\\s+([E|W|O]){1}\\s*([0-9]+\\.[0-9]+)\\W*\\s*$");
+const QRegExp IUnit::reCoord2("^\\s*([N|S]){1}\\s*([0-9]+\\.[0-9]+)\\W*\\s+([E|W|O]){1}\\s*([0-9]+\\.[0-9]+)\\W*\\s*$");
 
-QRegExp IUnit::reCoord3("^\\s*([-0-9]+\\.[0-9]+)\\s+([-0-9]+\\.[0-9]+)\\s*$");
+const QRegExp IUnit::reCoord3("^\\s*([-0-9]+\\.[0-9]+)\\s+([-0-9]+\\.[0-9]+)\\s*$");
 
-QRegExp IUnit::reCoord4("^\\s*([N|S]){1}\\s*([0-9]+)\\W+([0-9]+)\\W+([0-9]+\\.[0-9]+)\\W*([E|W|O]){1}\\W*([0-9]+)\\W+([0-9]+)\\W+([0-9]+\\.[0-9]+)\\W*\\s*$");
+const QRegExp IUnit::reCoord4("^\\s*([N|S]){1}\\s*([0-9]+)\\W+([0-9]+)\\W+([0-9]+\\.[0-9]+)\\W*([E|W|O]){1}\\W*([0-9]+)\\W+([0-9]+)\\W+([0-9]+\\.[0-9]+)\\W*\\s*$");
 
-QRegExp IUnit::reCoord5("^\\s*([-0-9]+\\.[0-9]+)([N|S])\\s+([-0-9]+\\.[0-9]+)([W|E])\\s*$");
+const QRegExp IUnit::reCoord5("^\\s*([-0-9]+\\.[0-9]+)([N|S])\\s+([-0-9]+\\.[0-9]+)([W|E])\\s*$");
 
 IUnit::IUnit(const type_e &type, const QString& baseunit, const qreal basefactor, const QString& speedunit, const qreal speedfactor, QObject * parent)
     : QObject(parent)
@@ -440,7 +440,7 @@ IUnit::IUnit(const type_e &type, const QString& baseunit, const qreal basefactor
     , speedfactor(speedfactor)
 {
     //there can be only one...
-    if(m_self)
+    if(nullptr != m_self)
     {
         delete m_self;
     }
@@ -448,10 +448,6 @@ IUnit::IUnit(const type_e &type, const QString& baseunit, const qreal basefactor
 }
 
 
-IUnit::~IUnit()
-{
-}
-
 void IUnit::setUnitType(type_e t, QObject * parent)
 {
     switch(t)
@@ -473,13 +469,13 @@ void IUnit::setUnitType(type_e t, QObject * parent)
     cfg.setValue("Units/type",t);
 }
 
-void IUnit::meter2speed(qreal meter, QString& val, QString& unit)
+void IUnit::meter2speed(qreal meter, QString& val, QString& unit) const
 {
     val.sprintf("%2.2f",meter * speedfactor);
     unit = speedunit;
 }
 
-void IUnit::seconds2time(quint32 ttime, QString& val, QString& unit)
+void IUnit::seconds2time(quint32 ttime, QString& val, QString& unit) const
 {
     QTime time(0,0,0);
     quint32 days  = ttime / 86400;
@@ -503,13 +499,7 @@ bool IUnit::parseTimestamp(const QString &time, QDateTime &datetime)
     int tzoffset;
     datetime = parseTimestamp(time, tzoffset);
 
-    if (!datetime.isValid())
-    {
-        return false;
-    }
-
-
-    return true;
+    return datetime.isValid();
 }
 
 
@@ -570,7 +560,6 @@ QDateTime IUnit::parseTimestamp(const QString &timetext, int& tzoffset)
     QDateTime datetime = QDateTime::fromString(timetext, format);
     datetime.setOffsetFromUtc(tzoffset);
 
-
     return datetime;
 }
 
@@ -617,7 +606,7 @@ QByteArray IUnit::pos2timezone(const QPointF& pos)
         return "UTC";
     }
 
-    int tz   = ((qRed(rgb) & 248) << 1) + ((qGreen(rgb) >> 4) & 15);
+    int tz = ((qRed(rgb) & 248) << 1) + ((qGreen(rgb) >> 4) & 15);
     if(tz >= N_TIMEZONES)
     {
         return 0;
@@ -638,21 +627,16 @@ void IUnit::degToStr(const qreal& x, const qreal& y, QString& str)
         bool signLat = GPS_Math_Deg_To_DegMin(y, &degN, &minN);
         bool signLon = GPS_Math_Deg_To_DegMin(x, &degE, &minE);
 
-        QString lat,lng;
-        lat = signLat ? "S" : "N";
-        lng = signLon ? "W" : "E";
+        const QString &lat = signLat ? "S" : "N";
+        const QString &lng = signLon ? "W" : "E";
         str.sprintf("%s%02d° %06.3f %s%03d° %06.3f",lat.toUtf8().data(),qAbs(degN),minN,lng.toUtf8().data(),qAbs(degE),minE);
         break;
     }
 
     case eCoordFormat2:
     {
-        bool signLat = y < 0;
-        bool signLon = x < 0;
-
-        QString lat,lng;
-        lat = signLat ? "S" : "N";
-        lng = signLon ? "W" : "E";
+        const QString &lat = (y < 0) ? "S" : "N";
+        const QString &lng = (x < 0) ? "W" : "E";
         str.sprintf("%s%02.6f° %s%03.6f°",lat.toUtf8().data(),qAbs(y),lng.toUtf8().data(),qAbs(x));
         break;
     }
@@ -668,9 +652,8 @@ void IUnit::degToStr(const qreal& x, const qreal& y, QString& str)
         qreal secN = (minN - qFloor(minN)) * 60;
         qreal secE = (minE - qFloor(minE)) * 60;
 
-        QString lat,lng;
-        lat = signLat ? "S" : "N";
-        lng = signLon ? "W" : "E";
+        const QString &lat = signLat ? "S" : "N";
+        const QString &lng = signLon ? "W" : "E";
         str.sprintf("%s%02d° %02d' %02.2f'' %s%03d° %02d' %02.2f''",lat.toUtf8().data(),qAbs(degN),qFloor(minN),secN,lng.toUtf8().data(),qAbs(degE),qFloor(minE),secE);
         break;
     }
@@ -742,13 +725,13 @@ bool IUnit::strToDeg(const QString& str, qreal& lon, qreal& lat)
     }
     else
     {
-        QMessageBox::warning(CMainWindow::getBestWidgetForParent(),QObject::tr("Error"),QObject::tr("Bad position format. Must be: \"[N|S] ddd mm.sss [W|E] ddd mm.sss\" or \"[N|S] ddd.ddd [W|E] ddd.ddd\""),QMessageBox::Ok,QMessageBox::NoButton);
+        QMessageBox::warning(CMainWindow::getBestWidgetForParent(),tr("Error"),tr("Bad position format. Must be: \"[N|S] ddd mm.sss [W|E] ddd mm.sss\" or \"[N|S] ddd.ddd [W|E] ddd.ddd\""),QMessageBox::Ok,QMessageBox::NoButton);
         return false;
     }
 
     if(fabs(lon) > 180.0 || fabs(lat) > 90.0)
     {
-        QMessageBox::warning(CMainWindow::getBestWidgetForParent(),QObject::tr("Error"),QObject::tr("Position values out of bounds. "),QMessageBox::Ok,QMessageBox::NoButton);
+        QMessageBox::warning(CMainWindow::getBestWidgetForParent(),tr("Error"),tr("Position values out of bounds. "),QMessageBox::Ok,QMessageBox::NoButton);
         return false;
     }
 
@@ -777,8 +760,5 @@ bool IUnit::isValidCoordString(const QString& str)
     {
         return true;
     }
-    else
-    {
-        return false;
-    }
+    return false;
 }
